# This is an apache perl module. Do not use directly.
# Copyright Cobalt Networks 1999.
#
# Author: Harris Vaegan-Lloyd <harris@cobaltnet.com>
#
# Any similarities between mod_perl and black magic are purely coincidental.
# No actual black magic practioners or covens are represented in this code.
#
package Cobalt::HtmlLocale;

use strict;
use Apache::File ();
use Apache::Constants qw(:common);
use Cobalt::Locale;
use Locale::gettext;
use CGI;

sub handler {
	my $r = shift;
	my $q = new CGI;
	return DECLINED unless $r->content_type() eq 'text/html';
	# Well, we were probably called as an error page,
	# let's avoid returning an error again so that we don't get into
	# a big loop over it.
	if ( $r->prev && $r->prev->filename =~ m#^/usr/admserv/html/.cobalt/error/#o ) {
		return OK;
	}

	my $locale=Cobalt::Locale->getByHTTPHeader( $ENV{ 'HTTP_ACCEPT_LANGUAGE' } );
	if( $locale ) { $ENV{ 'LANGUAGE' }=$locale->getLocale(); }


	my $file = $r->filename;

	unless ( -e $file ) {
		$r->log_error("File does not exist: $file");
		return 404;
	}

	unless ( -r $file ) {
		$r->log_error("File is not readable: $file");
		return FORBIDDEN;
	}

	my $modtime = localtime( (stat $file) [9] );

	my $fh;
	unless ($fh = Apache::File->new($file)) {
		$r->log_error("Couldn't open $file for reading: $!");
		return FORBIDDEN;
	}

	if ( $q->param('status') ) {
		$r->status($q->param('status'));
	}

	$r->send_http_header;

	while (<$fh>) {
    	s/\[str.([^\]]+)\]/Locale::gettext::dgettext("ui", "$1")/geo;
		$r->print($_);
	}

	return ( $q->param('status') || 0 )
} 

1;
__END__
