#
# $Id: Email.pm,v 1.63 1999/10/14 22:21:31 duncan Exp $
#
# Written by
#   Will DeHaan    <will@cobaltnet.com>
#   Kevin Chiu     <kevin@cobaltnet.com>
# Rewritten by
#   Duncan Laurie  <duncan@cobaltnet.com>
#
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com/
#

package Cobalt::Email;

use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK);

require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(mail_add_relay
	     mail_get_relay
	     mail_del_relay
	     mail_set_privacy
	     mail_set_AutoAlias
	     mail_set_max_document
	     mail_get_max_document
	     mail_set_smart_relay
	     mail_get_smart_relay
	     mail_virtuser_add_alias
	     mail_virtuser_del_alias
	     mail_virtuser_add_route
	     mail_virtuser_del_route
	     mail_virtuser_set_byuser
	     mail_virtuser_get_byuser
	     mail_virtuser_del_byuser
	     mail_virtuser_list_alias
	     mail_virtuser_list_route
	     mail_virtuser_change_domain
	     mail_list_cw
	     mail_add_cw
	     mail_del_cw
	     mail_set_masq
	     mail_get_masq
	     mail_set_deny
	     mail_get_deny
	     mail_apop_adduser
	     mail_apop_deluser
	     mail_apop_isuser
	     mail_build_virtusertable
	     sendmail_set_delivery_mode
	     sendmail_set_queue_interval
	     sendmail_get_queue_interval
             mail_makemap
	     mail_sendmail_hup);

use Cobalt::Util;
use Cobalt::Product;
require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);

use integer;

1;

__DATA__

# rebuilds the hash database supplied
# returns 1 for success, null for error
sub mail_makemap
{
    my $map = shift;

    system("$MakeMap_cmd hash $map < $map >>/dev/null 2>&1")
	&& return;

    return(1);
}

sub mail_add_relay
# adds a relay to the access database, format:  "host/network/domain RELAY"
# Enables email relaying by host/network/domain only
# Arguments: array of network/domains for whom to add relaying permissions
# Return value: status string of the form "1288 Cheap irony not supported
#   in this release."
{
        my (@new) = @_;
        my ($relay,%exists);
	my ($notend) = 1;
	my $ret;

        foreach (@new) {
	    $exists{$_} = 1;
	}
	
	FILE_lock_rw($MailAccessDB,*IN,*OUT)
	    || return MSG_error("email_cant_lock_accessDB", $!);

        while (<IN>) {
	    if ($notend) {
		if (/Mail rejected due to possible SPAM/o) {
		    print OUT; # not for us...
		} elsif (/^(\S+)\s+RELAY/o){
		    # there's a relay line here...but it may
		    # be the same as one we're adding
		    print OUT "$1\tRELAY\n" unless ($exists{$1});
		} elsif (/^\# Put custom additions below/o) {
		    # Now add all our new relay hosts
		    foreach $relay (@new) {
			print OUT "$relay\tRELAY\n";
		    }
		    print OUT "# Put custom additions below (Do not change/remove this line).\n";
		    $notend = 0;
		} else {
		    print OUT;
		}
	    } else {
		print OUT;
	    }
        }

	FILE_unlock_rw($MailAccessDB,*IN,*OUT)
	    || return MSG_error("email_cant_write_accessDB", $!);
	    
	# Now that the flat files have been built, make the database file
	mail_makemap($MailAccessDB)
	    || return MSG_error("email_cant_buildhash", $!);

	return MSG_ok("email_ok_relay_on");
}

sub mail_get_relay
# Obtains the networks/domains allowed to use sendmail to relay mail
# Arguments: none
# Return value: array of networks/domains
{
    my (@relays);
    my ($notend) = 1;

    FILE_lock_r($MailAccessDB,*ACCESSDB)
	|| return MSG_error("email_cant_read_accessDB", $!);

    while (<ACCESSDB>) {
	chomp;
	if ($notend) {
	    if (/(\S+)\s+RELAY/o) {
		push (@relays, $1);
	    } elsif ( /^\# Put custom additions below/o ) {
		$notend = 0;
	    }
	}
    }
    FILE_unlock(*ACCESSDB)
	|| return MSG_error("email_cant_read_accessDB", $!);

    return(@relays);
}

sub mail_del_relay
# Removes networks/domains from sendmail's ip_allow and name_allow files
# Disables  email relaying by network/domain only
# Arguments: array of networks/domains
# Return value: string of the form MSG_error("3100")
{
        my (@del) = @_;
        my ($relay,%exists);
        my ($notend) = 1;
	my $ret;

        foreach (@del) {
	    $exists{$_} = 1;
	}
	
	FILE_lock_rw($MailAccessDB,*IN,*OUT)
	    || return MSG_error("email_cant_lock_accessDB", $!);

        while (<IN>) {
	    if ($notend) {
		if (/Mail rejected due to possible SPAM/o && $notend) {
		    print OUT;
		} elsif (/^(\S+)\s+RELAY/o) {
		    print OUT "$1\tRELAY\n" unless ($exists{$1});
		} elsif (/^\# Put custom additions below/o) {
		    print OUT "# Put custom additions below (Do not change/remove this line).\n";
		    $notend = 0;
		} else {
		    print OUT;
		}
	    } else {
		print OUT;
	    }
        }

	FILE_unlock_rw($MailAccessDB,*IN,*OUT)
	    || return MSG_error("email_cant_write_accessDB", $!);

	# Build the database version of the access database
	mail_makemap($MailAccessDB) 
	    || return MSG_error("email_cant_buildhash", $!);

	return MSG_ok("email_ok_relays_removed");
}

sub mail_set_privacy
# Enables/Disables both expn and vrfy smtp commands in /etc/sendmail.cf
# Arguments: 1 to enable, 0 to disable
# Return value: string like MSG_error("4000")
# Site effects: edits /etc/sendmail.cf
{
	my($privacies) = $_[0];

	FILE_lock_rw($Sendmail_conf,*IN,*OUT)
	    || return MSG_error("email_cant_sendmailcf_locked");
	while(<IN>) {
		if(/O PrivacyOptions=/o) {
			print OUT "O PrivacyOptions=authwarnings,noexpn,novrfy\n" if($privacies);
			print OUT "O PrivacyOptions=authwarnings\n" unless($privacies);
		} else {
		    print OUT;
		}
	}

	FILE_unlock_rw($Sendmail_conf,*IN,*OUT)
	    || return MSG_error("email_cant_write_sendmailcf");

	return MSG_ok("email_ok_privacy_set") if ($privacies);
	return MSG_ok("email_ok_privacy_off") unless ($privacies);
}

sub mail_set_AutoAlias
# Enables/Disables AutoRebuildAliases in /etc/sendmail.cf
# Arguments: 1 to enable, 0 to disable
# Return value: string like MSG_error("2000")
# Site effects: edits /etc/sendmail.cf
{
	my($auto_rebuild) = $_[0];

	FILE_lock_rw($Sendmail_conf,*IN,*OUT)
	    || return MSG_error("email_cant_sendmailcf_locked");

	while(<IN>) {
		if(/AutoRebuildAliases/) {
			print OUT "O AutoRebuildAliases\n" if($auto_rebuild);
			print OUT "#O AutoRebuildAliases\n" unless($auto_rebuild);
		} else {
		    print OUT;
		}
	}

	FILE_unlock_rw($Sendmail_conf,*IN,*OUT)
	    || return MSG_error("email_cant_write_sendmailcf");

	return MSG_ok("email_ok_autorebuild_set") if ($auto_rebuild);
	return MSG_ok("email_ok_autorebuild_off") unless ($auto_rebuild);
}

sub mail_set_max_document
# Sets the maximum document size in bytes that sendmail will accept
# Arguments: natural byte value, 0 for unlimited
# Return value: string like MSG_error("2000")
# Site effects: edits /etc/sendmail.cf
{
        # local variables should be lowercased for consistency (and for
        # the preprocessor, unfortunately...)
	my($max_doc_size) = $_[0];
	return MSG_error("email_invalid_max_doc_size") if ($max_doc_size < 0);
	return MSG_error("email_cant_sendmail_not_configured") unless (-e "$Sendmail_conf");

	FILE_lock_rw($Sendmail_conf,*IN,*OUT)
	    || return MSG_error("email_cant_sendmailcf_locked");

	while(<IN>) {
	    if(/MaxMessageSize/o) {
		print OUT "O MaxMessageSize=$max_doc_size\n" if($max_doc_size);
		print OUT "#O MaxMessageSize=1000000\n" if(!$max_doc_size);
	    } else {
		print OUT;
	    }
	}
	
	FILE_unlock_rw($Sendmail_conf,*IN,*OUT)
	    || return MSG_error("email_cant_write_sendmailcf");

	return MSG_ok("email_ok_max_doc_set");
}

sub mail_get_max_document
# Gets the maximum document size in bytes that sendmail will accept
# Arguments: none
# Return value: natural byte value, 0 for unlimited
{
        my $docsize = 0;

        FILE_lock_r($Sendmail_conf,*IN)
	    || return MSG_error("email_cant_sendmail_not_configured");
	while(<IN>) {
	    if (/^O MaxMessageSize=(\d+)/) {
		$docsize = $1;
	    }
	}

	FILE_unlock(*IN)
	    || return MSG_error("email_cant_sendmail_not_configured");

	return $docsize;
}

sub mail_set_smart_relay
# Sets the not-so-smart SMTP outgoing relay 
# Arguments: Hostname or null to disable
# Return value: string like MSG_error("2000")
# Site effects: edits /etc/sendmail.cf
{
        # local variables should be lowercased for consistency (and for
        # the preprocessor, unfortunately...)
	my($ds) = $_[0];
	return MSG_error("email_invalid_smart_relay") if ($ds && !Cobalt::Netutil::netutil_test_domainname($ds));
	return MSG_error("email_cant_sendmail_not_configured") unless (-e "$Sendmail_conf");

	FILE_lock_rw($Sendmail_conf,*IN,*OUT)
	    || return MSG_error("email_cant_sendmailcf_locked");

	while(<IN>) {
	    if(/^DS/o) {
		print OUT "DS$ds\n";
	    } else {
		print OUT;
	    }
	}
	
	FILE_unlock_rw($Sendmail_conf,*IN,*OUT)
	    || return MSG_error("email_cant_write_sendmailcf");

	return MSG_ok("email_ok_smart_relay_set");
}

sub mail_get_smart_relay
# Gets the not-so-smart relay host
# Arguments: none
# Return value: hostname
{
        my $ds;

        FILE_lock_r($Sendmail_conf,*IN)
	    || return MSG_error("email_cant_sendmail_not_configured");
	while(<IN>) {
	    if (/^DS(.+)$/) {
		$ds = $1;
		$ds =~ s/[\n\r]//g;
	    }
	}

	FILE_unlock(*IN)
	    || return MSG_error("email_cant_sendmail_not_configured");

	return $ds
}

sub mail_virtuser_add_alias
# Adds a hash of login name and fully qualified email addresses or @domains
# Arguments: hash  (alias => user)
# Return value: string like MSG_error("2000")
# Side effects: Rebuilds /etc/virtusertable.db from modified /etc/virtusertable
{
	my(%add_alias) = @_;
	my(%aliases) = Cobalt::Email::mail_virtuser_list_alias();
	my(%routes) = Cobalt::Email::mail_virtuser_list_route();
	my($alias);

	foreach $alias (keys %add_alias) {
	    if ($aliases{$alias}) {
		return MSG_error("email_cant_alias_used", $alias);
	    }
	    $aliases{$alias} = $add_alias{$alias}; # No duplicate aliases allowed
	}
	
	Cobalt::Email::mail_build_virtusertable(\%aliases, \%routes)
	    || return MSG_error("email_cant_modify_list");

	return MSG_ok("email_ok_virtusers_added");
}

sub mail_virtuser_del_alias
# Deletes a hash of login name and fully qualified email addresses or @domains
# Arguments: hash  (alias => user)
# Return value: string like MSG_error("2000")
# Side effects: Rebuilds /etc/virtusertable.db from modified /etc/virtusertable
{
	my(%delAlias) = @_;
	my(%aliases) = Cobalt::Email::mail_virtuser_list_alias();
	my(%routes) = Cobalt::Email::mail_virtuser_list_route();
	my($alias);

	foreach $alias (keys %delAlias) {
	    delete $aliases{$alias};
	}

	Cobalt::Email::mail_build_virtusertable(\%aliases, \%routes)
	    || return MSG_error("email_cant_modify_list");

	return MSG_ok("email_ok_virtusers_removed");
}

sub mail_virtuser_add_route
# Adds a hash of "domain" and "fqdn to route to"
# Arguments: hash  (domain => fqdn to route to)
# Return value: string like MSG_error("2000")
# Side effects: Rebuilds /etc/virtusertable.db from modified /etc/virtusertable
{
	my(%add_route) = @_;
	my($r);
	my(%aliases) = Cobalt::Email::mail_virtuser_list_alias();
	my(%routes) = Cobalt::Email::mail_virtuser_list_route();

	foreach $r (keys %add_route) {
	    if ($routes{$r}) {
		return MSG_error("email_cant_route_used", $r);
	    }
	    $routes{$r} = $add_route{$r}; # No duplicate aliases allowed
	}
	Cobalt::Email::mail_build_virtusertable(\%aliases, \%routes)
	    || return MSG_error("email_cant_modify_list");

	return MSG_ok("email_ok_virtusers_added");
}

sub mail_virtuser_del_route
# Deletes a hash of "domain" and "fqdn to route to"
# Arguments: hash  (domain => fqdn to route to)
# Return value: string like MSG_error("2000")
# Side effects: Rebuilds /etc/virtusertable.db from modified /etc/virtusertable
{
	my(%del_route) = @_;
	my(%aliases) = Cobalt::Email::mail_virtuser_list_alias();
	my(%routes) = Cobalt::Email::mail_virtuser_list_route();
	my($r);

	foreach $r (keys %del_route) {
	    delete $routes{$r};
	}
	Cobalt::Email::mail_build_virtusertable(\%aliases, \%routes)
	    || return MSG_error("email_cant_modify_list");

	return MSG_ok("email_ok_virtusers_removed");
}

sub mail_virtuser_set_byuser
# Adds/Removes aliases as necessary
# Arguments: username, array of aliases
# Return value: string of the form MSG_error("4200")
{
	my($username,@alii) = @_;
	my($alias,%newalii,%oldalii,$ret);
	my(@oldalias) = Cobalt::Email::mail_virtuser_get_byuser($username);

	foreach $alias (@alii) {
	    $newalii{$alias} = $username;
	}
	foreach $alias (@oldalias) {
	    $oldalii{$alias} = $username;
	}

	$ret = Cobalt::Email::mail_virtuser_del_alias(%oldalii);
	return $ret if ($ret !~ /^2/o);

	$ret = Cobalt::Email::mail_virtuser_add_alias(%newalii);
	if ($ret !~ /^2/o) {
	    # try to restore to the initial value
	    # this is NOT guaranteed to work, though
	    Cobalt::Email::mail_virtuser_add_alias(%oldalii);
	    return $ret;
	}

	return MSG_ok("email_ok_virtalias_updated");
}

sub mail_virtuser_get_byuser
# Parses a list of aliases a user owns
# Arguments: username
# Return value: Array of aliases
{
	my($username) = @_;
	my(%virtuser) = Cobalt::Email::mail_virtuser_list_alias();
	my($alias,$user,@alii);
	foreach $alias (keys %virtuser) {
	    push(@alii,$alias) if ($virtuser{$alias} eq $username);
	}
	return(@alii);
}

sub mail_virtuser_del_byuser
# Sames as VirtuserSetByUser with no aliases specified
#   This will iterate VirtuserSetByUser calls when there are multiple arguments
# Arguments: a username or an array of usernames
# Return value: String like usual
{
	my(@username) = @_;
	my($user,$ret);
	foreach $user (@username) {
	    $ret = Cobalt::Email::mail_virtuser_set_byuser($user);
	    return $ret if ($ret !~ /^2/);
	}
	return MSG_ok("email_ok_virtaliase_removed");
}

sub mail_virtuser_list_alias
# Returns a hash of fully qualified email addresses or @domains and 
# target users or error messages
# Arguments: none
# Return value: an alias hash (alias => user)
# Example: my(%aliases) = Cobalt::Email::mail_virtuser_list_alias();
{
	my(%aliases); 

	FILE_lock_r($EmailVirtUser,*VRT)
	    || return MSG_error("email_cant_list_locked");
	while(<VRT>) {
            last if (/custom additions below/o);
	    chop;
	    next if (/^\s*\#/o);
	    next if (/^\s*\@\S+\s+\%1\@\S+/o); # skip accept-email-at-domain routes
	    $aliases{$1} = $2 if (/^\s*(\S+)\s*(.*)$/o); 
	}
	FILE_unlock(*VRT)
	    || return MSG_error("email_cant_list_locked");

	return(%aliases);
}

sub mail_virtuser_list_route
# Returns a hash of fully qualified "domain" and target "fqdn" routes, or error messages
# Arguments: none
# Return value: a routes hash (domain => fqdn to route to)
# Example: my(%routes) = Cobalt::Email::mail_virtuser_list_route();
{
	my(%routes); 

	FILE_lock_r($EmailVirtUser,*VRT)
	    || return MSG_error("email_cant_list_locked");
	while(<VRT>) {
            last if( /custom additions below/o );
	    chop;
	    next if (/^\s*\#/o);
	    next unless (/^\@/o);
	    $routes{$1} = $2 if (/^\s*\@(\S+)\s+\%1\@(.*)$/o);
	}
	FILE_unlock(*VRT)
	    || return MSG_error("email_cant_list_locked");

	return(%routes);
}

sub mail_virtuser_change_domain
# Changes email alias domains for when any site host or domain name changes
# Arguments: old qualified hostname, new qualified host name
#              or, hash of (old qualified hostname -> new qualified host name)
# Return value: string of the regular sort
# Rebuilds /etc/virtusertable
{
        my(%aliasdelta) = @_;
        my($alias,$route,@splut,$foundModification);
        my(%Alii) = Cobalt::Email::mail_virtuser_list_alias();
	my(%routes) = Cobalt::Email::mail_virtuser_list_route();

        foreach $alias (keys %Alii) {
	    next unless ($alias =~ /\@/o); # Not willing to mess with custom entries
	    @splut = split('@', $alias); # (alias name,alias domain)
	    if ($aliasdelta{$splut[1]}) {
		$splut[1] = $aliasdelta{$splut[1]};
		$Alii{join('@', @splut)} = $Alii{$alias};
		delete $Alii{$alias}; # Delete old record
		$foundModification = 1;
	    }
        }

	# Rebuild aliases
        if ($foundModification) {
	    Cobalt::Email::mail_build_virtusertable(\%Alii, \%routes)
		|| return MSG_error("email_cant_modify_list");
	}

        return MSG_ok("email_ok_virtusers_modified");
}

sub mail_list_cw
# Reports a sorted list of hostnames that sendmail is willing to accept mail to
# Arguments: none
# Return value: sorted array of domain/host names
# Example: my(@sendmail_server_aliases) = sendmail_cw_list;
{
    my @aliases;

    FILE_lock_r($Sendmail_cw,*IN)
	|| return MSG_error("email_cant_open_sendmailcw", $!);
    while (<IN>) {
	last if (/include all aliases for your machine here/o);
	next if (/^\s*\#/o);
	next if (/^$/o);
	chomp;
	push( @aliases, $_ );
    }
    FILE_unlock(*IN)
	|| return MSG_error("email_cant_open_sendmailcw", $!);

    return sort(@aliases);
}

sub mail_add_cw 
# Adds alias(s) to sendmail.cw, unless it's already defined
# Arguments: an array of host/domain names
# Return value: a string like MSG_error("2000")
# Example: &good_stuff if (Sendmail_cw_add('mail.com','me.mail.com'));
{
    my @additions=@_;
    my @aliases=Cobalt::Email::mail_list_cw();
    my ($alias,$name);
    my %aliasesHash;
    my $userArea = 0;

    foreach $alias (@aliases) {
	$aliasesHash{$alias}=1;
    }
    foreach $alias (@additions) {
	if(! Cobalt::Netutil::netutil_test_domainname($alias)) {
	    $name=$alias;
	    return MSG_error("email_invalid_hostname", $name);
	}
	$aliasesHash{$alias} = 1;
    }
    
    FILE_lock_rw($Sendmail_cw,*IN,*OUT)
	|| return MSG_error("email_cant_list_locked");

    foreach $alias (keys %aliasesHash) {
	print OUT "$alias\n";
    }
    while (<IN>) {
	$userArea = 1 if (/include all aliases for your machine here/);
	print OUT if ($userArea);
    }

    FILE_unlock_rw($Sendmail_cw,*IN,*OUT)
	|| return MSG_error("email_cant_list_locked");

    return MSG_ok("email_ok_server_added");
}

sub mail_del_cw 
# Removes alias(s) from sendmail.cw
# Arguments: an array of host/domain names
# Return value: a string like MSG_error("4000")
# Example: &good_stuff if (Sendmail_cw_del('nomore.mail.com','less.mail.isp'));
{
    my @subtractions=@_;
    my %aliasesHash;
    my ($alias,$name);
    my @aliases=Cobalt::Email::mail_list_cw();
    my $userArea = 0;

    foreach $alias (@aliases) {
	$aliasesHash{$alias} = 1;
    }
    foreach $alias (@subtractions) {
	delete $aliasesHash{$alias};
    }
    foreach $alias (keys %aliasesHash) {
	if (! Cobalt::Netutil::netutil_test_domainname($alias)) {
	    $name=$alias;
	    return MSG_error("email_invalid_hostname", $name);
	}
    }

    FILE_lock_rw($Sendmail_cw,*IN,*OUT)
	|| return MSG_error("email_cant_list_locked");

    foreach $alias (keys %aliasesHash) {
	print OUT "$alias\n";
    }
    while (<IN>) {
	$userArea = 1 if (/include all aliases for your machine here/);
	print OUT if ($userArea);
    }
    FILE_unlock_rw($Sendmail_cw,*IN,*OUT)
	|| return MSG_error("email_cant_list_locked");

    return MSG_ok("email_ok_server_removed");	
}

sub mail_set_masq
# Set the domain that we want to masquerade as
# Arguments: domain name, or blank for no masquerading
# Side effects: edits sendmail.cf, restarts sendmail
# Return value: status string of the form MSG_error("2743")
# Example: Cobalt::Email::mail_set_masq("cobaltnet.com");
{
    my ($masqas) = @_;
    $masqas ||= '';

    FILE_lock_rw($Sendmail_conf,*IN,*OUT)
	|| return MSG_error("email_cant_sendmail_locked");
    while (<IN>) {
	if (/^DM/o) {
	    print OUT "DM$masqas\n";
	} else {
	    print OUT;
	}
    }
    FILE_unlock_rw($Sendmail_conf,*IN,*OUT)
	|| return MSG_error("email_cant_sendmail_locked");

    return MSG_ok("email_ok_set_masq", $masqas);
}

sub mail_get_masq
# Get the name of the domain we masquerade as
# Arguments: none
# Return value: a domain name, or blank for no masquerading
# Side effects: none
# Example: my $domain = get_mail_masq();
{
    my $masqas = '';

    FILE_lock_r($Sendmail_conf,*IN)
	|| return MSG_error("email_cant_read_sendmailcf", $!);

    while (<IN>) {
	if (/^DM(.*)/o) {
	    $masqas = $1;
	    last;
	}
    }
    FILE_unlock(*IN);

    return $masqas;
}

sub mail_set_deny
# Set the list of domains we refuse to accept mail from
# Arguments: list of domain or host names
# Side effects: modifies /etc/mail/deny and rebuilds deny.db
# Example: Cobalt::Email::mail_set_deny("cybersell.com","owlsnest.com");
{
        my (@denys) = @_;
	my ($notend) = 1;
	my ($i);

	FILE_lock_rw($MailAccessDB,*IN,*OUT)
	    || return MSG_error("email_cant_lock_accessDB", $!);

	while (<IN>) {
	    if ($notend) {
		if (/Mail rejected due to possible SPAM/o) {
		    next;
		} elsif (/^\# Put custom additions below/o) {
		    foreach $i (@denys) {
			print OUT "$i\t550 Mail rejected due to possible SPAM\n";
		    }
		    print OUT "# Put custom additions below (Do not change/remove this line).\n";
		    $notend = 0;
		} else {
		    print OUT;
		}
	    } else {
		print OUT;
	    }
	}

	FILE_unlock_rw($MailAccessDB,*IN,*OUT);

	# rebuild the hash database
	mail_makemap($MailAccessDB)
	    || return MSG_error("email_cant_buildhash", $!);

	return MSG_ok("email_ok_set_blocking");
}

sub mail_get_deny
# Get the list of site from which we refuse mail
# Arguments: none
# Return value: a list of sites we block
# Example: @spammers = Cobalt::Email::mail_get_deny();
{
        my (@denys);
	
	FILE_lock_r($MailAccessDB,*IN)
	    || return MSG_error("email_cant_read_accessDB", $!);
	while (<IN>) {
	    if (/(\S+)\s+550 Mail rejected/o) {
		push(@denys,$1);
	    } elsif (/^\# Put custom additions below/o) {
		last;
	    }
	}
	FILE_unlock(*IN);
        return(@denys);
}

sub mail_build_virtusertable
# Build the virtusertable database based on the entries in the
# virtusertable file 
# Arguments: 
#       1. a set of aliases to put in the table, in the form
#          of a reference to a hash table, $aliasname => aliasto
#       2. a set of domains and their fqdn routes to put in the
#          table, in the form of a reference to a hash table, $domain => fqdnroute
# Rewrites the virtusertable with whatever aliases it is given as an
#   argument, plus whatever it finds below the "custom additions below"
#   lines at the bottom.
# Return value: 0 on failure, 1 on success
# Side effects: little green men on acid
{
	my($aliases,$routes) = @_;
	my($table_data,$key,$val,$end);

                                                # do the aliases
	while (($key,$val) = each %$aliases) {
	    $table_data .= "$key\t$val\n";
	}
	$table_data .= "\n# accept-email-at-domain routes\n";
	                                        # do the routes
	while (($key,$val) = each %$routes) {
	    $table_data .= "\@".$key."\t\%1\@".$val."\n";
	}

	FILE_lock_rw($EmailVirtUser,*DBREAD,*DBWRITE)
	    || return MSG_error("email_cant_list_locked");

	print DBWRITE "# This file is automatically generated\n";
	print DBWRITE "# Please put custom changes at the end\n";
	print DBWRITE "# Cobalt Networks 1998\n\n";
	print DBWRITE "$table_data\n";

	                                        # handle custom lines
        while (<DBREAD>) {
	    unless($end) {
		if (/^\# Put custom additions below/o) {
		    print DBWRITE;
		    $end = 1;
		}
	    } else {
		print DBWRITE;
	    }
	}

	FILE_unlock_rw($EmailVirtUser,*DBREAD,*DBWRITE);

	                                        # build the database, save the text
	mail_makemap($EmailVirtUser)
	    || return MSG_error("email_cant_buildhash", $!);

	                                        # if we've come this far...
        return MSG_ok("email_ok_virtusers_modified");
}


sub mail_sendmail_hup
# Sends sendmail a HUP so it rereads its configuration
# Arguments: none
# Side effects: kill -HUP's sendmail
# Return value: none
{
    my $pid;

    FILE_lock_r($Sendmail_pidfile, *PID) || return;
    $pid=<PID>;
    FILE_unlock(*PID) || return;

    chomp $pid;
    kill 'HUP', $pid if ($pid);
    return;
}

sub mail_apop_adduser
# adds a user to the APOP database
#   (can also be used to change password)
# arguments: user, password
# side effects touches APOP database
# return value: standard
{
    my ($user,$pw) = @_;
    my (%db,$obscurepw,@ppw);

    @ppw = unpack('C*', $pw);
    foreach (@ppw) {
	$obscurepw .= pack('C*', $_ ^= hex("ff"));
    }

    use GDBM_File;
    tie (%db, "GDBM_File", "/etc/pop.auth", GDBM_WRITER, 0600)
	or return(MSG_error("email_apop_cant_opendb"));

    $db{$user} = $obscurepw;
    untie %db;

    return(MSG_ok("email_apop_useradd_ok"));
}

sub mail_apop_deluser
# deletes users from the APOP database
# arguments: list of users
# side effects touches APOP database
# return value: standard
{
    my (@users) = @_;
    my (%db);

    use GDBM_File;
    tie (%db, "GDBM_File", "/etc/pop.auth", GDBM_WRITER, 0600)
	or return(MSG_error("email_apop_cant_opendb"));

    foreach (@users) {
	delete($db{$_});
    }
    untie %db;

    return(MSG_ok("email_apop_userdel_ok"));
}

sub mail_apop_isuser
# verifies if a user is in the APOP database
# arguments: optional user
# return value: 0/1 if user specified, otherwise return list of users in apop database
{
    my $user = shift;
    my @users;
    my (%db);

    use GDBM_File;
    tie (%db, "GDBM_File", "/etc/pop.auth", GDBM_READER, 0600)
	or return(MSG_error("email_apop_cant_opendb"));

    if ($user) {
	if (exists($db{$user})) {
	    untie %db;
	    return 1;
	} else {
	    untie %db;
	    return 0;
	}
    } else {
	@users = keys %db;
	untie %db;
	return (@users);
    }

}

sub sendmail_set_delivery_mode
# sets the delivery mode of sendmail
#
# Arguments: $mode = (i)nteractive, (b)background, (q)ueue 
{
    my($mode) = shift;
  
    my %delivery_modes = ( 'i' => 'interactive',
			   'interactive' => 'interactive',
			   'b' => 'background',
			   'background' => 'background',
			   'q' => 'queue',
			   'queue' => 'queue' );
  
    return MSG_error("email_cant_invalid_mode", $mode) if (! $delivery_modes{$mode});
  
    FILE_lock_rw($Sendmail_conf,*IN,*OUT)
	|| return MSG_error("email_cant_sendmailcf_locked");
    while(<IN>) {
	if(/O DeliveryMode=/o) {
	    print OUT "O DeliveryMode=$delivery_modes{$mode}\n";
	} else {
	    print OUT;
	}
    }

    FILE_unlock_rw($Sendmail_conf,*IN,*OUT);
    return MSG_ok("email_ok_deliverymode_set");
}

sub sendmail_set_queue_interval
# sets the sendmail queue flushing interval by setting a symbolic link
# from the correct /etc/cron.* directory to the flush_sendmail_queue script.
# we flush the sendmail queue this way rather than letting sendmail do it
# to synchronize the flushing with the fetchmail retrieval
# args: $interval = fetchmail interval
{
    my ($interval) = shift;
    my $crondir;
    my $found = 0;
    
    # if $interval is 'never' set $found = 1
    # this causes any existing interval to be removed
    $found = 1 if ($interval eq 'never');
    
    foreach $crondir (@Cron_Intervals) {
	# sanity check to make sure not more than one symlink exists
	if (-e "$Cron_basedir$crondir/flush_sendmail_queue") {
	    unlink("$Cron_basedir$crondir/flush_sendmail_queue");
	}
	if ($interval eq $crondir) {
	    symlink($Sendmail_flush_script, "$Cron_basedir$crondir/flush_sendmail_queue");
	    $found = 1;
	}
    }
    
    if ($found) {
	return MSG_ok("email_ok_set_interval");
    } else {
	return MSG_error("email_cant_unsupported_interval");
    }
}

sub sendmail_get_queue_interval
# retrieves the fetchmail frequency by checking for a symbolic link in each
# of the /etc/cron.* directories.
# args: none
# returns the string name of the cron interval, or 'never' if not yet set
{
    my $found = 'never';
    my $crondir;

    foreach $crondir (@Cron_Intervals) {
	return $crondir if (-e "$Cron_basedir$crondir/flush_sendmail_queue");
    }

    return $found;
}

1;
