#
# Written by John D. Blair <jdblair@cobaltnet.com>
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Fetchmail;

use vars qw(@ISA @EXPORT @EXPORT_OK);

require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(fm_set_user_rc
	     fm_get_user_rc
	     fm_check_user_rc
	     fm_in_crontab
	     fm_set_domain_rc
	     fm_get_domain_rc
	     fm_check_domain_rc
	     fm_set_interval
	     fm_get_interval
	     fm_do_fetch);
@EXPORT_OK = qw(_write_user_rc
		_write_domain_rc);
#require Cobalt::Product;
require Cobalt::Language;
use Cobalt::Util;
use Cobalt::Product;

use strict;

1;
__DATA__

sub fm_set_user_rc 
# writes the fetchmailrc file
# just a wrapper around Cobalt::Fetchmail::_write_user_rc(see docs for _write_user_rc below)
{
  my ($server, $username, $localname, $passwd) = @_;
  
  return Cobalt::Fetchmail::_write_user_rc($server, $username, $localname, $passwd);
}

sub _write_user_rc
# writes the fetchmailrc file to a specified location
# Arguments:
#    $filename:  filename
#    $server:    remote e-mail server hostname
#    $username:  username on remote e-mail server
#    $localname: username on the qube
#    $passwd:    password to log into the remote server
{
  my ($filename, $server, $username, $localname, $passwd) = @_;

  # set umask so file is created with only user read/write
  umask(0177);

  # get uid, gid
  my ($login,$pass,$uid,$gid) = getpwnam($localname)
    or return "fetchmail_invalid_nosuchuser User does not exist on this system";

  # there might be a double-quote in the password
  $passwd =~ s/\"/\\\"/g;

  open(FMRC, ">$filename")
    or return MSG_error("fetchmail_cant_write_rc", $!);
  print FMRC "# last modifed " . localtime() . "\n";
  print FMRC "poll \"$server\"\n";
  print FMRC "user \"$username\"\n";
  print FMRC "is \"$localname\"\n";
  print FMRC "password \"$passwd\"\n";
  close(FMRC);
}

sub fm_set_domain_rc
# sets a multidrop mail file
# just a wrapper around Cobalt::Fetchmail::_write_domain_rc(see docs 
# for _write_domain_rc below)
{
  my ($server, $username, $localdomain, $passwd, $etrn) = @_;

  # create the /var/fetchmail dir if it doesn't exist
  if (! -e "$Fetchmailrc_dir") {
    mkdir("$Fetchmailrc_dir", 0755);
  }

  if ($passwd eq '') {
    # no password is passed. reuse the current password
    $passwd = (Cobalt::Fetchmail::fm_get_domain_rc())[2];
  }
  
  return Cobalt::Fetchmail::_write_domain_rc("$Fetchmailrc_dir/multidrop:localdomain", $server, $username, $passwd, $etrn);
}

# writes the fetchmailrc file for multidrop to a specified filename
# Arguments:
#    $filename:  filename
#    $server:    remote e-mail server hostname
#    $username:  username on remote e-mail server
#    $passwd:    password to log into the remote server
#    $etrn:      use sendmail etrn mode instead of POP3 or IMAP4
sub _write_domain_rc
{
  my ($filename, $server, $username, $passwd, $etrn) = @_;

  # set umask so file is created with only user read/write
  umask(0177);

  # there might be a " in the password
  $passwd =~ s/\"/\\\"/g;

  open(FMRC, ">$filename")
    or return MSG_error("fetchmail_cant_write_rc", $!);
  print FMRC "# last modifed " . localtime() . "\n";
  print FMRC "poll \"$server\"\n";
  print FMRC "localdomains " . (Cobalt::Network::net_get_dnsclient())[0] . "\n";
  if ($etrn) { print FMRC "proto ETRN\n"; }
  print FMRC "username \"$username\"\n";
  print FMRC "password \"$passwd\"\n";
  print FMRC "to *\n";
  close(FMRC);
}


sub fm_get_user_rc 
# returns the contents of the user's fetchmailrc file
# arguments:
#    $user: local user name
{
  my($user) = @_;
  my ($server, $username, $localname, $passwd, $poll);

  # create the /var/fetchmail dir if it doesn't exist
  if (! -e "$Fetchmailrc_dir") {
    mkdir("$Fetchmailrc_dir", 0755);
  }  

  open(FMRC, "$Fetchmailrc_dir/$user")
    || return MSG_error("fetchmail_cant_read_rc", $!);
  foreach (<FMRC>) {
    if (/^poll \"([^\"]+)\"/o) {
      $server = $1;
    }
    if (/^user \"([^\"]+)\"/o) {
      $username = $1;
    }
    if (/^is \"([^\"]+)\"/o) {
      $localname = $1;
    }
    if (/^password \"([^\"]+)\"/o) {
      $passwd = $1;
    }
  }
  close(FMRC);

  return ($server, $username, $localname, $passwd);
}

sub fm_get_domain_rc 
# returns the contents of the domain's multidrop fetchmailrc file
# arguments:
#    none
{
  my ($server, $username, $passwd, $poll, $etrn);

  $etrn = 0;

  if (-e "$Fetchmailrc_dir/multidrop:localdomain") {
    open(FMRC, "$Fetchmailrc_dir/multidrop:localdomain")
      || return MSG_error("fetchmail_cant_read_rc", $!);
    foreach (<FMRC>) {
      if (/^poll \"([^\"]+)\"/o) {
	$server = $1;
      }
      if (/^username \"([^\"]+)\"/o) {
	$username = $1;
      }
      if (/^password \"([^\"]+)\"/o) {
	$passwd = $1;
      }
      if (/^proto ETRN/) {
	$etrn = 1;
      }
    }
    close(FMRC);
  } else {
    $server = '';
    $username = '';
    $passwd = '';
  }

  return ($server, $username, $passwd, $etrn);
}

sub fm_set_interval
# sets the fetchmail frequency by setting a symbolic link from the correct
# /etc/cron.* directory to the fetch_remote_mail script.
# args: $interval = fetchmail interval
{
  my ($interval) = shift;
  my $crondir;

  my $found = 0;
  
  # if $interval is 'never' set $found = 1
  # this causes any existing interval to be removed
  $found = 1 if ($interval eq 'never');

  foreach $crondir (@Cron_Intervals) {
    # sanity check to make sure not more than one symlink exists
    if (-e "$Cron_basedir$crondir/fetch_remote_mail") {
      unlink("$Cron_basedir$crondir/fetch_remote_mail");
    }
    if ($interval eq $crondir) {
      symlink($Fetchmail_script, "$Cron_basedir$crondir/fetch_remote_mail");
      $found = 1;
    }
  }

  if (! $found) {
    return "4666: unsupported fetchmail interval";
  }
}

sub fm_get_interval
# retrieves the fetchmail frequency by checking for a symbolic link in each
# of the /etc/cron.* directories.
# args: none
# returns the string name of the cron interval, or 'never' if not yet set
{
  my $found = 'never';
  my $crondir;

  foreach $crondir (@Cron_Intervals) {
    if (-e "$Cron_basedir$crondir/fetch_remote_mail") {
      return $crondir;
    }
  }

  return $found;
}

sub fm_check_domain_rc
# attempts to use the specified domain rc config
# returns a string on error, nothing on success
# args: none
{
  my ($server, $username, $passwd, $etrn) = @_;
  my ($line);

  my $checkFile="/tmp/fm_check_domain_rc.$$";
  Cobalt::Fetchmail::_write_domain_rc($checkFile, $server, $username, $passwd, $etrn);

  my $ret=system( "$Fetchmail -s -c -f $checkFile 2>&1" );
  # to get the real return code
  $ret=int( $ret/256 );

  unlink($checkFile);

  if( $ret==0 || $ret==1 ) { return; }
  if( $ret==2 ) { return MSG_error("fetchmail_cant_pop"); }
  elsif( $ret==3 || $ret==9 ) { return MSG_error("fetchmail_cant_auth"); }
  elsif( $ret==4 || $ret==7 ) { return MSG_error("fetchmail_cant_connect"); }
  elsif( $ret==10 ) { return MSG_error("fetchmail_cant_smtp"); }
  elsif( $ret==11 ) { return MSG_error("fetchmail_cant_dns"); }
  else { return MSG_error("fetchmail_err_generic"); }
}

sub fm_check_user_rc
# attempts to use the specified user rc config
# returns a string on error, nothing on success
# args: none
{
  my ($server, $username, $localname, $passwd, $etrn) = @_;
  my ($line);

  Cobalt::Fetchmail::_write_user_rc("/tmp/fm_check_user_rc.$$", $server, $username, $localname, $passwd, $etrn);

  open(FM, "$Fetchmail -s -c -f /tmp/fm_check_domain_rc.$$ 2>&1|");
  my @msgs = (<FM>);
  close(FM);

  unlink("/tmp/fm_check_user_rc.$$");

  # remove 'fetchmail:' from each line of error message
  foreach $line (@msgs) {
    $line =~ s/^fetchmail: //o;
  }
  return join('<br>', @msgs);
}

sub fm_do_fetch
# retrieves remote mail based on fetchmail rc files in /var/fetchmail
# args: none
{
  my $file;
  my $ret;
  my $msg;

  print STDERR "fetchmail: called fm_do_fetch\n" if ($Debug);

  if (-e $Fetchmail_lock) {
    # should we check for a really old lock here to make sure a stale lock
    # isn't left around, preventing mail access?
    print STDERR "fetchmail: fetchmail is locked\n" if ($Debug);
    return MSG_error("fetchmail_cant_rc_locked");
  }

  # set lockfile (this isn't race-safe, but it should be good enough)
  open(LOCK, ">$Fetchmail_lock");
  close(LOCK);
  
  # grab text messages
  my $fetchmailErrorSubject = MSG_error("fetchmailErrorSubject");
  my $fetchmailErrorMsg = MSG_error("fetchmailErrorMsg");

  # step through each fetchmail resource file
  opendir(RCDIR, $Fetchmailrc_dir);
  while ($file = readdir(RCDIR)) {
    if ($file !~ /^\./) {

      # check if proto is ETRN
      if ((Cobalt::Fetchmail::fm_get_domain_rc())[3]) {
	print STDERR "fetchmail: calling $Fetchmail -s -f $Fetchmailrc_dir/$file" if ($Debug);
	$ret = system("$Fetchmail -s -f $Fetchmailrc_dir/$file >>$Sauce_log 2>&1");
      } else {
	# use the 'a' flag to make sure all mail is retrieved
	# with 'a' read messages will be left on the server
	print STDERR "fetchmail: calling $Fetchmail -as -f $Fetchmailrc_dir/$file" if ($Debug);
	$ret = system("$Fetchmail -as -f $Fetchmailrc_dir/$file >>$Sauce_log 2>&1");
      }

      # convert to the "real" return code
      $ret=int( $ret/256 );

      if( $ret==2 ) { $msg = MSG_error("fetchmail_cant_pop"); }
      elsif( $ret==3 || $ret==9 ) { $msg = MSG_error("fetchmail_cant_auth"); }
      elsif( $ret==4 || $ret==7 ) { $msg = MSG_error("fetchmail_cant_connect"); }
      elsif( $ret==10 ) { $msg = MSG_error("fetchmail_cant_smtp"); }
      elsif( $ret==11 ) { $msg = MSG_error("fetchmail_cant_dns"); }
      elsif( ($ret != 0) && ($ret != 1) ) { $msg = MSG_error("fetchmail_err_generic"); }

      # send error report if necessary
      if ($msg ne '') {
	print STDERR "fetchmail: fetchmail threw an error: $msg\n" if ($Debug);
	
	if ($file =~ /^multidrop/) {
	  # send mail to admin
	  open(MAIL, "|$Mail_cmd -s '$fetchmailErrorSubject' sys");
	} else {
	  # send mail to user
	  open(MAIL, "|$Mail_cmd -s '$fetchmailErrorSubject' $file");
	}
	print MAIL $fetchmailErrorMsg . "\n";
	print MAIL "$msg\n";
	close(MAIL);
      }
    }
  }
  close(RCDIR);

  # remove lock
  unlink($Fetchmail_lock);
}

1;
