#
# $Id: Group.pm,v 1.65 1999/10/08 06:19:43 asun Exp $
#
# Package for managing groups
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Group;

use vars qw(@ISA @EXPORT @EXPORT_OK);

require Exporter;
require SelfLoader;

@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(group_add group_add_user group_del_user group_del
	     group_show group_list group_set_user group_exist
	     group_set_owner group_get_owner
	     group_set_quota group_get_quota
	     group_add_nohome group_del_nohome group_set_membership
	    );

require Cobalt::Quota;
import Cobalt::Quota qw(:DEFAULT);
require Cobalt::List;
import Cobalt::List qw(:DEFAULT);
require Cobalt::Admserv;
import Cobalt::Admserv qw(admserv_set_access);
require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);
require Cobalt::Utility;
import Cobalt::Utility qw(:DEFAULT);
require Cobalt::Atalk;
import Cobalt::Atalk qw(:DEFAULT);
use Cobalt::Util;
use Cobalt::Product;

use File::Copy;
use integer;

1;

__DATA__

sub group_del
# group_del: delete a group
# Arguments: the name of the group to delete
# Side effects: Removes the group from the group and htgroup file
# Return value: a status string of the form "604 Gotta dance!"
{
    my($name) = @_;
    return MSG_error("4282", $name) unless (getgrnam($name));
    my $return;
    return MSG_error("4200", $name)
	if (($System_groups{$name}) || ($name eq $MainGroup));
    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);

    print STDERR "Cleaning from $Gtmp\n" if ($Debug);
    open(GROUP,$Gtmp) || return Cobalt::Group::group_abort(MSG_error("4203", $Gtmp));
    open(GTMP,">$Lockdir/group") || 
	return Cobalt::Group::group_abort(MSG_error("4204", $Lockdir));
    "$name:" =~ /^$name:/;   # pre-compile the pattern
    while (<GROUP>)
    {
	print GTMP unless (//);
    }
    close GTMP;
    close GROUP;
    rename("$Lockdir/group",$Gtmp) ||
	return Cobalt::Group::group_abort(MSG_error("4205", $Gtmp));

    unless ($Features{'pam'}) {
      open(GROUP,$Htgtmp) 
	|| return Cobalt::Group::group_abort(MSG_error("4206", $Htgtmp));
      open(GTMP,">$Lockdir/htgroup") || 
	return Cobalt::Group::group_abort(MSG_error("4207", $Lockdir));
      "$name:" =~ /^$name:/;   # pre-compile, not in the loop
      while (<GROUP>)
      {
	print GTMP unless (//);
      }
      close GTMP;
      close GROUP;
      rename("$Lockdir/htgroup",$Htgtmp) ||
	return Cobalt::Group::group_abort(MSG_error("4208", $Htgtmp));
    }

    # check to see if it's a symbolic link. if so, delete that first.
    # as we use absolute symbolic links, we can just go ahead and use
    # the name.
    my $site_home = "$Htgroup_dir/$name";
    if (-l $site_home) {
      Cobalt::Utility::sauce_deltree(readlink($site_home));
      unlink($site_home);
    } else {
      Cobalt::Utility::sauce_deltree($site_home);
    }

    Cobalt::Group::samba_del($name) if ($Features{'smb'});
    if ($Features{'atalk'})
    {
      my $atalk_ret = Cobalt::Atalk::atalk_del_volume("$Htgroup_dir/$name");
      return $atalk_ret unless ($atalk_ret =~ /^2/o);
    }

    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);

    # Remove the discussion group and mailing list
    Cobalt::List::list_delete("", "", $name);

    print STDERR "Completed deletion of $name\n" if ($Debug);
    return MSG_ok("2235");
}


sub group_del_nohome
# group_del:_nohome delete a group that does not have a home directory
# Arguments: the name of the group to delete
# Side effects: Removes the group from the group and htgroup file
# Return value: a status string of the form "604 Gotta dance!"
{
    my($name) = @_;
    return MSG_error("4201", $name) unless (getgrnam($name));
    my $return;
    return MSG_error("4202", $name)
        if (($System_groups{$name}) || ($name eq $MainGroup));
    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);

    print STDERR "Cleaning from $Gtmp\n" if ($Debug);
    open(GROUP,$Gtmp) || return Cobalt::Group::group_abort(MSG_error("4283", $Gtmp));
    open(GTMP,">$Lockdir/group") ||
        return Cobalt::Group::group_abort(MSG_error("4284", $Lockdir));
    "$name:" =~ /^$name:/;   # pre-compile the pattern
    while (<GROUP>)
    {
        print GTMP unless (//);
    }
    close GTMP;
    close GROUP;
    rename("$Lockdir/group",$Gtmp) ||
        return Cobalt::Group::group_abort(MSG_error("4285", $Gtmp));

    unless ($Features{'pam'}) {
      open(GROUP,$Htgtmp) || return Cobalt::Group::group_abort(MSG_error("4286", $Htgtmp));
      open(GTMP,">$Lockdir/htgroup") ||
        return Cobalt::Group::group_abort(MSG_error("4287", $Lockdir));
      "$name:" =~ /^$name:/;   # pre-compile, not in the loop
      while (<GROUP>)
      {
        print GTMP unless (//);
      }
      close GTMP;
      close GROUP;
      rename("$Lockdir/htgroup",$Htgtmp) ||
        return Cobalt::Group::group_abort(MSG_error("4288", $Htgtmp));
    }

    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);


    print STDERR "Completed deletion of $name\n" if ($Debug);
    return MSG_ok("2231");
}



sub group_del_user
# group_del_user: delete a member from a group
# Arguments: the group name and a list of users to delete
# Side effects: calls &group_set_membership
# Return value: a status string of the form "725 plane out of fuel, can't land"
{
    my $return;
    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);
    my ($group,@victims) = @_;
    print STDERR "Removing @victims from group $group\n" if ($Debug);
    my $victim;
    my @members = Cobalt::Group::group_show($group);
    foreach $victim (@victims)
    {
	@members = grep(!/^$victim$/,@members);
    }
    return $return unless (($return = Cobalt::Group::group_set_membership($group,@members))
			   =~ /^2/o);
    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);
    return MSG_ok("2236");
}

sub group_show
# group_show: list who is in this group
# Arguments: group name
# Side effects: none
# Return value: a sorted array of group members
# Doesn't include the admin or the web server user
{
    my ($group) = @_;
    my @users = sort(split(/\s+/o,(getgrnam($group))[3]));

    @users = (grep(!/^($Adminuser|$WWWuser)$/o,sort(@users)));
    return @users;
}

sub group_list
# group_list: Returns a list of groups, not including system groups
# Arguments: none
# Side effects: none
# Return value: a sorted array of group names
{
    my ($name,@return);
    setgrent();
    while ($name = getgrent)
    {
	push(@return,$name) unless ($System_groups{$name});
    }
    endgrent();
    return sort(@return);
}

sub group_exist
# group_exist: does a group exist?
# Arguments: login name
# Side effects: none
# Return value: true if they do, false if they don't
{
  setgrent; # flush
  (getgrnam($_[0])) ? (return 1) : (return 0);
}

sub group_set_quota
# Set the quota for a group
# Arguments: group name, quota
# Side effects: sets the quota for the group
# Return value: string of the form "816 This onion already has a definition"
{
    my ($group,$quota) = @_;
    # Error message in MB, not KB
    my $warn_quota = ($MaxQuota / 1024);
    return MSG_error("4281", $warn_quota)
	if ($quota > $MaxQuota);
    return Cobalt::Quota::setqlim("$Htgroup_dir/$group",
				  (getgrnam($group))[2],$quota,
				  ($quota + $Quota_harder),1);
}

sub group_get_quota
# Get quota information for a group
# Arguments: group name
# Side effects: none
# Return value: array containing used, limit
{
    my ($group) = @_;
    return (Cobalt::Quota::repquota($group,1))[0,1];
}

sub group_add
# group_add: Create a group
# Arguments: the name of the group, group quota, 
# (optional) mountpoint, and (optional) list of members
# Side effects: adds entries to the group file
# Return value: the usual status string of format "816 I said NO camels"
{
    my $locale=`cat /etc/cobalt/locale`;
    chop($locale);

    my ($return,$gid);
    return MSG_error("1253") unless ($Features{'groups'});
    my ($group,$quota,$mountpoint,@members) = @_;
    return MSG_error("4234", $group) if ($group !~ /^[\w\-\.]+$/o);
    return MSG_error("4242", $MaxNameLen)
	if (length($group) > $MaxNameLen);
    my $warn_quota = ($MaxQuota / 1024);
    return MSG_error("4243", $warn_quota)
	if ($quota > $MaxQuota);
    return MSG_error("4267", $group) 
	if ($System_groups{$group});
    return MSG_error("4299", $group)
	if (getgrnam($group));
    return MSG_error("4240", $group) 
	if (getpwnam($group));
    return MSG_error("4241", $group)
	if (Cobalt::List::alias_exist($group));
    return 
	Cobalt::Group::group_abort(MSG_error("4244", $MaxMembers))
	    if ($#members >= $MaxMembers);

    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);

    # make sure it's not in the database
    my $home = "$Htgroup_dir/$group";
    return Cobalt::Group::group_abort(MSG_error("4226", $group)) if
	(scalar(Cobalt::Meta::query("type" => "vsite", 
				    "keys" => ["name"],
				    "where" => ["name", "=", "$group"])) and
	 (-e $home));

    ### Add to the files in /etc
    setgrent();
    for ($gid = $Firstgid; getgrgid($gid); $gid++) {;}
    my $line = "${group}::${gid}:" . join(',',@members,$Adminuser) . "\n";
    print STDERR "Adding to $Gtmp: $line" if ($Debug);
    open(GTMP,">>$Gtmp") 
	|| return Cobalt::Group::group_abort(MSG_error("4222", $Gtmp));
    print GTMP $line;
    close GTMP;
    unless ($Features{'pam'}) {
      my $htline = "${group}:" . join(' ',@members,$Adminuser). "\n";
      open(GTMP,">>$Htgtmp") 
	|| return Cobalt::Group::group_abort(MSG_error("4223", $Htgtmp));
      print GTMP $htline;
      close GTMP;
    }

    ### Make the directory
    if ($Features{'groupdirs'}) {
      print STDERR "Creating directories under $home\n" if ($Debug);
      # Make sure the top-level group directory exists
      unless (-d $Htgroup_dir)	{
	# this line doesn't chmod sometimes
	mkdir($Htgroup_dir,0755);
	chmod 0755, $Htgroup_dir;

	if ($Features{'atalk'}) {
	  # AppleTalk seems to feel some odd need for this directory.
	  mkdir("$Htgroup_dir/Network Trash Folder",01777)
	    || return Cobalt::Group::group_abort(MSG_error("4247", $Htgroup_dir));
	  chmod(01777,"$Htgroup_dir/Network Trash Folder");
	}
      }
      
      # wipe out an existing directory if necessary. we've already
      # checked to make sure that we can do this.
      Cobalt::Utility::sauce_deltree($home) if (-e $home);

      # shift the directory to the designated directory. this will
      # delete directories that already exist if necessary. those
      # can only show up if you deleted a site while the disk was 
      # disabled.
      if ($mountpoint) {
	  my $actual_dir = "$mountpoint/sites/$group";
	  Cobalt::Utility::sauce_deltree($actual_dir) if (-e $actual_dir);
	  mkdir($actual_dir, 02775) || 
	      group_abort(MSG_error("group_cant_create_home", $home));
	  unless (symlink($actual_dir, $home)) { 
	      rmdir($actual_dir);
	      group_abort(MSG_error("group_cant_create_home", $home));
	  }
      } else {
	  mkdir($home,02775) 
	      || return Cobalt::Group::group_abort(MSG_error("group_cant_create_home", $home));
      }

      chown($WWWuid,$gid,$home);
      # The setgid bit means that all files or subdirectories will be created
      # also owned by this group.  The user writable bit has to be set or
      # appletalk won't work
      chmod(02775,$home);
      
      mkdir("$home/$Webdir",02775) 
	|| return Cobalt::Group::group_abort(MSG_error("group_cant_create_home", $home));
      chown($WWWuid,$gid,"$home/$Webdir");
      chmod(02775,"$home/$Webdir");
      
      if ($Features{'privatedirs'})	{
	mkdir("$home/$Webdir/$Private_html",02770) 
	  || return Cobalt::Group::group_abort(MSG_error("4228", $home));
	chown($WWWuid,$gid,"$home/$Webdir/$Private_html");
	chmod(02770,"$home/$Webdir/$Private_html");
      }
      
      # special case for home site
      if ($group eq $MainGroup) {
	  if (-d "$Group_skel/$MainGroup/$locale") {
	      system("cp -r $Group_skel/$MainGroup/$locale/\* $home/ > /dev/null 2>&1");
	  }
	  if (-d "$Group_skel/$MainGroup/common") {
	      system("cp -r $Group_skel/$MainGroup/common/\* $home/ > /dev/null 2>&1");
	  }
      } else {
	  if (-d "$Group_skel/$locale") {
	      system("cp -r $Group_skel/$locale/\* $home/ > /dev/null 2>&1");
	  }
	  if (-d "$Group_skel/common") {
	      system("cp -r $Group_skel/common/\* $home/ > /dev/null 2>&1");
	  }
      }

      system("chown -R $WWWuid.$gid $home > /dev/null 2>&1");
	  
      if ($Features{'privatedirs'}) {
	# lock down access to the private directory
	Cobalt::Admserv::admserv_set_access("$home/$Webdir/$Private_html","require","group",$group);
	
	# now do index.html for the private area
	if (-e "$Group_skel/private/private.tmpl") {
	  my $cgroup = ucfirst($group);
	  open(SRC,"$Group_skel/private/private.tmpl") 
	    || return Cobalt::Group::group_abort(MSG_error("4279", $Group_skel));
	  open(DEST,">$home/$Webdir/$Private_html/index.html") 
	    || return Cobalt::Group::group_abort(MSG_error("4280", $home));
	  while (<SRC>) {
	    if (/\[/o) {
	      s/\[GROUP\]/$cgroup/;
	      s/\[PATH\]/$home\/$Private_html\/index.html/;
	    }
	    print DEST;
	  }
	  close DEST;
	  close SRC;
	  
	  chown($WWWuid,$gid,"$home/$Webdir/$Private_html/index.html");
	  chmod(0664,"$home/$Webdir/$Private_html/index.html");
	}
      }

      Cobalt::Group::samba_add($home,$group) if $Features{'smb'};

      if ($Features{'atalk'})
      {
        my $atalk_ret = Cobalt::Atalk::atalk_set_volume($home, $group);
        return Cobalt::Group::group_abort($atalk_ret) if ($atalk_ret !~ /^2/o);
      }      

    } # if groupdirs

    if ($Features{'vmachine'}) {
	# hate any and all stupid implicit creations -- gsri 6/26/99
	# if the group is site-adm, add the fqdn of each user to get
	# the member list; else all users belong to the same fqdn
	if ($group eq $Sites_AdminGroup) {
	    map {
		my $ref = \$_;
		$_ = "$$ref\@" . (Cobalt::User::user_list_site($$ref))[0];
	    } @members;
	}
	else {
	    # Remove the user from all virtual mailing lists here
	    my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];
	    map { $_ .= "\@${fqdn}"; } @members;
	}
    }
    else {
	my $fqdn = Cobalt::Netutil::netutil_get_fulldomain();
	map { $_ .= "\@${fqdn}"; } @members;
    }

    ### end game
    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);
    Cobalt::Group::group_set_quota($group,$quota);

    print STDERR "Completed adding $group\n" if ($Debug);

    # We restart the web server so it re-reads its group membership;
    # otherwise it may not be able to read group files properly.
    # If we just hup'ed, there would be no permissions set
    # For a virtual machine model a group is only added 
    #   with a virtual interface, which then restarts
    #   the web server, no need to restart twice
    unless($Features{'vmachine'})
    {
        # Restart the web server, if it's up
        if (-f "$Subsys/httpd")
        {
            system("$Htscript stop >>$Sauce_log 2>&1");
            system("$Htscript start >>$Sauce_log 2>&1");
            system("$Lcdshowscript >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
        }
    }
    return MSG_ok("2237", $group);
}


sub group_add_nohome
# group_add_nohome: Create a group
# Arguments: the name of the group, group quota, and (optional) list of members
# Side effects: adds entries to the group file
# Return value: the usual status string of format "816 I said NO camels"
{
    my ($return,$gid);
    my ($group,$quota,@members) = @_;
    return MSG_error("4289", $group) if ($group !~ /^[\w\-\.]+$/o);
    return MSG_error("4290", $MaxNameLen)
        if (length($group) > $MaxNameLen);
    my $warn_quota = ($MaxQuota / 1024);
    return MSG_error("4291", $warn_quota)
        if ($quota > $MaxQuota);
    return MSG_error("4293", $group)
        if (getgrnam($group));
    return MSG_error("4294", $group)
        if (getpwnam($group));
    return MSG_error("4295", $group)
        if (Cobalt::List::alias_exist($group));
    return
        Cobalt::Group::group_abort(MSG_error("4296", $MaxMembers))
            if ($#members >= $MaxMembers);

    return $return unless (($return = Cobalt::Group::lock_group()) =~ /^2/o);

    ### Add to the files in /etc
    setgrent();
    for ($gid = $Firstgid; getgrgid($gid); $gid++) {;}
    my $members = join(',',@members,$Adminuser);
    my $line = "${group}::${gid}:${members}\n";
    print STDERR "Adding to $Gtmp: $line" if ($Debug);
    open(GTMP,">>$Gtmp")
        || return Cobalt::Group::group_abort(MSG_error("4297", $Gtmp));
    print GTMP $line;
    close GTMP;
    unless ($Features{'pam'}) {
      my $hmembers = join(' ',@members,$Adminuser);
      my $htline = "${group}:${hmembers}\n";
      open(GTMP,">>$Htgtmp")
        || return Cobalt::Group::group_abort(MSG_error("4298", $Htgtmp));
      print GTMP $htline;
      close GTMP;
    }

    ### end game
    return $return unless (($return = Cobalt::Group::commit_group()) =~ /^2/o);
    Cobalt::Group::group_set_quota($group,$quota);

    if ($Features{'vmachine'}) {
       # hate any and all stupid implicit creations -- gsri 6/26/99
       # if the group is site-adm, add the fqdn of each user to get
       # the member list; else all users belong to the same fqdn
       if ($group eq $Sites_AdminGroup) {
           map {
               my $ref = \$_;
               $_ = "$$ref\@" . (Cobalt::User::user_list_site($$ref))[0];
           } @members;
       }
       else {
           # Remove the user from all virtual mailing lists here
           my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];
           map { $_ .= "\@${fqdn}"; } @members;
       }
    }
    else {
       my $fqdn = Cobalt::Netutil::netutil_get_fulldomain();
       map { $_ .= "\@${fqdn}"; } @members;
    }
    list_create("", "", $group, @members);

    print STDERR "Completed adding $group\n" if ($Debug);
    return MSG_ok("2230", $group);
}

sub group_add_user
# group_add_user: Adds a user to the group
# Arguments: the name of the group and a list of new users to add
# Side effects: modifies the entry in the group file by calling
#   &group_set_membership
# Return value: usual status string of format "567 take back your mink"
{
    my($group,@newguys) = @_;

    my $ret = Cobalt::Group::lock_group();
    return $ret if ($ret !~ /^2/o);

    my @members = Cobalt::Group::group_show($group);
    push(@members,@newguys);
    print STDERR "Putting @newguys in $group\n" if ($Debug);

    $ret = Cobalt::Group::group_set_membership($group,@members);
    if ($ret !~ /^2/o) {
        Cobalt::Group::unlock_group();
	return $ret;
    }

    $ret = Cobalt::Group::commit_group();
    if ($ret !~ /^2/o) {
        Cobalt::Group::unlock_group();
	return $ret;
    }

    return MSG_ok("2234", $group);
}

sub group_set_user
# group_set_user: set the members list for a group
# Arguments: the name of the group and an array of members
# Side effects: modifies /etc/group, /etc/htgroup
{
    my($group,@members) = @_;
    my $ret;

    $ret = Cobalt::Group::lock_group();
    return $ret if ($ret !~ /^2/o);

    $ret = Cobalt::Group::group_set_membership($group,@members);
    if ($ret !~ /^2/o) {
        Cobalt::Group::unlock_group();
	return $ret;
    }

    $ret = Cobalt::Group::commit_group();
    if ($ret !~ /^2/o) {
        Cobalt::Group::unlock_group();
	return $ret;
    }

    return MSG_ok("2222", $group);
}

sub group_set_membership
# group_set_membership: set the members list for a group
# Arguments: the name of the group and an array of members
# Side effects: modifies /etc/group, /etc/htgroup
#   This function ensures that the list contains no duplicate names.
{
    my ($group,@list) = @_;
    return Cobalt::Group::group_abort(MSG_error("4210", $group)) unless (getgrnam($group));
    return 
	Cobalt::Group::group_abort(MSG_error("4245", $MaxMembers))
	    if ($#list >= $MaxMembers);

    print STDERR "Membership of $group is being set to @list\n" if ($Debug);

    # Using the hash table is a cute way to guarantee no repeat
    # entries
    my %temp = map { $_, 1} @list;
    $temp{$Adminuser} = 1; # ditto for admin
    my $list = join(',',sort(keys(%temp)));

    # Don't go to any trouble if nothing is changing
    my $oldmembers = join(',',Cobalt::Group::group_show($group));
    if ($oldmembers eq $list)
    {
	return MSG_ok("2224");
    }

    # fix the group file
    open(GROUP,$Gtmp) ||  return Cobalt::Group::group_abort(MSG_error("4211", $Gtmp));
    open(GTMP,">$Lockdir/group") 
	|| return Cobalt::Group::group_abort(MSG_error("4212", $Lockdir));
    "$group:x:" =~ /^$group:(.+):/;   # pre-compiled regexps are better
    while (<GROUP>)
    {
	if (//)
	{
	    print GTMP "$group:$1:$list\n";
	}
	else
	{
	    print GTMP;
	}
    }
    close GTMP;
    close GROUP;
    rename("$Lockdir/group",$Gtmp) ||
	return Cobalt::Group::group_abort(MSG_error("4213", $Gtmp));

    unless ($Features{'pam'}) {
      # Fix the htgroup file
      $list = join(' ',keys(%temp));
      open(GROUP,$Htgtmp) ||  return Cobalt::Group::group_abort(MSG_error("4214", $Htgtmp));
      open(GTMP,">$Lockdir/htgroup") 
	|| return Cobalt::Group::group_abort(MSG_error("4215", $Lockdir));
      "$group:" =~ /^$group:/;   # pre-compile for speed
      while (<GROUP>)
      {
	if (//)
	{
	    print GTMP "$group:$list\n";
	}
	else
	{
	    print GTMP;
	}
      }
      close GTMP;
      close GROUP;
      rename("$Lockdir/htgroup",$Htgtmp) ||
	return Cobalt::Group::group_abort(MSG_error("4216", $Htgtmp));
    }

    # Fix the list membership to match the group membership
    if ($Features{'vmachine'}) {
	# hate any and all stupid implicit creations -- gsri 6/26/99
	# if the group is site-adm, add the fqdn of each user to get
	# the member list; else all users belong to the same fqdn
	if ($group eq $Sites_AdminGroup) {
	    map {
		my ($fqdn,$gr) = Cobalt::User::user_list_site($_);
		$_ .= "\@${fqdn}";
	    } @list;
	}
	else {
	    # Remove the user from all virtual mailing lists here
	    my $fqdn = (Cobalt::Vsite::vsite_get_bygroup($group))[1];
	    map { $_ .= "\@${fqdn}"; } @list;
	}
    }
    else {
	my $fqdn = Cobalt::Netutil::netutil_get_fulldomain();
	map { $_ .= "\@${fqdn}"; } @list;
    }
    # no checkee returnee
    Cobalt::List::list_set_mem("", "", $group, @list);
    return MSG_ok("2225");
}

##############################

sub lock_group
# Locks the group file so that nobody can collide with us.
# Arguments: none
# Side effects: copies /etc/group to /etc/gtmp
{
    my ($return);
    if (-e $Gtmp)
    {
	return Cobalt::Group::group_abort(MSG_error("4217"));
    }
    else
    {
	# Lock the group file by creating the gtmp file as a
	# copy of the group file.
	print STDERR "Copying group file $Group to $Gtmp\n" if ($Debug);

	# inline for better performance
	open(SRC,$Group) 
	    || return Cobalt::Group::group_abort(MSG_error("4236", $Group));
	open(DEST,">$Gtmp") 
	    || return Cobalt::Group::group_abort(MSG_error("4237", $Gtmp));
	print DEST while (<SRC>);
	close DEST;
	close SRC;

	open(LOCK,">$Lockdir/group-$$")
	  || return Cobalt::Group::group_abort(MSG_error("4265", $!));
	print LOCK "$$\n$0";
	close LOCK;

    	unless ($Features{'pam'}) {
	  if (! -e $Htgroup)
	  {
	    # this looks like a fresh system
	    print STDERR "Creating new htgroup file $Htgroup\n" if ($Debug);
	    open(HTGROUP,">$Htgroup")
	      || return MSG_error("4248", $Htgroup);
	    print HTGROUP "${Adminuser}:$Adminuser\n";
	    close HTGROUP;
	  }
	  
	  open(SRC,$Htgroup) 
	    || return Cobalt::Group::group_abort(MSG_error("4238", $Htgroup));
	  open(DEST,">$Htgtmp") 
	    || return Cobalt::Group::group_abort(MSG_error("4239", $Htgtmp));
	  print DEST while (<SRC>);
	  close DEST;
	  close SRC;
    	}
    }
    print STDERR "Locked $Group with $Gtmp\n" if ($Debug);
    return MSG_ok("2226");
}

sub commit_group
# Commits the changes to the group file by moving gtmp back onto group
# Arguments: none
# Side effects: rebuilds the group files based on the aliases file, and
#   replaces /etc/group with /etc/gtmp
{
    rename($Gtmp,$Group) 
	|| return Cobalt::Group::group_abort(MSG_error("4219", $Group));
    chmod(0644,$Group);
    print STDERR "Unlocked $Gtmp\n" if ($Debug);
    if ($Features{'pam'}) {
        # currently sauce supports shadow group only
        # support for radius/nis/etc requires libpwdb
	system("$Grpconv 2>/dev/null") &&
	    return (MSG_error("group_cant_grpconv"));
    }
    else {
    rename($Htgtmp,$Htgroup) 
	|| return Cobalt::Group::group_abort(MSG_error("4218", $Htgroup));
    chmod(0644,$Htgroup);
    }

    unlink("$Lockdir/group-$$");
    endgrent();   # sync
    return MSG_ok("2227");
}

sub unlock_group
# Unlocks the group file by wiping $Gtmp
# Arguments: none
# Side effects: deletes $Gtmp and $Htgtmp
{
    unless ($Features{'pam'}) {
      unlink($Htgtmp) || return MSG_error("4220", $Gtmp);
    }
    unlink($Gtmp) || return MSG_error("4221", $Gtmp);
    print STDERR "Unlocked $Gtmp\n" if ($Debug);

    unlink("$Lockdir/group-$$");
    endgrent();   # sync
    return MSG_ok("2229");
}

sub group_abort
# leave in a graceful manner
# Arguments: a string to print
# Side effects: unlocks anything we've locked
{
    Cobalt::Group::unlock_group() if (-e "$Lockdir/group-$$");
    return join('',@_);
}

sub samba_del
# Removes a share from the samba configuration file
# Arguments: share name (generally the group name)
# Side effects: modifies smb.conf
{
    my ($share) = @_;
    return MSG_error("1256") 
      unless ($Features{'smb'});
    return MSG_error("4235") if (-e "$Lockdir/smb.conf");
    open(SMB,"$Smbconf") 
	|| return MSG_error("4230", $Smbconf);
    open(SNB,">$Lockdir/smb.conf") 
	|| return MSG_error("4231", $Lockdir);
    # compile the expression now, not each time we loop
    ";$share BEGIN" =~ /;$share BEGIN/; 
    while (<SMB>)
    {
	last if (//);
	print SNB;
    }
    # compile the expression now, not each time we loop
    ";$share END" =~ /;$share END/;  
    while (<SMB>)
    {
	last if (//);
    }
    print SNB while (<SMB>);
    close SMB;
    close SNB;
    rename("$Lockdir/smb.conf",$Smbconf) ||
	return MSG_error("4232", $Smbconf);
    return MSG_ok("2258");
}

sub samba_add
# Add a share to the samba config file
# Arguments: directory name, share name
# Side effects: modifies smb.conf
{
    my ($dir,$share) = @_;
    return MSG_error("1255") 
      unless ($Features{'smb'});
    return MSG_error("4209") if (-e "$Lockdir/smb.conf");
    open(SMB,">>$Smbconf") 
	|| return MSG_error("4229", $Smbconf);
    print SMB ";$share BEGIN (do not delete this line)\n";
    print SMB "[$share]\n  path = $dir\n  public = no\n  browseable = yes\n";
    print SMB "  writable = yes\n  printable = no\n  create mask = 0775\n";
    print SMB "  valid users = $Adminuser \@$share\n";
    print SMB "  force create mode = 0664\n";
    print SMB "  force directory mode = 0775\n";
    print SMB "  hide dot files = yes\n";
    print SMB ";$share END (do not delete this line)\n";
    close SMB;
    return MSG_ok("2257");
}

sub group_mkdiscuss
# Creates a discussion group; sets up all the directories and files
# Arguments: group name, group id
# Side effects: creates the directories and copies HTML files
# Return value: the usual status string
{
    my ($group,$gid) = @_;
    my $cgroup = ucfirst($group);
    my $sfile;
    return MSG_error("1254")
      unless ($Features{'discuss'});

    # Make sure the top-level directories for discussion stuff
    # are all there and in place
    mkdir ($Discuss_html_dir,0755) unless (-d $Discuss_html_dir);
    mkdir ($Discuss_dir,0755) unless (-d $Discuss_dir);

    if ((-d "$Discuss_html_dir/$group") ||
	(-d "$Discuss_dir/$group"))
    {
	return MSG_error("4246", $group);
    }

    # make directories
    mkdir("$Discuss_dir/$group",0755)
	|| print STDERR "Cannot make discussion directory: $!\n";
    mkdir("$Discuss_html_dir/$group",0775);
    mkdir("$Discuss_html_dir/$group/private",0775);
    chown($WWWuid,$gid,"$Discuss_html_dir/$group",
	  "$Discuss_html_dir/$group/private");
    chmod(0775,"$Discuss_html_dir/$group",
	  "$Discuss_html_dir/$group/private");
    
    # touch the data.txt file
    open(D,">$Discuss_html_dir/$group/data.txt")
      || return MSG_error("4292", $Discuss_html_dir);
    close D;
    chmod(0664,"$Discuss_html_dir/$group/data.txt");
    chown($WWWuid,$gid,"$Discuss_html_dir/$group/data.txt");

    # HTML templates
    foreach $sfile (@Discuss_html)
    {
	open(IN,"$Discuss_skel/$sfile")
	    || return MSG_error("4276", $sfile);
	open(OUT,">$Discuss_html_dir/$group/$sfile")
	    || return MSG_error("4275", $sfile);
	while (<IN>)
	{
	    if (/XXX|GROUP/o)
	    {
		$_ =~ s/XXX/$cgroup/g;
		$_ =~ s/GROUP/$group/g;
	    }
	    print OUT;
	}
	close IN;
	close OUT;
	chown($WWWuid,$gid,"$Discuss_html_dir/$group/$sfile");
	chmod(0664,"$Discuss_html_dir/$group/$sfile");
    }

    # CGI templates
    foreach $sfile (@Discuss_cgis)
    {
	open(IN,"$Discuss_skel/$sfile")
	    || return MSG_error("4249", $sfile);
	open(OUT,">$Discuss_dir/$group/$sfile")
	    || return MSG_error("4269", $sfile);
	while (<IN>)
	{
	    if (/ROUP/o)
	    {
		$_ =~ s/CROUP/$cgroup/g;
		$_ =~ s/GROUP/$group/g;
	    }
	    print OUT;
	}
	close IN;
	close OUT;
	chmod(0755,"$Discuss_dir/$group/$sfile");
    }
    Cobalt::Admserv::admserv_set_access("$Discuss_html_dir/$group","require","group",$group);
    Cobalt::Admserv::admserv_set_access("$Discuss_dir/$group","require","group",$group);
    return MSG_ok("2223");
}

1;
