# Http.pm: Functions for managing the user web server
#
# $Id: Http.pm,v 1.5 1999/10/02 16:44:26 duncan Exp $
# 
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Http;

use vars qw(@ISA @EXPORT @EXPORT_OK);

require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(http_restart);

use Cobalt::Product;

1;

sub http_restart
# Restart the HTTP server
# According to the Apache docs, sending a USR1 signal is the best
# way to signal the server to restart.  This is much faster and
# more graceful than messing with init scripts.
# See http://www.apache.org/docs/stopping.html
# Arguments: 1 if we should start up the server when it's not already up
#  (otherwise, if it's not up we'll leave it down).
# Return value: none
# Side effects: "The USR1 signal causes the parent process to advise the 
#  children to exit after their current request (or to exit immediately if 
#  they're not serving anything). The parent re-reads its configuration files 
#  and re-opens its log files. As each child dies off the parent replaces it 
#  with a child from the new generation of the configuration, which begins 
#  serving new requests immediately."
# If the server is not currently running, start it.
{
  my ($bring_it_up) = @_;

  if (! -e "$Subsys/httpd") {
    # The server is not currently running, start it up.
    return unless ($bring_it_up);
    print STDERR "Starting httpd\n" if ($Debug);
    system("$Htscript start >>$Sauce_log 2>&1");
    system("$Lcdshowscript >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
  } elsif (! -e $Htpid) {
    # The init script thinks the server is running, but it isn't
    # Clear out the init script's state and start it up.
    return unless ($bring_it_up);
    print STDERR "Restarting httpd\n" if ($Debug);
    system("$Htscript stop >>$Sauce_log 2>&1");
    system("$Htscript start >>$Sauce_log 2>&1");
    system("$Lcdshowscript >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
  } else {
    # The server is running and seems normal
    # Read the PID from the file and send it the signal
    open(PID,$Htpid) || return undef;
    my $pid = <PID>;
    close PID;
    chomp $pid;
    print STDERR "Signalling httpd ($pid)\n" if ($Debug);
    kill('USR1',$pid) if ($pid);
  }
}

1;




