#
# $Id: Language.pm,v 1.23 1999/10/02 16:44:26 duncan Exp $
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#
# Cobalt Networks http://www.cobaltnet.com
#
#                               ACHTUNG!
#
# The messages are now stored in DB format.  Edit messages/*.txt in the
# special-sauce CVS repository from now on.  The DB file is generated
# from the text file by autoize.

package Cobalt::Language;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
use Cobalt::Product;
@ISA    = qw(Exporter SelfLoader);
# We re-export all the routines and variables from the Language_*
# files.  This is the only place we need to list them all.
@EXPORT = qw($LANGUAGE %Language
	     msg_get msg_build_database msg_localtime
	     $DefaultAdminFullName @Months @Days
	     %Servers
	    );
use Cobalt::Util;
use strict;
use vars qw($LANGUAGE %Language $DefaultAdminFullName @Months @Days %Servers);
use DB_File;
use Fcntl qw(O_CREAT O_RDWR O_RDONLY);

my $Debug   = -e "/etc/DEBUG";
   $^W      = ($Debug ||= 0);
my $Perldir = "/usr/lib/perl5/site_perl";  # Where do we find our Perl modules?

# Globals we use so we don't have to constantly open and close the database
use vars qw(%Messages $Messages_Tied);

####
# Figure out what language we're speaking today

# English is the default language.
# Look, we need to default somehow; I'd rather pick some default
# than just barf if the user hasn't set things up.
$LANGUAGE = "en";

# If the language is defined in the environment, that
# should override whatever we got so far.
$LANGUAGE = $ENV{'LANGUAGE'} if ($ENV{'LANGUAGE'});

####
# We use this hash table to set the properties of the language
# These override features for the purpose of releasify, undep,
# and autoize (providing you have current versions of the tools)
$Language{'english'}   = 0;
$Language{'japanese'}  = 0;
$Language{"$LANGUAGE"} = 1;
$Language{'lastNameFirst'} = 1 if ($LANGUAGE eq "english");
$Language{'lastNameFirst'} = 0 if ($LANGUAGE eq "japanese");

####
# For backwards-compatability
if (-e "$Perldir/auto/Cobalt/Language/msg_get.al") {  # dodge the bootstrap
  $DefaultAdminFullName = MSG_get("DefaultAdminFullName");
  # The UI should be using @Services instead...
  my $svr;
  foreach $svr (MSG_get("services_L")) {
    $Servers{$svr} = MSG_get("servers_$svr");
  }
  
  @Months               = MSG_get('Months_L');  # get as a list
  @Days                 = MSG_get('Days_L');    # get as a list

  $Language{'shiftjis'}  = MSG_get("shiftjis");
}  

1;  # the loneliest number

__DATA__
####
# Now, some subroutines...

sub msg_get
# Read in a message from the database
# Arguments: message name
# Note that the string returned may include variables, so you may get
#   back a message like "User $name has violated the laws of physics"
#   which then needs to be eval'ed at run time to insert the proper
#   definition of (in this example) $name.
# If the string ends in "_L", then it is recognized as a list value,
#   and a list is returned.  The list is built out of the DB based on
#   key_L001 through key_Lnnn.
# Return value: A text string
{
  my ($package, $filename, $line) = caller;
  my ($msg_tag) = @_;
  #my %Messages;
#  unless ($Cobalt::Language::Messages_Tied) {
#    unless(tie(%Cobalt::Language::Messages, 'DB_File', 
#	       "/etc/cobalt/messages/$LANGUAGE.db",O_RDONLY(),0644))
#      {
#	return "Cannot read messages database /etc/cobalt/messages/$LANGUAGE.db: $!";
#      }
#    $Cobalt::Language::Messages_Tied = 1;
#  }
  my $message;
  if ($msg_tag =~ /_L$/o) {
    # Asking for a list
    my $item;
    my $i = 1;
    my $tag = sprintf("$msg_tag%.3d",$i);
    my @message;
#    while ($item = $Cobalt::Language::Messages{$tag}) {
    while (($item = MSG_get($tag)) !~ /_L\d\d\d$/) {
      push(@message,$item);
      $i++;
      $tag = sprintf("$msg_tag%.3d",$i);
      if ($i > 1000) {
	print STDERR "$msg_tag has sent us into an infinite loop at $package::$filename line $line\n";
	last;
      }
    }
    return "Cobalt interface message for $msg_tag (We should never see this)"
      unless ($#message > -1);
    return @message;
  } else {
    # Normal message string
#    $message = $Cobalt::Language::Messages{$msg_tag};
    $message = MSG_get($msg_tag);
  }
#  untie %Messages;
  $message = "$message (($package $filename $line))" if ($Debug);
  return "Cobalt interface message for $msg_tag (We should never see this)"
    unless ($message);
  return $message;
}

sub msg_build_database
# Build the DB_File hash database from the text file
# The text file should be a series of lines of the form
#                 'key' => 'value',
# The quotes are optional, either single or double quotes can be
# used.
# Leading/trailing whitespace and blank lines will be ignored.
# Each value must be terminated by a trailing , (this is how
# we deal with multi-line strings).
# A comma, a space, and then three pound signs indicate the
# beginning of a comment.
# Arguments: language (e.g., "english", "japanese")
# Side effects: builds the .db file based on the .txt file
# This is pretty much only used by autoize...
{
  my ($lang) = @_;
  my %DB;
  if ($Cobalt::Language::Messages_Tied) {
    untie %Cobalt::Language::Messages;
    $Cobalt::Language::Messages_Tied = 0;
  }
  unless(tie(%DB, 'DB_File', "/etc/cobalt/messages/$lang.db",
	     O_RDWR()|O_CREAT(),0644))
  {
    return "Cannot read messages database /etc/cobalt/messages/$lang.db";
  }
  eval 'use Cobalt::Product';
  my %found;
  open(TXT,"/etc/cobalt/messages/$lang.txt") ||
    die "Can't open /etc/cobalt/messages/$lang.txt: $!\n";
  while (<TXT>) {
    next if (/^\s*\#/o);  # lines begining with comments are ignored
    next if (/^\s*$/o);   # blank lines are ignored
    my ($key,$val);

    # could also use "$line = split(/('\||\|')/)" and then switch on $#line
    if (/^\s*([^\|]+)\s*\|\s*\'(.+)\'\|$/o) {
      # whole string, right here
      ($key,$val) = ($1,$2);
    }
    elsif (/^\s*([^\|]+)\s*\|\s*\'(.+)$/o) {
      # start of a multi-line string
      ($key,$val) = ($1,"$2\n");
      while (<TXT>) {
	if (/^(.*)\'\|$/o) { # end of the string
	  $val .= $1;
	  last;
	}
	else {
	  $val .= $_;
	}
      }
    }

    print STDERR "Duplicate key `$key' in /etc/cobalt/messages/$lang.txt\n" 
      if ($found{$key});
    $found{$key} = 1;

    # Replace any Product.pm variables with their values
    my $val_copy = $val;
    while ($val_copy =~ /\$([A-Z]\w+)/o) {
      my $variable = $1;
      # There may be uppercase variables that do not appear
      # in product.pm--strip them out of the string so we don't
      # keep trying to match them
      $val_copy =~ s/\$$variable//g;
      if (defined eval('$$variable')) {
	# If it's in Product.pm, then substitute the value in
	$val =~ s/\$$variable/$$variable/g;
      }
    }

    $DB{$key} = $val;
  }
  close TXT;
  untie %DB;
  undef %found;
}

sub msg_localtime
# Return the time string, formatted the way marketing likes it
# Arguments: time in seconds since the epoch
# Return value: a text string telling the time
{
    my ($time) = @_;
    my ($sec,$min,$hour,$mday,$mon,$year,$wday) = localtime($time);
    my $m;
    $year += 1900;
    $hour = "0$hour" if ($hour < 10);
    $min = "0$min" if ($min < 10);
    $sec = "0$sec" if ($sec < 10);
    if ($hour > 12)
    {
	$hour -= 12;
	$m = "P.M.";  # post meridian
    }
    elsif ($hour == 12)
    {
      $m = MSG_get("post_meridian");  # P.M. in english, pm in japanese
    }
    elsif ($hour == 24)
    {
      $hour = 0;
      $m = MSG_get("ante_meridian");
    }
    else
    {
      $m = MSG_get("ante_meridian");
    }

  if ($LANGUAGE eq "japanese") {
    return "${year}N $Months[$mon] $mday $Days[$wday] ${hour}:${min}${m}";
  } else {
    # Personally, I don't like this format.
    return "${hour}:${min} $m on $Days[$wday] $Months[$mon] $mday $year";

    # This would be more standard:
    #my $zone = Cobalt::Time::time_get_zone();
    #return "$Days[$wday] $Months[$mon] $mday ${hour}:${min}${m} $zone $year";
  }
}

1;  # Do not remove this line
