# $Id: Legato.pm,v 1.3 1999/10/02 16:44:26 duncan Exp $
#
# Legato.pm
# Legato related routines
#
# John D. Blair <jdblair@cobaltnet.com>
# Copyright 1998 Cobalt Networks, Inc., http://www.cobaltnet.com
#

package Cobalt::Legato;

use Cobalt::Product;
use Cobalt::Netutil;
use Cobalt::Util;

require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT	= qw(
	     legato_set_serveraddr
	     legato_get_serveraddr
	    );
1;
__DATA__

use strict;

# legato_set_servername
#
# Sets the name of the legato server in the legato config file
# and in /etc/hosts
#
# args: dns name, ip address
# returns: the usual
#
sub legato_set_serveraddr {
  my ($dnsname, $ipaddr) = @_;

  if (! Cobalt::Netutil::netutil_test_ipaddr($ipaddr)) {
    return MSG_error("legato_cant_invalid_ipaddr");
  }

  if (! Cobalt::Netutil::netutil_test_domainname($dnsname)) {
    return MSG_error("legato_cant_invalid_dnsname");
  }

  # edit the legato config file
  open(CONF, $Networker_config);
  open(LOCK, ">$Lockdir/legato.conf");
  my $founddns = 0;
  my $foundip = 0;
  while (<CONF>) {
    if (/^LEGATO_SERVER/) {
      print LOCK "LEGATO_SERVER=$dnsname\n";
      $founddns = 1;
    } elsif (/^LEGATO_IP/) {
      print LOCK "LEGATO_IP=$ipaddr\n";
      $foundip = 1;
    } else {
      print LOCK $_;
    }
  }
  # add the settings at the end even if they weren't found
  if (! $founddns) {
    print LOCK "LEGATO_SERVER=$dnsname\n";
  }
  if (! $foundip) {
    print LOCK "LEGATO_IP=$ipaddr\n";
  }
  close(LOCK);
  close(CONF);
  rename("$Lockdir/legato.conf", $Networker_config);

  # edit the /etc/hosts file
  open(HOSTS, $Hostfile);
  open(LOCK, ">$Lockdir/hosts");
  my $foundhost = 0;
  while (<HOSTS>) {
    if (/\# legato server/) {
      print LOCK "$ipaddr\t$dnsname\t# legato server\n";
      $foundhost = 1;
    } else {
      print LOCK $_;
    }
  }
  if (! $foundhost) {
    print LOCK "$ipaddr\t$dnsname\t# legato server\n";
  }
  close(LOCK);
  close(HOSTS);
  rename("$Lockdir/hosts", $Hostfile);
  
  return MSG_ok("legato_ok_server_updated");
}
      

# legato_get_serveraddr
#
# returns the legato servername and ip in the form (hostname, ipaddr)
#
sub legato_get_serveraddr {
  my $dnsname = '';
  my $ipaddr = '';

  # edit the legato config file
  open(CONF, $Networker_config);
  while (<CONF>) {
    if (/^LEGATO_SERVER=(.*)\n$/) {
      $dnsname = $1;
    } elsif (/^LEGATO_IP=(.*)\n$/) {
      $ipaddr = $1;
    }
  }
  close(CONF);
  
  return ($dnsname, $ipaddr);
}


