#
# $Id: Modem.pm,v 1.32 1999/10/02 16:44:26 duncan Exp $
#
# Written by
#   Andrew Bose andrew@cobaltnet.com
#   Brion Moss (the real stuff, really)
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com/
#

package Cobalt::Modem;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(modem_get_settings    modem_set_settings 
	     modem_get_dialmode    modem_set_dialmode 
	     modem_get_model       modem_set_model 
	     modem_get_connmode    modem_set_connmode 
	     modem_get_dialoutacct modem_set_dialoutacct 
	     modem_get_localip     modem_set_localip 
	     modem_get_gatewaymode modem_set_gatewaymode
	     modem_get_initstr     modem_set_initstr
	     modem_is_online); 

require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);
require Cobalt::Network;
import Cobalt::Network qw(net_set_gateway);
use Cobalt::Util;
use Cobalt::Product;

use strict;
use Fcntl;

1;

__DATA__

########################################################################
#
# PPP using DIALD config information
#
# RPMs required:  diald-0.16-3C1.mips.rpm
#                 diald-config-1.2.1-1.mips.rpm
#                 ppp-2.2.0f-5C1.mips.rpm
#
# Config Files:   /etc/diald.conf - diald and pppd options
#		  /etc/ppp/chatscript - script to authenticate connection
#		  /etc/ppp/pap-secrets - holds user name and passwd for 
#				pap authentication
#		  /etc/ppp/chap-secrets - holds user name and passwd for 
#				chap authentication
#		  /usr/sbin/diald - diald binary
#		  /etc/rc.d/init.d/diald.init  - diald initscript 
#		  /etc/rc.d/rc3.d/S90diald - diald boot script link
#
#########################################################################


sub modem_get_settings
# Descr:	Read modem settings from system files and setup a hash.
#		ie.  model => 'hayes generic'
# Arguments: 	None
# Return:	A hash containing all the modem settings
# 
#  $modem{'MODEL'} => modem model  (not stored or used anywhere at this time) 
#  $modem{'LOCALIP'} => local IP Address or NONE
#  $modem{'NETMASK'} => local netmask or NONE
#  $modem{'USERNAME'} => user name  
#  $modem{'PASSWD'} => password  
#  $modem{'PHONE'} => phone number  
#  $modem{'GWMODE'} = 1 modem is defaultroute, 0 not
#  $modem{'CONNMODE'} => 0 dialondemand, 1 always connected
#			 2 never connected (diald off)
#  $modem{'DIALMODE'} => 0 not used, 1 dial out, 
#			 2 dial in, 3 dial in/out (always 0 for 2800)
#  $modem{'INITSTR'} => modem initstring, like ATZ 
{
  my %modem;

  # get user and password from pap-secrets file
  # user and passwd will be on line that looks like - <user> * <password>
  open( PAP, $Pap_secrets)
    || return MSG_error("modem_cant_read_papsecrets", $Pap_secrets);
  while( <PAP> ) {
    if(/^([^\#]\S*)\s+\*\s+(\S+)/o) {
      ($modem{'USERNAME'},$modem{'PASSWD'}) = ($1,$2);
      last;
    }
  }
  close PAP;

  # get info from chatscript
  # initstring will be on like that looks like - "" ATZ
  # phone number will be on line that looks like - OK ATDT5551212
  open( CHAT, "<$Chat_script")
    || return MSG_error("modem_cant_read_chatscript", $Chat_script);
  while( <CHAT> ) {
    if (/^OK ATDT(.+)/o) {
      $modem{'PHONE'}=$1;
      last;
    }
    elsif (/^OK (.+)/) {
      # when no phone number is set INITSTR shows up as ATDT...bogus!
      # also, if multiple AT lines exist, this gives the last one before ATDT
      $modem{'INITSTR'}=$1 unless ($1 eq "ATDT");
    }
  }
  close CHAT;

  # get info from diald.conf
  $modem{'GWMODE'} = 0;  # default, unless we see otherwise
  # if only 'up' is present then {'CONNMODE'} = 1 = always connected
  # if only 'down' is present then {'CONNMODE'} = 2 = always down = not used
  # if both 'up' & 'down' are absent then {'CONNMODE'} = 0 = dial-on-demand
  # if both are present the first line matters (always recheck w/ new rpms)
  $modem{'CONNMODE'} = 0;	# both are not present unless we see either
  open(DCONF,$Diald_conf)
    || return MSG_error("modem_cant_read_dialdconf", $Diald_conf);
  while (<DCONF>) {
    # if diald.conf has a line with 'netmask 255.255.255.0'
    #	-> {'NETMASK'} = '255.255.255.0'  
    if (/^netmask\s+(\S+)/o) {
      $modem{'NETMASK'} = $1;
    }
    # if diald.conf has a line with 'defaultroute'
    #	-> {'GWMODE'} = 1     - yes
    #   else 
    #	-> {'GWMODE'} = 0     - no 
    elsif (/^defaultroute$/o) {
      $modem{'GWMODE'} = 1;
    }
    # 192.168.0.1, and 192.168.0.2 are fake IP addresses that are 
    # used when we really want to dynamically assign the IP address
    # of the Qube, this also means there should be a line 
    # 'dynamic' in diald.conf
    #
    # if diald.conf has a line with 'local 192.168.0.1'
    #	-> {'LOCALIP'} = 'NONE'
    # if diald.conf has a line with 'local 10.9.25.40'
    #  where 10.9.25.40 could be any real IP address
    #	-> {'LOCALIP'} = '10.9.25.40'
    elsif (/^local\s+(\S+)/o) {
      my $addr = $1;
      if ($addr =~ /^192\.168\.0\.[12]$/o) {
	$modem{'LOCALIP'} = 'NONE';
      } else {
	$modem{'LOCALIP'} = $addr;
      }
    }
    # diald is active...get info on how active from diald.conf
    # if qube-admin adds "restrict" lines, we are screwed!
    elsif (/^up$/o) {  # always connected
      $modem{'CONNMODE'} = 1 if ($modem{'CONNMODE'} == 0);
    }
    elsif (/^down$/o) {  # not used or always down
      $modem{'CONNMODE'} = 2 if ($modem{'CONNMODE'} == 0);
    }
  }
  close DCONF;

  if (! -e $Diald_link) {
    # diald is off...override settings from $Diald_conf
    $modem{'CONNMODE'} = 2;
  }

  return(%modem);
}

sub modem_set_settings
# Decr:		Write modem settings to system files
# Arguments:	A reference to a hash containing all the modem settings
# Return:	Return code that looks like MSG_ok("2000")
# Ex:		$ret=Cobalt::Modem::modem_set_settings( $modemRef );
{
  my($modemref) = @_;
  my %modem = %$modemref;  # dereference the pointer (ugh)
	
  # Start/Restart diald if CONNMODE = 0 or 1
  # Stop diald if CONNMODE = 2
  # Create or delete initscripts link as appropriate
  # /etc/rc.d/init.d/diald.init (start|stop)  
  # S90diald, K55diald
  #

  if ($modem{'LOCALIP'} ne "NONE") {
    if (!Cobalt::Netutil::netutil_test_ipaddr($modem{'LOCALIP'})) {
      return MSG_error("modem_invalid_localip");
    }
    if (!Cobalt::Netutil::netutil_test_ipaddr($modem{'NETMASK'})) {
      return MSG_error("modem_invalid_netmask");
    }
  }
  if ($modem{'CONNMODE'}!=2 && $modem{'USERNAME'} =~ /\s/) {
    return MSG_error("modem_invalid_username");
  }
  if ($modem{'CONNMODE'}!=2 && $modem{'PHONE'} !~ /^[\d\-\(\)\+\-\d\*\,\s\#]+$/o) {
    return MSG_error("modem_invalid_phone");
  }
  # we don't really check the password...that could be anything...

  # for chap-secrets and pap-secrets
  my $commentLine="\# The following line is automatically generated. Do not change or remove";

  # Setup pap-secrets (user,passwd)
  # add/replace last line of pap-secrets to look like:
  #  USERNAME * PASSWD
  # We replace only the line with a matching USERNAME...the name that
  # pppd looks for is determined as follows (from the pppd man page):
  #
  # if the usehostname option is given,
  #      then  the  local  name  is the hostname of this machine
  #      (with the domain appended, if given)
  # else if the name option is given,
  #      then use the argument of the first name option seen
  # else if the local IP address is specified with a hostname,
  #      then use that name
  # else use the hostname of this machine (with the domain appended,
  #      if given)
  # So...instead of using a user-defined username, maybe we should
  # just use our hostname?
  return MSG_error("modem_cant_lock_pap") if (-e "$Lockdir/pap.secrets");
  # we use sysopen so we can specify the file mode
  sysopen(PAPOUT,"$Lockdir/pap.secrets",O_RDWR()|O_CREAT(),0600)
    || return MSG_error("modem_cant_write_pap", $!);
  open(PAPIN,$Pap_secrets)
    || return MSG_error("modem_cant_read_papsecrets", $Pap_secrets);

  while (<PAPIN>) {
    # remove Cobalt lines
    if( /^$commentLine$/ ) {
      # this line is the account info
      <PAPIN>;
    }
    else {
      print PAPOUT;
    }
  }
  print PAPOUT "$commentLine\n";
  print PAPOUT $modem{'USERNAME'}, " * ", $modem{'PASSWD'}, "\n";
  close PAPIN;
  close PAPOUT;
  rename("$Lockdir/pap.secrets",$Pap_secrets)
    || return MSG_error("modem_cant_save_papsecrets", $!);
  chmod(0600,$Pap_secrets);  # make sure

  # Setup chap-secrets (user,passwd)
  # add/replace last line of chap-secrets to look like:
  #  USERNAME * PASSWD	
  return MSG_error("modem_cant_lock_chap") if (-e "$Lockdir/chap.secrets");
  # use sysopen to set the creation mode
  sysopen(CHAPOUT,"$Lockdir/chap.secrets",O_RDWR()|O_CREAT(),0600)
    || return MSG_error("modem_cant_write_chap", $!);
  open(CHAPIN,$Chap_secrets)
    || return MSG_error("modem_cant_read_chapsecrets", $Chap_secrets);

  while (<CHAPIN>) {
    # remove Cobalt lines
    if( /^$commentLine$/ ) {
      # this line is the account info
      <CHAPIN>;
    }
    else {
      print CHAPOUT;
    }
  }
  print CHAPOUT "$commentLine\n";
  print CHAPOUT $modem{'USERNAME'}, " * ", $modem{'PASSWD'}, "\n";
  close CHAPIN;
  close CHAPOUT;
  rename("$Lockdir/chap.secrets",$Chap_secrets)
    || return MSG_error("modem_cant_save_chapsecrets", $!);
  chmod(0600,$Chap_secrets);

  # Setup chatscript (initstring,phone#)
  #
  # chatscript should look like:
  #
  #  TIMEOUT 5
  #  "" ATZ
  #  OK <init string>
  #  OK ATDT<phone number>
  #  ...etc...
  #
  # set phone number on line that begins with - OK ATDT
  # OK ATDT5551212
  #	
  return MSG_error("modem_cant_chatscript_locked", $Chat_script) if ( -e "$Lockdir/chatscript" );
  open( CHATSLOCK, ">$Lockdir/chatscript" )
    || return MSG_error("modem_cant_write_chatlock", $Lockdir);
  open( CHATS, "$Chat_script")
    || return MSG_error("modem_cant_read_chatscript", $Chat_script);
  while( <CHATS> ) {
    if (/^OK ATDT/) {
      # this is the number
      if ($modem{'INITSTR'} ne "NONE") {
        print CHATSLOCK "OK $modem{'INITSTR'}\n";
      }
      print CHATSLOCK "OK ATDT$modem{'PHONE'}\n";
    }
    elsif (! /^OK /) {
      print CHATSLOCK;
    }
  }
  close CHATSLOCK;
  close CHATS;
  rename( "$Lockdir/chatscript", $Chat_script ) ||
    return MSG_error("modem_cant_save_chatscript", $Chat_script);
  chmod(0755,$Chat_script);

  # set up diald.conf
  #
  # if {'CONNMODE'} = 1 = always connected
  #  -> add 'up' to the first line of /etc/diald.conf
  # if {'CONNMODE'} = 0 = dial-on-demand
  #  -> remove 'up' and 'down' from /etc/diald.conf
  my ($netmasked,$gwed);
  return MSG_error("modem_cant_dialdconf_locked", $Diald_conf) if ( -e "$Lockdir/diald.conf" );
  open( DIALDLOCK, ">$Lockdir/diald.conf" )
    || return MSG_error("modem_cant_write_dialdlock", $Diald_conf);
  open( DIALD, "$Diald_conf")
    || return MSG_error("modem_cant_read_dialdconf", $Diald_conf);
  while (<DIALD>) {
    if ($. == 1) { # first line of the file
      if ($modem{'CONNMODE'}==1) {
	print DIALDLOCK "up\n";   # always connected
      }
    }
    if (/^up$/o || /^down$/o) {
      # do not print this line for either CONNMODE
    }
    # replace line 'netmask xxx.xxx.xxx.xxx' -> 'netmask {'NETMASK'}'
    #  remove line 'netmask xxx.xxx.xxx.xxx' if no NETMASK specified
    elsif (/^netmask/o) {
      if ($modem{'NETMASK'} && ($modem{'NETMASK'} ne "NONE")) {
        print DIALDLOCK "netmask ", $modem{'NETMASK'}, "\n";
        $netmasked = 1;
      }
    }
    # if {'LOCALIP'} = '10.9.25.40'
    #  replace line 'local 10.9.25.40'
    # if {'LOCALIP'} = '192.168.0.1' = dynamic
    #   -> replace line 'local 192.168.0.1' 
    #   -> add line     'dynamic'
    elsif (/^local/o) {
      if ($modem{'LOCALIP'} eq "NONE") {
	print DIALDLOCK "dynamic\nlocal 192.168.0.1\n";
      }
      else {
	print DIALDLOCK "local ", $modem{'LOCALIP'}, "\n";
      }
    }
    elsif (/^dynamic/o) {
      # ignore, we'll re-add it as needed when we hit the local line
    }
    # if {'GWMODE'} = 0 = not the defaultroute
    #   the line 'defaultroute' should not exist in diald.conf
    # if {'GWMODE'} = 1 = yes the defaultroute
    #   the line 'defaultroute' should exist in diald.conf
    elsif (/^defaultroute/o) {
      print DIALDLOCK "defaultroute\n" if ($modem{'GWMODE'});
      $gwed = 1;
    }
    # the 'pppd-options user will' should become
    #   -> 'pppd-options user USERNAME'
    elsif (/^pppd-options/o) {
      # this line could have anything on it, so modify only the username
      $_ =~ s/user [\S]+/user $modem{'USERNAME'}/;
      print DIALDLOCK;
    }
    else {
      print DIALDLOCK;
    }
  }

  # These lines may not be in the configuration file, so add them if we
  # didn't find them.
  if ($modem{'NETMASK'} && ($modem{'NETMASK'} ne "NONE") && ! $netmasked) {
    print DIALDLOCK "netmask ", $modem{'NETMASK'}, "\n"
  }
  if ($modem{'GWMODE'} && ! $gwed) {
    print DIALDLOCK "defaultroute\n";
    # delete any other default routes; update GATEWAYDEV in network setup
    my $ret;
    return $ret if (($ret = net_set_gateway("none", "eth-doug")) !~ /^2/o);
  }
  close DIALDLOCK;
  close DIALD;
  rename( "$Lockdir/diald.conf", $Diald_conf )
    || return MSG_error("modem_cant_save_dialdconf", $Diald_conf);

  # handle modem not used
  if ($modem{'CONNMODE'}==2) {
    # deactivate diald, activate kernel messages to console
    system("$Diald_script stop >>$Sauce_log 2>&1") 
      && return MSG_error("modem_cant_stop", $!); 
    unlink($Diald_link,$Diald_stop0,$Diald_stop6);
    # klogd args
    return MSG_error("modem_cant_syslog_locked") if (-e "$Lockdir/syslog");
    open(SYSLOK,">$Lockdir/syslog") 
      || return MSG_error("modem_cant_write_syslog", $!);
    open(SYSLOG,$Syslog_script) || return MSG_error("modem_cant_read_syslog", $!);
    while (<SYSLOG>) {
      if (/^\s*daemon klogd/o) {
	s/daemon klogd.*$/daemon klogd -c 1/o;
      }
      print SYSLOK;
    }
    close SYSLOG;
    close SYSLOK;
    rename("$Lockdir/syslog",$Syslog_script) 
      || return MSG_error("modem_cant_update_syslog", $!);
    chmod(0755,$Syslog_script);

    # restart syslog
    system("$Syslog_script stop >>$Sauce_log 2>&1")
      && return MSG_error("modem_cant_stop_syslog", $!); 
    system("$Syslog_script start >>$Sauce_log 2>&1")
      && return MSG_error("modem_cant_start_syslog", $!); 
    
    return MSG_ok("modem_ok_set_config");
  }

  # klogd args
  return MSG_error("modem_cant_syslog_locked") if (-e "$Lockdir/syslog");
  open(SYSLOK,">$Lockdir/syslog") 
    || return MSG_error("modem_cant_write_syslog", $!);
  open(SYSLOG,$Syslog_script) || return MSG_error("modem_cant_read_syslog", $!);
  while (<SYSLOG>) {
    if (/^\s*daemon klogd/o) {
      s/daemon klogd.*$/daemon klogd/o;
    }
    print SYSLOK;
  }
  close SYSLOG;
  close SYSLOK;
  rename("$Lockdir/syslog",$Syslog_script) 
    || return MSG_error("modem_cant_update_syslog", $!);
  chmod(0755,$Syslog_script);
  system("$Diald_script stop >>$Sauce_log 2>&1") 
    && return MSG_error("modem_cant_stop", $!); 
  system("$Diald_script start >>$Sauce_log 2>&1")
    && return MSG_error("modem_cant_start", $!); 
  symlink($Diald_script,$Diald_link);
  symlink($Diald_script,$Diald_stop0);
  symlink($Diald_script,$Diald_stop6);

  # restart syslog
  system("$Syslog_script stop >>$Sauce_log 2>&1")
    && return MSG_error("modem_cant_stop_syslog", $!); 
  system("$Syslog_script start >>$Sauce_log 2>&1")
    && return MSG_error("modem_cant_start_syslog", $!); 

  return MSG_ok("modem_ok_set_config");
}

sub modem_get_dialmode
# Descr:        Get the modem dialing mode 
# Arguments:    A reference to a hash containing all the modem settings
# Return:       0 for not used
#		1 for dial out only
#		2 for dial in only
#		3 for dial in and dial out 
# Ex:           $dialMode=Cobalt::Modem::modem_get_dialmode( $modemRef );
{

        my($modemRef_in) = @_;
        return $$modemRef_in{'DIALMODE'};

}

sub modem_set_dialmode
# Descr:        Set the modem dialing mode 
# Arguments:    A reference to a hash containing all the modem settings
#               An integer indicating the dialing mode
# 	        0 for not used
#               1 for dial out only
#               2 for dial in only
#               3 for dial in and dial out 
# Return:       Return code like MSG_ok("2000")
# Ex:           $ret=Cobalt::Modem::modem_set_dialmode( $modemRef, 1 );
{

        my($modemRef,$dialmode_in) = @_;
        $$modemRef{'DIALMODE'} = $dialmode_in;

        return MSG_ok("modem_ok_dialmode");

}

sub modem_get_model
# Descr:        Get the modem model
# Arguments:    A reference to a hash containing all the modem settings
# Return:       A string containing the model
# Ex:           $modemModel=Cobalt::Modem::modem_get_model( $modemRef );
{

        my($modemRef_in) = @_;
        return $$modemRef_in{'MODEL'};

}

sub modem_set_model
# Descr: 	Set the modem model
# Arguments:	A reference to a hash containing all the modem settings
#		a string containing the model
# Return: 	Return code like MSG_ok("2000")
# Ex: 		$ret=Cobalt::Modem::modem_set_model( $modemRef, 'hayes generic' );
{

	my($modemRef,$model_in) = @_;
	$$modemRef{'MODEL'} = $model_in;

	return MSG_ok("modem_ok_model");	

}
	
sub modem_get_connmode
# Descr:        Get the modem connection mode
# Arguments:    A reference to a hash containing all the modem settings
# Return:       0 for dial on demand
#               1 for always connected 
#		2 never connected (diald off)
# Ex:           $connMode=Cobalt::Modem::modem_get_connmode( $modemRef );
{

        my($modemRef_in) = @_;
        return $$modemRef_in{'CONNMODE'};

}

sub modem_set_connmode
# Descr:        Set the modem connection mode
# Arguments:    A reference to a hash containing all the modem settings
#               An integer indicating the connection mode
#               0 for dial on demand 
#               1 for always connected 
#		2 never connected (diald off)
# Return:       Return code like MSG_ok("2000")
# Ex:           $ret=Cobalt::Modem::modem_set_connmode( $modemRef, 1 );
{

        my($modemRef,$connmode_in) = @_;
        $$modemRef{'CONNMODE'} = $connmode_in;

        return MSG_ok("modem_ok_connmode");

}


sub modem_get_dialoutacct
# Descr: 	Get the dial out account settings
# Arguments:	A reference to a hash containing all the modem settings
# Return:	A string containing the phone number
#		A string containing the user name
#		A string containing the password
# Ex:		(phoneNum,userName,password) = Cobalt::Modem::modem_get_dialoutacct( $modemRef );
#
{

	my($modemRef_in) = @_;
	return ($$modemRef_in{'PHONE'},$$modemRef_in{'USERNAME'},$$modemRef_in{'PASSWD'});


}


sub modem_set_dialoutacct
# Descr:        Set the dial out account settings
# Arguments:    A reference to a hash containing all the modem settings
# 	        A string containing the phone number
#               A string containing the user name
#               A string containing the password 
# Return:	return code like 2000 OK`		
# Ex:           $ret = Cobalt::Modem::modem_set_dialoutacct( $modemRef,'650-9302500','kevin','secret');
#
{

        my($modemRef,$phone_in,$user_in,$pass_in) = @_;
        $$modemRef{'PHONE'} = $phone_in;
        $$modemRef{'USERNAME'} = $user_in;
        $$modemRef{'PASSWD'} = $pass_in;

        return MSG_ok("modem_ok_account");
} 

sub modem_get_localip
# Descr:        Get the local IP address and netmask 
# Arguments:    A reference to a hash containing all the modem settings
# Return:       A string containing the IP address 
#               A string containing the netmask
# Ex:           ($ipAddr, $netmask) = Cobalt::Modem::modem_get_localip( $modemRef );
#       LOCALIP of 192.168.0.1 implies using dynamic address
#
{

        my($modemRef_in) = @_;
        return ($$modemRef_in{'LOCALIP'},$$modemRef_in{'NETMASK'});


}


sub modem_set_localip
# Descr:        Set the local IP address and netmask 
# Arguments:    A reference to a hash containing all the modem settings
#               A string containing the IP Address 
#               A string containing the netmask
# Return:       return code like 2000 OK`
# Ex:           $ret = Cobalt::Modem::modem_set_localip( $modemRef,'10.9.25.238','255.255.255.0');
#
{

        my($modemRef,$localip_in,$netmask_in) = @_;
        $$modemRef{'LOCALIP'} = $localip_in;
        $$modemRef{'NETMASK'} = $netmask_in;

        return MSG_ok("modem_ok_ip");


}

sub modem_get_gatewaymode
# Descr:        Find out if modem should be default gateway 
# Arguments:    A reference to a hash containing all the modem settings
# Return:       An integer indicating if the modem connection is used as the default gateway 
#		0 means no 
#		1 means yes, modem is default gateway
# Ex:           $gatewayMode = Cobalt::Modem::modem_get_gatewaymode( $modemRef );
#
{

        my($modemRef_in) = @_;
        return ($$modemRef_in{'GWMODE'});


}


sub modem_set_gatewaymode
# Descr:        Set the mode of using the modem as the default gateway or not 
# Arguments:    A reference to a hash containing all the modem settings
#		An integer indicating if the modem connection is used as the default gateway               
#               0 means no
#               1 means yes, modem is default gateway
# Return:       return code like 2000 OK`
# Ex:           $ret = Cobalt::Modem::modem_set_gatewaymode( $modemRef, 1);
#
{

        my($modemRef,$gatewaymode_in) = @_;
        $$modemRef{'GWMODE'} = $gatewaymode_in;

        return MSG_ok("modem_ok_gwmode");
}

sub modem_get_initstr
# Descr:        Get the modem initstring 
# Arguments:    A reference to a hash containing all the modem settings
# Return:       A string containing the initstring  
# Ex:           $modemModel=Cobalt::Modem::modem_get_model( $modemRef );
{

        my($modemRef_in) = @_;
        return $$modemRef_in{'INITSTR'};

}

sub modem_set_initstr
# Descr:        Set the modem initstring 
# Arguments:    A reference to a hash containing all the modem settings
#               a string containing the initstring 
# Return:       Return code like MSG_ok("2000")
# Ex:           $ret=Cobalt::Modem::modem_set_model( $modemRef, 'ATZ' );
{

        my($modemRef,$initstr_in) = @_;
        $$modemRef{'INITSTR'} = $initstr_in;

        return MSG_ok("modem_ok_initstring");

}

sub modem_is_online
# Descr:        Determine if the ppp link is up 
# Arguments:    None 
# Return:       1 if the ppp0 link is up
#		0 if the ppp0 link is down 
# Ex:           $modemOnline=Cobalt::Modem::modem_is_online();
{

	my $online = 0;

  open(IFCFG,"$Ifconfig_cmd |")
    || return MSG_error("modem_cant_ifconfig", $Ifconfig_cmd);
  while (<IFCFG>)
    {
      chomp;
      if (/^ppp0      Link encap:Point-to-Point Protocol/o)
        {
		$online = 1;
		last;
        }
    }	
	close IFCFG;
        return $online;

}

1;
