#
# $Id: Netutil.pm,v 1.28 1999/10/02 16:44:26 duncan Exp $
#
# author: Kevin K.M. Chiu and Brion Moss
#       Cobalt Networks
# version: 1.0
# date: 10-30-97
#
# Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Netutil;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(netutil_test_macaddr
	     netutil_test_emailaddr
	     netutil_test_netaddr
	     netutil_test_domainname
	     netutil_test_ipaddr
	     netutil_test_network_in_network
	     netutil_get_network
	     netutil_get_hostip
	     netutil_get_fulldomain
	     netutil_ping
	     netutil_test_dns_server
	     netutil_get_mx);

require Cobalt::Network;
import Cobalt::Network qw(net_get_dnsclient net_get_hostname net_get_ipaddr);

use Cobalt::Util;
use Cobalt::Product;

use Socket qw(AF_INET PF_INET SOCK_STREAM);
#use strict;
#use Cobalt::Paths;

1;
__DATA__

sub netutil_test_macaddr
# tests to see if the given MAC address has valid syntax
# argument 0 is a MAC address
# return 0 ( false ) or 1 ( true )
{
    my ( $mac )=@_;

    if( $mac=~/^[0-9A-Fa-f]{2}\:[0-9A-Fa-f]{2}\:[0-9A-Fa-f]{2}\:[0-9A-Fa-f]{2}\:[0-9A-Fa-f]{2}\:[0-9A-Fa-f]{2}$/ ) { return 1; }

    return 0;
}

sub netutil_test_emailaddr
# tests to see if the given email address has valid syntax
# argument 0 is an email address
# return 0 ( false ) or 1 ( true )
{
    my( $emailAddr )=@_;

    my $userName;
    my $domainName;

    if( $emailAddr=~/([^\@]+)\@([^\@]+)/ ) {
	$userName=$1;
	$domainName=$2;
    }
    else {
	$userName=$emailAddr;
    }

    if( $userName!~/^[A-Za-z0-9\.\-\_]+$/o ) { return 0; }

    if( !$domainName ) { return 1; }

    return Cobalt::Netutil::netutil_test_domainname( $domainName );
}


sub netutil_test_netaddr
# argument 0 is a network address (domain name or IP address)
# argument 1 is true if the network address can be a network IP instead of a host IP (optional)
# return 0 ( false ) or 1 ( true )
{
    if( $_[ 0 ]=~/^[0-9\.]*$/ ) { return Cobalt::Netutil::netutil_test_ipaddr( $_[ 0 ], $_[ 1 ] ); }
    else { return Cobalt::Netutil::netutil_test_domainname( $_[ 0 ] ); }
}

sub netutil_test_domainname
# argument 0 is a domain name
# return 0 ( false ) or 1 ( true )
{
    # check if ends with '.'
    # technically, a domain name that ends in '.' is just fine;
    # the '.' is just a flag to the resolver telling it not to tack
    # a domain name on the end.  But, *we* don't accept the trailing '.'
    # Using substr seems like a lot of trouble to go to...
    #if( substr( $_[ 0 ], -1, 1 )=~/\./o ) { return 0; }
    #return 0 if( $_[ 0 ] =~ /\..?$/o );  # end in . or .a

    # Everybody including JPNIC violates RFCs
    # Therefore we allow 1 character subdomain
    # check for blank or 1-letter domain/subdomain names
#    return 0 if ($_[0] =~ /\..(\.|$)/o);

    # dots and dashes cannot stick together or end a name
    return 0 if( $_[0] =~ /(\.|\-)(\.|\-|$)/o );

    # domain can only have letters, digits and dashes, seperated by .'s,
    # and must begin with an alphanumeric character
    return 0 if( $_[0] !~ /^[a-z0-9][a-zA-Z0-9\-\.]*$/ );

    # an all numeric domain is not valid
    return 0 if( $_[ 0 ] =~ /^[0-9\.]+$/ );

    return 1;

#    my @subdomains=split /\./o, $_[ 0 ];
#    if( $#subdomains<0 ) { $subdomains[ 0 ]=$_[ 0 ]; }

#    my $subdomain;
#    foreach $subdomain ( @subdomains ) {
	# subdomain cannot be nothing or less than 2 characters
	# Of course, in practice you have things like www.x.org, which 
	# violates the RFC but exists anyway.  If we wanted to support
	# RFC-violation (we already do by allowing domains which begin with
	# number) use the second line instead of the first to just check
	# that the hostname is not null.
	#return 0 if( length( $subdomain )<=1 );
#	return 0 unless ($subdomain);

        # 1st character must be an alphanumeric
	# RFC 822 says it must be an alphabetic character, but our 
        # friends at Candlestick Park don't like it
#        if( $subdomain!~/^[a-z0-9]/o ) { return 0; }

        # 2 dashes cannot stick together
#        if( $subdomain=~/--/o ) { return 0; }

        # domain can only have letters, digits and dashes
#        if( $subdomain!~/^[a-z0-9\-]+$/ ) { return 0; }
#    }

#    return 1;
}

sub netutil_test_ipaddr
# argument 0 is an IP address
# argument 1 is true if the IP address can be a network IP instead of a host IP (optional)
# return 0 ( false ) or 1 ( true )
{
    my @numbers=split /\./o, $_[ 0 ];

    if( !$_[ 1 ] ) {
	# a valid host IP has 4 numbers
	if( $#numbers!=3 ) { return 0; }
    }

    my $num;
    foreach $num ( @numbers ) {
        # check length
        my $len=length $num;
        if( $len<1 || $len>3 ) { return 0; }

        # check digit
        if( $num=~/\D/o ) { return 0; }

        #check number limit
        if( $num>255 ) { return 0; }
    }

    if( !$_[ 1 ] ) {
	# network 0 is not valid
	if( $numbers[ 0 ]==127 ) { return 0; }

	# loopback address is not valid
	if( $numbers[ 0 ]==127 && $numbers[ 1 ]==0 && $numbers[ 2 ]==0 && $numbers[ 3 ]==1 ) { return 0; }
    }

    return 1;
}

sub netutil_test_network_in_network
# Test if one IP network is a subset of another network
#   This is not only for testing if an address is on a network--
#   it also tests wether a subnet is within a larger subnet
#   (e.g., 1.2.3.0/24 is a subnet of 1.2.0.0/16)
#   (generally: is the first network within the second?)
# argument 0 is the IP address of the first network
# argument 1 is the netmask of the first network
# argument 2 is the IP address of the second network
# argument 3 is the netmask of second network
# IP addresses do not have to have all 4 octets; any octets that are not
#   defined will be assumed to be 0 (e.g., "10.10.0.0" is equivalent
#   to "10.10".)
# The netmask can be given either in dotted quad (255.255.0.0) or in 
#   decimal (16).  If in dotted quad, all four quads must be defined
#   (e.g., "255.255" is not acceptable--you must use "255.255.0.0")
# Return value: 1 if the first network is entirely within the second
#   network or if the two networks are the same; 0 otherwise.
#   Note that if the second network is entirely within the first,
#   we will return 0.
# Example: netutil_test_network_in_network("192.168.45.0",24,"192.168.0.0",16);
{
    my ($ip1, $netmask1, $ip2, $netmask2) = @_;

    # convert the IP addresses from dotted quad (e.g., 192.168.123.45) 
    # to binary (e.g., 11000000101010000111101100101101)
    $ip1 = unpack('B32',pack('C4',split('\.',$ip1)));
    $ip2 = unpack('B32',pack('C4',split('\.',$ip2)));

    # If the netmask is given in dotted quad, convert to decimal
    $netmask1 = Cobalt::Network::net_convert_netmask($netmask1) 
      if ($netmask1 =~ /\./o);
    $netmask2 = Cobalt::Network::net_convert_netmask($netmask2)
      if ($netmask2 =~ /\./o);

    # chop off the bits according to the netmask; if the netmask
    # is 32, then no chopping is done (it's exactly that IP)
    # (To return all but the last character, use substr($string, 0, -1))
    $ip1 = substr($ip1,0,$netmask1) if ($netmask1 != 32);
    $ip2 = substr($ip2,0,$netmask2) if ($netmask2 != 32);

    # Now do the comparisons...
    if ($netmask1 >= $netmask2) {
      # the second network is as large or larger than the first
      # (a smaller subnet mask means a larger network)
      return 1 if ($ip1 =~ /^$ip2/);
    }
    # The following test would be if we wanted to check if the second
    # network is within the first...but we don't do that.
    #else {
    #  # the first network is larger (smaller mask)
    #  return -1 if ($ip2 =~ /^$ip1/);
    #}
    return 0;
}

sub netutil_get_network
# apply the netmask on the IP address to get the network IP address
# argument 0 is the IP address of the network
# argument 1 is the netmask of the network
# IP addresses not necessary have 4 octets
# netmask must have 4 octets
# return the IP address of the network
{
    my( $ipAddr, $netmask )=@_;

    my @ipAddrNums=split /\./, $ipAddr;
    my @netMaskNums=split /\./, $netmask;

    my $i;
    for( $i=0; $i<4; $i++ ) {
	# bitwise apply the netmask
	$ipAddrNums[ $i ]=( $ipAddrNums[ $i ]+0 ) & ( $netMaskNums[ $i ]+0 );
    }

    return join '.', @ipAddrNums;
}

sub netutil_get_hostip
# apply the netmask on the IP address to get the host IP address
# argument 0 is the IP address
# argument 1 is the netmask of the network
# IP addresses not necessary have 4 octets
# netmask must have 4 octets
# return the host IP address
{
    my( $ipAddr, $netmask )=@_;

    my @ipAddrNums=split /\./, $ipAddr;
    my @netMaskNums=split /\./, $netmask;

    my $i;
    for( $i=0; $i<4; $i++ ) {
	# bitwise apply the netmask
	$ipAddrNums[ $i ]=( $ipAddrNums[ $i ]+0 ) & ~( $netMaskNums[ $i ]+0 );
    }

    return join '.', @ipAddrNums;
}

sub netutil_get_fulldomain
#
# return full domain name, or IP address if host or domain name is missing
#
{
    my $hostName = Cobalt::Network::net_get_hostname();
    my $domainName=( Cobalt::Network::net_get_dnsclient() )[ 0 ];
    if ($hostName && $domainName)
    { 
	($hostName !~ /$domainName$/) ?
	    (return "$hostName.$domainName") :
		(return $hostName);
    }
    else
    {
	return ( Cobalt::Network::net_get_ipaddr() )[ 0 ];
    }
}

sub netutil_ping
# Check to see if a host is on the network
# Arguments: an IP address
# Return value: a string of the form "656 all is well"
# Side effects: runs ping
{
    my ($ip) = @_;
    my $IP = $ip;
    my $timeout = 3;
    eval 'use Net::Ping';
    my $pingthing = Net::Ping->new("icmp");
    my $status = $pingthing->ping($ip,$timeout);
    $pingthing->close();
    
    ($status)   ? (return MSG_ok("2470", $ip))
                : (return MSG_error("4301", $ip));
}

sub netutil_test_dns_server
# Check to see if a system responds to DNS requests
# Arguments: an IP address
# Return value: a string of the form "814 can't resolve the onion"
{
    my ($ip) = @_;
    my $IP = $ip;
    my $dns_port = 53;
    my $timeout = 3;
    my ($packedip);
    local (*SSOCK);
    $packedip = pack('S n a4 x8', AF_INET(), $dns_port, 
                     pack ('C4', split (/\./o, $ip)));
    $SIG{'ALRM'} = sub { close SSOCK; return MSG_error("4302"); } ;
    alarm($timeout);
    unless (socket(SSOCK,PF_INET(),SOCK_STREAM(),getprotobyname('tcp')))
    {
        close(SSOCK);
        return MSG_error("4303", $!);
    }
    unless (connect(SSOCK, $packedip))
    {
        close(SSOCK);
        return MSG_error("4304", $!);
    }
    alarm(0);
    $SIG{'ALRM'} = 'DEFAULT';
    close(SSOCK);
    return MSG_ok("2480", $ip);
}

sub netutil_get_mx
# Get MX records for a host/domain
# Arguments: a host/domain name
#   return type (optional): 0 for IP address, 1 for name
# Return value: an array of MX records sorted by preferrence
#   (from numerically lower to numerically higher)
{
    my( $name, $returnName )=@_;

    my %names;
    my %preferenceToName;
    my %nameToIP;

    open( NSLOOKUP, "echo \"set type=mx\n$name.\n\" | $Nslookup 2>&1 |" );
    while( <NSLOOKUP> ) {
	# get names and preferrences
	if( /preference = (\d+), mail exchanger = (.+)/ ) {
	    # DNS server can have 1 mail exchanger with different preferences
	    if( !$names{ $2 } ) { $preferenceToName{ $1 }=$2; }
	    $names{ $2 }=1;
	}

	# get IPs
	if( /(\S+)\s+internet address = (.+)/ ) {
	    $nameToIP{ $1 }=$2;
	}
    }
    close( NSLOOKUP );

    # sort by preference in ascending order
    my @preferences=keys %preferenceToName;
    @preferences=sort { $a <=> $b } @preferences;

    my @returnValues;
    my $preference;
    foreach $preference ( @preferences ) {
	if( $returnName ) {
	    push @returnValues, $preferenceToName{ $preference };
	}
	else {
	    push @returnValues, $nameToIP{ $preferenceToName{ $preference } };
	}
    }

    return @returnValues;
}

#
# not supported
#
#sub isMXExist
# Check to see if a MX record for a host/domain exists
# Arguments: a host/domain name
# Return value: a string of the form '2000 ready'
#{
#    my ( $name )=@_;

#    open( NSLOOKUP, "echo \"set type=mx\n$name.\n\" | $Nslookup 2>&1 |" );
#    while( <NSLOOKUP> ) {
#	if( /\*\*\*.+can't find/ ) { return 0; }
#    }
#    close( NSLOOKUP );

#    return 1;
#}

1;
