#
# Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Quota;
use Quota;
use vars qw(@ISA @EXPORT @EXPORT_OK);

require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(setqlim repquota);

use Cobalt::Product;
use Cobalt::Util;

sub setqlim
# Set the quota for a user or group
# Arguments: path on which to set quotas, uid, soft limit, hard limit,type
#   (type = 0 for users, 1 for groups)
# Side effects: modifies the quota tables
{
    my ($path,$uid,$soft,$hard,$type) = @_;
    my ($ret,$dev);
    return MSG_error("quota_unsupported_noquotas") unless ($Features{'quotas'});

    my $typename = ($type ? "group" : "user");

    $dev = Quota::getqcarg($path);

    return MSG_error("quota_cant_sync_user_quota", $uid) if (Quota::sync($dev) && ($! != 1));

    my($bc,$bs,$bh,$bt,$fc,$fs,$fh,$ft) = Quota::query($dev,$uid,$type);
    return MSG_error("quota_cant_get_user_quota", $uid) unless (defined($bc));

    Quota::setqlim($dev,$uid,$hard,$soft,0,0,1,$type) &&
	return MSG_error("quota_cant_set_user_quota", $uid);

    return MSG_error("quota_cant_sync_user_quota", $uid) if (Quota::sync($dev) && ($! != 1));
    return MSG_ok("quota_ok_user_set", $uid);
}

sub repquota
# Report on the quota for a user or group
# Arguments: user/group name and type (type=0 for user, 1 for group)
#   Optional args: ID, homedir
#   If ID and homedir are not given, read them from the password or
#   group file.
# Return value: array: (used, soft limit, hard limit)
# In practice, hard limit and soft limit will always be the same
# Example: ($used,$soft,$hard) = Cobalt::Quota::repquota("john",0,9124,"/home/users/john");
{
    my ($user,$type,$uid,$home) = @_;
    return MSG_error("quota_unsupported_noquotas") unless ($Features{'quotas'});
    unless ($uid) {
      if ($type)
      {
	($uid) = (getgrnam($user))[2];
	$home = "$Htgroup_dir/$user";
      }
      else
      {
	($uid,$home) = (getpwnam($user))[2,7];
      }
    }

    my $dev = Quota::getqcarg($home);
    return MSG_error("quota_cant_get_user_quota", $uid) unless $dev;

    my($bc,$bs,$bh,$bt,$fc,$fs,$fh,$ft) = Quota::query($dev,$uid,$type);
    
    return ($bc,$bs,$bh);
}

1;
