#
# $Id: Repair.pm,v 1.41 1999/10/02 16:44:27 duncan Exp $
#
# Attempt to repair things when they break
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Repair;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(repair_examine repair_fix repair_notify);

require Cobalt::Admserv;
import Cobalt::Admserv qw(:DEFAULT);
require Cobalt::Group;
import Cobalt::Group qw(:DEFAULT);
require Cobalt::Language;
import Cobalt::Language qw(:DEFAULT);
require Cobalt::Modem;
import Cobalt::Modem qw(modem_get_settings modem_get_connmode);
require Cobalt::Monitor;
import Cobalt::Monitor qw(:DEFAULT);
require Cobalt::Network;
import Cobalt::Network qw(:DEFAULT);
require Cobalt::Services;
import Cobalt::Services qw(:DEFAULT);
require Cobalt::User;
import Cobalt::User qw(:DEFAULT);
use Cobalt::Util;
use Cobalt::Product;

#use Cobalt::Paths;
#use strict;

1;
__DATA__

sub repair_examine
# Generic function for checking the state of a subsystem
# Arguments: Service name
# Side effects: Does its best to see if that service is working properly
#    Updates the database with the result.
# Return value: none
{
    my ($service) = @_;
    print STDERR "Checking the state of $service\n" if ($Debug);

    # Get our IP address so we know what address to connect to
    my $myipaddr = (Cobalt::Network::net_get_ipaddr($Netif))[0];

    # Big nasty case statement; do the right thing for whatever service
    # we're examining.
    if    ($service eq "adm")
      {
	# Just see if the port answers
	Cobalt::Monitor::mon_portwatch("adm","$myipaddr",$AdminServer_port,"adm");
      }
    elsif ($Features{'atalk'} && ($service eq "atalk"))
      {
        if (-e "$Subsys/atalk")	# check if it's all the way up
	  {
	    Cobalt::Monitor::mon_atalktest();	# check using nbplkup
	    Cobalt::Monitor::mon_procwatch("atalkd","atalk"); # verify the procs are there
	    Cobalt::Monitor::mon_procwatch("afpd","atalk");
	  }
	else
	  {
	    mon_set_state('atalk','PRIME',0,'-','-','-','-');
	    mon_set_state('atalk','nbplkup',0,'-','-','-','-');
	    mon_set_state('atalk','atalkd',0,'-','-','-','-');
	    mon_set_state('atalk','afpd',0,'-','-','-','-');
	  }
      }
    elsif ($service eq "cpu")
      {
	Cobalt::Monitor::mon_loadwatch($LoadYel,$LoadRed); # Check load average
      }
    elsif ($service eq "crond")
      {
	Cobalt::Monitor::mon_procwatch("crond","crond");
	Cobalt::Monitor::mon_cronwatch();
      }
    elsif ($service eq "custo")
      {
	Cobalt::Monitor::mon_procwatch("custodiat","custo");
      }
    elsif ($service eq "modem")
      {
	# Check that the modem is doing its thing
	my %modem_settings = Cobalt::Modem::modem_get_settings();
	my $conn_mode = Cobalt::Modem::modem_get_connmode(\%modem_settings);
	if ($conn_mode != 2) {
	  Cobalt::Monitor::mon_modem();
	}
	else {
	  # register the modem as off
	  mon_set_state('modem','PRIME',0,'-','-','-','-');
	  mon_set_state('modem','diald',0,'-','-','-','-');
	}
      }
    elsif ($service eq "filesystem")
      {
	# Check the overall usage for each partition
	Cobalt::Monitor::mon_diskwatch("/",$DiskYel,$DiskRed);
	Cobalt::Monitor::mon_diskwatch("/var",$DiskYel,$DiskRed);
	Cobalt::Monitor::mon_diskwatch("/home",$DiskYel,$DiskRed);
      }
    elsif (($Features{'users'}) && ($service eq "userspace"))
      {
	# Check the usage for each user (quota and mail spool)
	my $name;
	Cobalt::Monitor::mon_quotawatch("*USERS");
	#foreach $name (Cobalt::User::user_list()) {
	#Cobalt::Monitor::mon_quotawatch($name,$UsrQuota,$QuotYel,$QuotRed);
	#Cobalt::Monitor::mon_spoolwatch($name) if (-e "$Mailspool/$name");
	#}
      }
    elsif (($Features{'groups'}) && ($service eq "groupspace"))
      {
	# Check the quota usage for each group
	my ($name);
	Cobalt::Monitor::mon_quotawatch("*GROUPS");
	#foreach $name (Cobalt::Group::group_list()) {
	#Cobalt::Monitor::mon_quotawatch($name,$GrpQuota,$QuotYel,$QuotRed);
	#}
      }
    elsif ($Features{'dhcpd'} && ($service eq "dhcpd"))
      {
	# Check that the DHCP server is running
	if (Cobalt::Services::get_dhcp_server_on())
	  {
	    Cobalt::Monitor::mon_procwatch("dhcpd","dhcpd");
	  }
	else
	  {
            mon_set_state('dhcpd','PRIME',0,'-','-','-','-');
            mon_set_state('dhcpd','dhcpd',0,'-','-','-','-');
	  }
      }
    elsif ($Features{'dns'} && ($service eq "dns"))
      {
	if (Cobalt::Services::get_dns_server_on())
	  {
	    Cobalt::Monitor::mon_portwatch("dns","$myipaddr","53","dns");
	    Cobalt::Monitor::mon_procwatch("named","dns");
	  }
        else
        {
            mon_set_state('dns','PRIME',0,'-','-','-','dns');
            mon_set_state('dns','dns',0,'-','-','-','-');
            mon_set_state('dns','named',0,'-','-','-','-');
        }
      }
    elsif ($service eq "inetd")
      {
	Cobalt::Monitor::mon_procwatch("inetd","inetd");
      }
    elsif ($service eq "lcd")
      {
	Cobalt::Monitor::mon_procwatch("lcdsleep","lcd"); # LCD panel process
      }
    elsif ($Features{'mail'} && ($service eq "mail"))
      {
	if (Cobalt::Services::get_mail_server_on())
	  {
	    Cobalt::Monitor::mon_portwatch("mail","$myipaddr","110","pop3");
	    Cobalt::Monitor::mon_portwatch("mail","$myipaddr","143","imap");
	    Cobalt::Monitor::mon_portwatch("mail","$myipaddr","25","smtp");
	  }
        else
        {
            mon_set_state('mail','PRIME',0,'-','-','-','smtp');
            mon_set_state('mail','pop3',0,'-','-','-','-');
            mon_set_state('mail','imap',0,'-','-','-','-');
            mon_set_state('mail','smtp',0,'-','-','-','-');
        }
      }
    elsif ($service eq "mem")
      {
	Cobalt::Monitor::mon_swapwatch($SwapYel,$SwapRed);
	Cobalt::Monitor::mon_memwatch($VmemYel,$VmemRed);
      }
    elsif ($service eq "net")
      {
	Cobalt::Monitor::mon_collwatch($CollYel,$CollRed);
      }
    elsif ($Features{'nfs'} && ($service eq "nfs"))
      {
	# NFS is not, technically, supported
	if (Cobalt::Services::get_nfs_server_on())
	  {
	    Cobalt::Monitor::mon_nfstest();
	    Cobalt::Monitor::mon_procwatch("rpc.nfsd","nfs");
	    Cobalt::Monitor::mon_procwatch("rpc.mountd","nfs");
	  }
        else
        {
            mon_set_state('nfs','PRIME',0,'-','-','-','-');
            mon_set_state('nfs','rpc.nfsd',0,'-','-','-','-');
            mon_set_state('nfs','rpc.mountd',0,'-','-','-','-');
        }
      }
    elsif (($service eq "ntp") && $Features{'ntp'})
      {
	if (Cobalt::Services::get_ntp_server_on())
	  {
	    Cobalt::Monitor::mon_procwatch("ntpd","ntp");
	  }
       else
       {
        mon_set_state('ntp','PRIME',0,'-','-','-','-');
        mon_set_state('ntp','ntpd',0,'-','-','-','-');
       }
      }
    elsif ($service eq "portmap")
      {
	# Only used if we're doing NFS (or NIS!?!)
	if (Cobalt::Services::get_nfs_server_on())
	  {
	    Cobalt::Monitor::mon_procwatch("portmap","portmap");
	  }
        else
        {
            mon_set_state('portmap','PRIME',0,'-','-','-','-');
            mon_set_state('portmap','portmap',0,'-','-','-','-');
         }
      }
    elsif ($Features{'smb'} && ($service eq "smb"))
      {
	if (Cobalt::Services::get_smb_server_on())
	  {
	    Cobalt::Monitor::mon_procwatch("smbd","smb");
	    Cobalt::Monitor::mon_procwatch("nmbd","smb");
	    Cobalt::Monitor::mon_portwatch("smb","$myipaddr","139","smb");
	  }
        else
        {
            mon_set_state('smb','PRIME',0,'-','-','-','smb');
            mon_set_state('smb','smb',0,'-','-','-','-');
            mon_set_state('smb','smbd',0,'-','-','-','-');
            mon_set_state('smb','nmbd',0,'-','-','-','-');
        }
      }
    elsif ($Features{'snmp'} && ($service eq "snmp"))
      {
	if (Cobalt::Services::get_snmp_server_on())
	  {
	    Cobalt::Monitor::mon_procwatch("snmpd","snmp");
	  }
       else
       {
         mon_set_state('snmp','PRIME',0,'-','-','-','-');
         mon_set_state('snmp','snmpd',0,'-','-','-','-');
       }
      }
    elsif ($Features{'ssl'} && ($service eq "ssl"))
      {
	Cobalt::Monitor::mon_portwatch("ssl","$myipaddr","443","ssl");
      }
    elsif ($service eq "telnet")
      {
	if (Cobalt::Services::get_telnet_server_on())
	  {
	    Cobalt::Monitor::mon_telnettest("$myipaddr");
	  }
       else
       {
         mon_set_state('telnet','PRIME',0,'-','-','-','telnet');
         mon_set_state('telnet','telnet',0,'-','-','-','-');
         mon_set_state('telnet','binary',0,'-','-','-','-');
         mon_set_state('telnet','inetd',0,'-','-','-','-');
       }
      }
    elsif ($service eq "www")
      {
	Cobalt::Monitor::mon_portwatch("www","$myipaddr","www","www");
      }
    elsif ($service eq "ftp")
      {
	# ftp is usually run from inetd, so generally needn't be
	# checked by itself.  We just check it for the UI's sake,
	# or in case we're running it standalone
	if (Cobalt::Services::get_ftp_server_on())
	  {
	    Cobalt::Monitor::mon_portwatch("ftp","$myipaddr","ftp","ftp");
	  }
       else
       {
         mon_set_state('ftp','PRIME',0,'-','-','-','ftp');
         mon_set_state('ftp','ftp',0,'-','-','-','-');
       }
      }
    else
      {
	# Don't print a message unless debugging is on, because
	# we may just be trying to check on an unsupported
	# service
	print STDERR "Unsupported service $service\n" if ($Debug);
      }
}

sub repair_fix
# Try to diagnose and fix problems
# Arguments: service name
# Side effects: attempts repairs
# Return value: status string
{
    my ($service) = @_;

    # Another big case statement...do the deed for the server we have
    if ($service eq "dns")        {
      return Cobalt::Repair::repair_generic($service,$Named_exe,$Named_boot,
			    $Named_script,0);    }
    elsif ($service eq "custo")   {
      return Cobalt::Repair::repair_generic("custo",0,0,$Custo_script,0);   }
    elsif ($service eq "crond")   {
      return Cobalt::Repair::repair_generic("crond",0,$Crontab,$Cron_script,0);   }
    elsif ($service eq "dhcpd")   {
      return Cobalt::Repair::repair_generic("dhcpd",0,$Dhcpd_conf,$Dhcpdscript,0); }
    elsif ($service eq "inetd")   {
      return Cobalt::Repair::repair_generic("inetd",0,$Inetd_conf,$Inetd_script,0); }
    elsif ($service eq "lcd")     {
      return Cobalt::Repair::repair_generic("lcd",$Lcdbin,0,$Lcdsleepscript,0);     }
    elsif ($service eq "modem") {
      return Cobalt::Repair::repair_modem();
    }
    elsif ($service eq "mail")    {
      return Cobalt::Repair::repair_generic("mail",$Sendmail,$Sendmail_conf,
			    $Mailscript,1);    }
    elsif ($service eq "ntp")     {
      return Cobalt::Repair::repair_generic("ntp",$NtpDate,$NtpcConf,
			    $NTPscript,0);     }
    elsif ($service eq "nfs")     {
      return Cobalt::Repair::repair_generic($service,$Nfsd,0,$Nfsscript,0);    }
    elsif ($service eq "portmap") {
      return Cobalt::Repair::repair_generic("portmap",$Portmap_bin,0,
			    $Portmap_script,0); }
    elsif ($service eq "snmp")    {
      return Cobalt::Repair::repair_generic("snmp",$SNMPd,$SNMPconf,
			    $SNMPscript,0);    }
    elsif ($service eq "ssl")     {
      return MSG_error("1055")
	unless ($Features{'ssl'});
      return Cobalt::Repair::repair_generic("ssl",0,0,$SSL_script,0);
    }
    elsif ($service eq "telnet")   {
      return Cobalt::Repair::repair_generic("telnet",0,0,0,1); }
    elsif ($service eq "adm")        { return Cobalt::Repair::repair_adm();           }
    elsif ($service eq "atalk")      { return Cobalt::Repair::repair_atalk();         }
    elsif ($service eq "cpu")        { return Cobalt::Repair::repair_cpu();           }
    elsif ($service eq "filesystem") { return Cobalt::Repair::repair_disk($service);  }
    elsif ($service eq "userspace")  { return Cobalt::Repair::repair_disk($service);  }
    elsif ($service eq "groupspace") { return Cobalt::Repair::repair_disk($service);  }
    elsif ($service eq "ftp")        { return Cobalt::Repair::repair_ftp();           }
    elsif ($service eq "mem")        { return Cobalt::Repair::repair_mem();           }
    elsif ($service eq "net")        { return Cobalt::Repair::repair_net();           }
    elsif ($service eq "smb")        { return Cobalt::Repair::repair_smb();           }
    elsif ($service eq "www")        { return Cobalt::Repair::repair_www();           }

    else                             { return MSG_error("4701"); }
}

sub repair_atalk
# Repair the appleshare server
# Arguments: none
# Side effects: restarts appletalk and resets the network interface
{
    return MSG_error("1051")
      unless ($Features{'atalk'});
    return MSG_ok("2705")
      unless (-e $Atalklink);	# not supposed to be on
    system("$Atalkscript stop >>$Sauce_log 2>&1");
    Cobalt::Network::net_set_ipaddr(Cobalt::Network::net_get_ipaddr());
    system("$Atalkscript start >>$Sauce_log 2>&1");
    system ("$Lcdshowscript start >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
    return MSG_ok("2704");
}

sub repair_ftp
# Fix the FTP server
# Arguments: none
# Side effects: none, ftp is repaired by repair_inetd
{
    if (-e $FTP_link)
      {
	return Cobalt::Repair::repair_generic("ftp",$Ftpd,$Ftpd_conf,
			      $FTP_script,"ftp",0)
      }
    return MSG_error("1010");
}


sub repair_cpu
# Fix the CPU
# Arguments: none
# Side effects: none yet, what are we supposed to do, kill processes and hope?
#
# remember this one (from memory):
#
#  halt(8)               SunOS Programmer's Manual          halt(8)
#
#   ...
#
# OPTIONS
#
#     -q    quick halt.  Does not flush disk buffers.  May be useful if the
#           processor is on fire.
#
# anyway, back to our regular scheduled sauce code
{
  return MSG_error("1020");
}

sub repair_mem
# Fix memory usage
# Arguments: none
# Side effects: none yet, nothing we can do
{
  return MSG_error("1030");
}

sub repair_disk
# Repair the disk
# Arguments: none
# Side effects: none yet, it doesn't work
{
  return MSG_error("1040");
}

sub repair_net
# Repair the network interface
# Arguments: none
# Side effects: none yet, it doesn't work
{
  return MSG_error("1050");
}

sub repair_www
# Repair the web server
# Arguments: none
# Side effects: restarts, etc, the server; updates the database
# Return value: status string
{
    my $service = "www";

    return MSG_error("1700")
        unless ($Features{$service});

    my ($condition,$cond_time,$update_time,$notified,$attempts,
	$reserved,$data) = Cobalt::Monitor::mon_get_state($service);

    print STDERR "Attempting ${attempts}th restart of $service: $condition\n"
      if ($Debug);
    my $hostname = Cobalt::Network::net_get_hostname();

    open(LOG,">>$MonDBDir/$service.log") ||
      return MSG_error("3706", $MonDBDir);
    my $servertext = $Servers{$service};

    # First check some basics
    if (! -x $Httpd) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext binary is missing!</FONT>\n";
    }
    if (! -e $Htconf) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext config file(s) are missing!</FONT>\n";
    }
    if (! -x $Htscript) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext startup script is missing!</FONT>\n";
    }

    # Attempt #1: restart.
    if ($attempts <= $Stat_relimit)
      {
	my ($bad_config) = 0;	# flag if we got config file errors

	print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	Cobalt::Language::msg_localtime(time()), ": attempting restart</FONT>\n";
	system ("$Htscript stop >>$Sauce_log 2>&1");

	# We watch the output of the startup script to see if there
	# are config file errors.
	my ($badfile);
	open(SCRIPTOUT,"$Htscript start 2>&1 |")
	  || return MSG_error("3707", $!);
	while (<SCRIPTOUT>)
	  {
	    if (/Syntax error on line \d+ of (.+):/o)
	      {
		$badfile = $1;
	      }
	    elsif (/^Invalid/o)
	      {
		chomp;
		print LOG "<FONT COLOR=\"red\">$servertext, ",
		Cobalt::Language::msg_localtime(time()),
		": $servertext error in $badfile: $_</FONT>\n";
	      }
	  }
	close SCRIPTOUT;
	Cobalt::Monitor::mon_set_state($service,"","-","-",($attempts + 1),"-","-");

	if ($badfile)
	  {
	    # take punitive action
	    print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	    Cobalt::Language::msg_localtime(time()),
	    ": Rebuilding config file $badfile from master copy</FONT>\n";
	    rename($badfile,"$badfile.$^T");
	    open(SRC,"$badfile.master")
	      || return MSG_error("3708", $badfile);
	    open(DEST,">$badfile")
	      || return MSG_error("3709", $badfile);
	    print DEST while (<SRC>);
	    close DEST;
	    close SRC;

	    # Do our edits, as needed
	    Cobalt::Network::net_set_ipaddr(Cobalt::Network::net_get_ipaddr())
	      if (($service eq "adm") && ($badfile =~ /srm.conf$/o));
	    Cobalt::Network::net_set_hostname($hostname,1)
	      if ($badfile =~ /httpd.conf$/o);

	    # okay, NOW try it
	    system ("$Htscript start >>$Sauce_log 2>&1");
	  }

	system ("$Lcdshowscript start")
	  if (-x $Lcdshowscript);
      }
    elsif (($cond_time - $^T) > $Stat_lim_expire)
      {
	# we hit the limit over a day ago, reset the restart counter
	print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	Cobalt::Language::msg_localtime(time()), ": resetting restart counter</FONT>\n";
	Cobalt::Monitor::mon_set_state($service,"","-","-",0,"-","-");
      }
    else
      {
	print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	Cobalt::Language::msg_localtime(time()), ": too many restarts</FONT>\n";
      }

    close LOG;

    # Check the pulse
    Cobalt::Repair::repair_examine($service);
    ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      Cobalt::Monitor::mon_get_state($service);

    if ($condition == $Status{'green'})
      {
	return MSG_ok("2709");
      }
    else
      {
	return MSG_error("4710");
      }
}

sub repair_adm
# Repair the admin server
# Arguments: none
# Side effects: restarts, etc, the server; updates the database
# Return value: status string
{
    my $service = "adm";
    my $servertext = $Servers{$service};

    return MSG_error("1701") unless ($Features{'adm'});

    my ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,
	$data) = Cobalt::Monitor::mon_get_state($service);

    print STDERR "Attempting restart of $service: $condition\n" if ($Debug);
    my $hostname = Cobalt::Network::net_get_hostname();

    open(LOG,">>$MonDBDir/$service.log") ||
      return MSG_error("3710", $MonDBDir);

    # First check some basics
    if (! -x $Ahttpd) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext binary is missing!</FONT>\n";
    }
    if (! -e $AdminConf) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext config file(s) are missing!</FONT>\n";
    }
    if (! -x $Admscript) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext startup script is missing!</FONT>\n";
    }

    # Attempt #1: restart.
    if ($attempts <= $Stat_relimit)
      {
	my ($bad_config) = 0;	# flag if we got config file errors

	print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	Cobalt::Language::msg_localtime(time()), ": attempting restart</FONT>\n";
	system ("$Admscript stop >>$Sauce_log 2>&1");

	# We watch the output of the startup script to see if there
	# are config file errors.
	my ($badfile);
	open(SCRIPTOUT,"$Admscript start 2>&1 |")
	  || return MSG_error("3711", $!);
	while (<SCRIPTOUT>)
	  {
	    if (/Syntax error on line \d+ of (.+):/o)
	      {
		$badfile = $1;
	      }
	    elsif (/^Invalid/o)
	      {
		chomp;
		print LOG "<FONT COLOR=\"red\">$servertext, ",
		Cobalt::Language::msg_localtime(time()),
		": $servertext error in $badfile: $_</FONT>\n";
	      }
	  }
	close SCRIPTOUT;
	Cobalt::Monitor::mon_set_state($service,"","-","-",($attempts + 1),"-","-");

	if ($badfile)
	  {
	    # take punitive action
	    print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	    Cobalt::Language::msg_localtime(time()),
	    ": Rebuilding config file $badfile from master copy</FONT>\n";
	    rename($badfile,"$badfile.$^T");
	    open(SRC,"$badfile.master")
	      || return MSG_error("3712", $badfile);
	    open(DEST,">$badfile")
	      || return MSG_error("3713", $badfile);
	    print DEST while (<SRC>);
	    close DEST;
	    close SRC;

	    # Do our edits, as needed
	    Cobalt::Network::net_set_ipaddr(Cobalt::Network::net_get_ipaddr())
	      if (($service eq "adm") && ($badfile =~ /srm.conf$/o));
	    Cobalt::Network::net_set_hostname($hostname,1)
	      if ($badfile =~ /httpd.conf$/o);

	    # okay, NOW try it
	    system ("$Admscript start >>$Sauce_log 2>&1");
	  }

	system ("$Lcdshowscript start")
	  if (-x $Lcdshowscript);
      }
    elsif (($cond_time - $^T) > $Stat_lim_expire)
      {
	# we hit the limit over a day ago, reset the restart counter
	print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	Cobalt::Language::msg_localtime(time()), ": resetting restart counter</FONT>\n";
	Cobalt::Monitor::mon_set_state($service,"","-","-",0,"-","-");
      }
    else
      {
	print LOG "<FONT COLOR=\"$Status{$condition}\">$servertext, ",
	Cobalt::Language::msg_localtime(time()), ": too many restarts</FONT>\n";
      }

    close LOG;

    # Check the pulse
    Cobalt::Repair::repair_examine($service);
    ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      Cobalt::Monitor::mon_get_state($service);

    if ($condition == $Status{'green'})
      {
	return MSG_ok("2710");
      }
    else
      {
	return MSG_error("4711");
      }
}

sub repair_smb
# Fix the samba server
# Arguments: none
# Side effects: tries to restart samba, tries to fix the config file
# Return value: status string
{
    my ($badlines);

    return MSG_error("1702") unless ($Features{'smb'});
    # kludge because testparm demands a "hit return to continue" and
    # open2 is soooo annoying
    open(TP,"$Smb_testparm < /proc/version |")
      || return MSG_error("4712", $Smb_testparm);
    while (<TP>)
      {
	$badlines++ if (/^Unknown parameter encountered/o);
      }
    close TP;
    print STDERR "$badlines bad lines in samba config file\n" if ($Debug);

    # if too many bad lines, or the file is too small, replace it
    if (($badlines > 2) || ((stat($Smbconf))[7] < 400))
      {
	# take punitive action
	print LOG "<FONT COLOR=\"red\">", $Servers{'smb'},
	", ", Cobalt::Language::msg_localtime(time()), "<!-- $badlines -->",
	": Rebuilding config file $Smbconf from master copy</FONT>\n";
	rename($Smbconf,"$Smbconf.$^T"); # keep bad copy
	open(SRC,"$Smbconf.master")
	  || return MSG_error("3714", $Smbconf);
	open(DEST,">$Smbconf")
	  || return MSG_error("3715", $Smbconf);
	print DEST while (<SRC>);
	close DEST;
	close SRC;
      }

    return Cobalt::Repair::repair_generic("smb",$Smbd,$Smbconf,
			  $Smbscript,139,0);
}

sub repair_generic
# Generic server restart script
# Arguments: service,path to server binary, path to server config,
#   path to startup script, and restart inetd flag (1=yes,0=no)
# Side effects: restarts, etc, the server; updates the database
# Return value: status string
{
    my ($service,$server_bin,$server_conf,$server_script,$restart_inetd) = @_;

    my ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      Cobalt::Monitor::mon_get_state($service);

    my $servertext = $Servers{$service};

    print STDERR "Attempting restart of $service: $condition\n" if ($Debug);

    open(LOG,">>$MonDBDir/$service.log") ||
      return MSG_error("3705", $MonDBDir);

    # First check some basics
    if ($server_bin && (! -x $server_bin)) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext binary is missing!</FONT>\n";
    }
    if ($server_conf && (! -e $server_conf)) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext config file(s) are missing!</FONT>\n";
    }
    if ($server_script && (! -x $server_script)) {
      print LOG "<FONT COLOR=\"red\">$servertext, ",
      Cobalt::Language::msg_localtime(time()),
      ": $servertext startup script is missing!</FONT>\n";
    }

    my $actiontext;
    if ($attempts <= $Stat_relimit) # not too many tries yet
      {
	$actiontext = "attempting restart";

	if ($server_script) {
	  # there may be no server script, just inetd
	  system ("$server_script stop >>$Sauce_log 2>&1");
	  system ("$server_script start >>$Sauce_log 2>&1");
	}

	# inetd is often a special case; we don't restart the server
	# itself, we start the master daemon that runs it
	if ($restart_inetd)
	  {
	    system ("$Inetd_script stop >>$Sauce_log 2>&1");
	    system ("$Inetd_script start >>$Sauce_log 2>&1");
	  }

	# record the additional attempt
	Cobalt::Monitor::mon_set_state($service,"","-","-",($attempts + 1),"-","-");

	system ("$Lcdshowscript start")
	  if (-x $Lcdshowscript);
      }
    elsif (($cond_time - $^T) > $Stat_lim_expire)
      {
	# we hit the limit over a day ago, reset the restart counter
	$actiontext = "resetting restart counter";
	Cobalt::Monitor::mon_set_state($service,"","-","-",0,"-","-");
      }
    else
      {
	$actiontext = "too many restarts";
      }

    # look at our state now
    Cobalt::Repair::repair_examine($service);
    ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      Cobalt::Monitor::mon_get_state($service);

    # "How we doin'?"
    if ($condition == $Status{'green'})
      {
	print LOG "<FONT COLOR=\"green\">$servertext, ", Cobalt::Language::msg_localtime(time()),
	": $actiontext ... $service restarted successfully</FONT>\n";
	close LOG;
	return MSG_ok("2703", $servertext);
      }
    else
      {
	print LOG "<FONT COLOR=\"red\">$servertext, ", Cobalt::Language::msg_localtime(time()),
	": Couldn't restart $service ... $actiontext</FONT>\n";
	close LOG;
	return MSG_error("4703", $servertext);
      }
}

##########

sub repair_notify
# Mail a report of what happened
# Arguments: The server, the instance,  and (optional) an list of
#    additional addresses to which to send the notification
# Note that this should only get called with an instance argument if we
#    want seperate notifications for each instance of a service.  In
#    practice, this means only for quota problems (users and groups).
#    Actually, this goes for the alsotell argument as well.
# This function checks to see if a notification has already been sent,
#    and does not repeat if one has.  It does *not* check the state of
#    the service to be sure that a notification is called for.
# Side effects: sends email to admin, sets the notified flag in the
#    database
# Doesn't warn about CPU problems if swapping is high
{
    my ($service,$instance,@alsotell) = @_;
    my $alsotell = join(',',@alsotell);
    $alsotell ||= '';		# just to be picky
    print STDERR "Notify $service/$instance ($alsotell)\n"
      if ($Debug);

    # We want the mail to appear to come from the site the user
    # is associated with
    my $adminEmail = $Adminuser;
    if (($Features{'vmachine'})) {
       if( $alsotell[0] ) {
           $adminEmail .= '@' . (Cobalt::User::user_list_site($alsotell[0]))[0];
       }

       my $i;
       for ($i=0; $i<=$#alsotell; $i++) {
           $alsotell[$i] .= '@' . (Cobalt::User::user_list_site($alsotell[$i]))[0];
       }
    }

    my $alsotell = join(',',@alsotell);
    $alsotell ||= '';  # just to be picky
    print STDERR "Notify $service/$instance ($alsotell)\n" if ($Debug);

    if ($service eq "cpu")
      {
	# special case: don't warn about load if it's really
	# a swapping issue
	my ($swap_sev) = Cobalt::Monitor::mon_get_state("mem","swapping");
	return if ($swap_sev != $Status{'green'});
      }

    my ($condition,$cond_time,$update_time,$notified,$attempts,$reserved,$data) =
      Cobalt::Monitor::mon_get_state($service,$instance);
    print STDERR "Notify $service/$instance: $condition\n" if ($Debug);
    return if ($notified);

    my $subject = Cobalt::Repair::repair_get_subject($service,$instance);
    my $header = "From: $adminEmail\nTo: $AlertList\n";
    $header.="Cc: $alsotell\n" if ($alsotell);
    $header.="Subject: $subject\n\n";

    if ($Language{'japanese'}) {
       # convert header from Shift-JIS to JIS for sendmail
       my $tmpShiftJISFile="/tmp/header.sjis.$$";

       open( SHIFTJIS, ">$tmpShiftJISFile" );
       print SHIFTJIS $header;
       close( SHIFTJIS );

       open( JIS, "$ShiftJISToJIS_prog $tmpShiftJISFile|")
           || die "Can't run $ShiftJISToJIS_prog $tmpShiftJISFile: $!\n";
       $header=join '', <JIS>;
       close( JIS );

       # cleanup
       unlink( $tmpShiftJISFile );
    }

    my $advice = Cobalt::Repair::repair_get_message($service,$instance);
    print STDERR "Sending mail to $AlertList $alsotell ($condition)\n"
      if ($Debug);
    if (open (MAIL, "|$Sendmail -oi -t"))
      {
	print MAIL $header;
	print MAIL "$advice\n";
	close MAIL;
	if ($service eq "net")
	  {
	    # special case: set the notified flag for both collisions and col15
	    Cobalt::Monitor::mon_set_state($service,"collisions","-",1,"-","-","-");
	    Cobalt::Monitor::mon_set_state($service,"col15","-",1,"-","-","-");
	  }
	Cobalt::Monitor::mon_set_state($service,$instance,"-",1,"-","-","-");
	open(LOG,">>$MonDBDir/$service.log") ||
	  return MSG_error("3051", $MonDBDir);
	print LOG "<FONT COLOR=\"$Status{$condition}\">$service, ",
	Cobalt::Language::msg_localtime($^T),
	": Sending alert mail to $Adminuser $alsotell</FONT><!-- $instance ($data)-->\n";
	close LOG;
      }
    else
      {
	return MSG_error("4705", $!);
      }
}

sub repair_get_message
# Given the flags from the monitoring database, return the message.
#   This message should provide advice on how the user should deal
#   with the problem at hand.
# Arguments: service, instance
# Return value: message string
{
    my ($service,$instance) = @_;
    my $return;

    # We're really only interested in the $data fields
    my ($condition,$cond_time,$update_time,$notified,$attempts,
	$reserved,$data) = Cobalt::Monitor::mon_get_state($service,$instance);

    # Handle a few special cases, but pretty much generalize
    # the loop
    if ($service eq "groupspace") # user quota
      {
	# $instance is the group or site name
	# $data is (percentage of quota used, amount used, limit)
	my $name;
	my @data = split(/,/o,$data);
	if($Features{'vsites'}) {
	  $name = ( Cobalt::Vsite::vsite_get_bygroup($instance) )[2];
	  $name ||= $instance;	# Group name better than no name
	  $return = MSG_get("mon_sitespace_body");
	  $return = eval "\"$return\"";
	  return $return;
	}
	else {
	  $name = $instance;	# group name is the instance
	  $return = MSG_get("mon_${service}_body");
	  $return = eval "\"$return\"";
	  return $return;
	}

      }
    elsif ($service eq "dns")
      {
	if ($instance eq "nslookup") # can't resolve our name
	  {
	    return MSG_get("mon_dns_nslookup_body");
	  }
	else
	  {
	    my @data = split(/,/o,$data);
	    $return = MSG_get("mon_${service}_body");
	    $return = eval "\"$return\"";
	    return $return;
	  }
      }
    elsif ($service eq "mail")	# mail server
      {
	if    ($instance eq "smtp") # SMTP server
	  {
	    return MSG_get("mon_mail_smtp_body");
	  }
	elsif ($instance eq "pop3") # POP3 server (run via inetd)
	  {
	    return MSG_get("mon_mail_pop3_body");
	  }
	elsif ($instance eq "imap") # IMAP server (run via inetd)
	  {
	    return MSG_get("mon_mail_imap_body");
	  }
	else
	  {
	    return MSG_get("mon_mail_body");
	  }
      }
    else {
      my @data = split(/,/o,$data);
      $return = MSG_get("mon_${service}_body");
      $return = eval "\"$return\"";
      return $return;
    }
}

sub repair_get_subject
# Build the subject line for alert email
# Arguments: server, instance
# Side effects: none
# Return value: A message string
{
    my ($service,$instance,$flags) = @_;
    my $return;
    if (($service eq "userspace") || ($service eq "mailspace"))
      {
	$return = MSG_get("mon_${service}_subject");
	$return = eval "\"$return\"";
	return $return;
      }
    elsif ($service eq "groupspace") # group quota
      {
	if($Features{'vsites'})
	  {
	    my($siteHostname) = ( Cobalt::Vsite::vsite_get_bygroup($instance) )[1];
        $siteHostname ||= $instance; # Group name better than no hostname
	$return = MSG_get("mon_sitespace_subject");
	$return = eval "\"$return\"";
	return $return;
      } else {
	$return = MSG_get("mon_groupspace_subject");
	$return = eval "\"$return\"";
	return $return;
      }
    }
    elsif ($service eq "filesystem")
    {
      return MSG_get("mon_filesystem_subject");
    }
    $return = MSG_get("mon_subject");
    $return = eval "\"$return\"";
    return $return;
}

sub repair_modem
# Fix the modem
# Arguments: none
# Return value: none
# Side effects: attempts to troubleshoot the modem
{
  my ($condition) = mon_query("modem","diald");
  if ($condition != $Status{'green'}) {  # is diald the problem?
    # try to restart diald
    Cobalt::Repair::repair_generic("modem",$Diald_bin,$Diald_conf,
				   $Diald_script,0);
  }
  return;
}

1;
