#
# $Id: Security.pm,v 1.15 1999/10/02 16:44:27 duncan Exp $
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Security;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(set_user_management set_group_management);

use Cobalt::Product;
use Cobalt::Util;
#use Cobalt::Paths;
#use strict;

1;
__DATA__

sub security_set_user_management
# Allow or disallow anyone in the world to manage users
# Arguments: 1 to allow anyone in the world to manage users,
#   0 to allow only user "admin" to manage users
# Return value: usual string of form "785 Green is a pretty color"
# Side effects: creates or deletes .htaccess files as appropriate
{
    my ($setting) = @_;
    return MSG_error("1400") unless ($Features{'users'});
    if ($setting)
    {
	# allow anyone
	if (! -e "$Users_dir/$Access_file")
	{
	    return MSG_ok("2441");
	}
	else 
	{
	    unlink("$Users_dir/$Access_file") ||
		return MSG_error("4401", $Access_file);
	    return MSG_ok("2447");
	}
    }
    else
    {
	# allow admin only
	if (-e "$Users_dir/$Access_file")
	{
	    return MSG_ok("2443");
	}
	else 
	{
	    open(UXS,">$Users_dir/$Access_file") ||
		return MSG_error("4402", $Access_file);
	    print UXS "<LIMIT GET>\norder allow,deny\nallow from all\n";
	    print UXS "require user $Adminuser\n</LIMIT>\n";
	    print UXS "Authname $Adm_Authname\n";
	    print UXS "Authtype Basic\n";
    	    unless ($Features{'pam'}) {
	    print UXS "AuthUserFile $Htpasswd\n";
	    print UXS "AuthGroupFile $Htgroup\n";
	    }
	    close UXS;
	    return MSG_ok("2449");
	}
    }
}


sub security_set_group_management
# Allow or disallow anyone in the world to manage groups
# Arguments: 1 to allow anyone in the world to manage groups,
#   0 to allow only user "admin" to manage groups
# Return value: usual string of form "924 I eat chalk"
# Side effects: creates or deletes .htaccess files as appropriate
{
    my ($setting) = @_;
    return MSG_error("1401") unless ($Features{'groups'});
    if ($setting)
    {
	# allow anyone
	if (! -e "$Groups_dir/$Access_file")
	{
	    return MSG_ok("2442");
	}
	else 
	{
	    unlink("$Groups_dir/$Access_file") ||
		return MSG_error("4403", $Access_file);
	    return MSG_ok("2440");
	}
    }
    else
    {
	# allow admin only
	if (-e "$Groups_dir/$Access_file")
	{
	    return MSG_ok("2444");
	}
	else 
	{
	    open(GXS,"$Groups_dir/$Access_file") ||
		return MSG_error("4404", $Access_file);
	    print GXS "<LIMIT GET>\norder allow,deny\nallow from all\n";
	    print GXS "require user $Adminuser\n</LIMIT>\n";
	    print GXS "Authname $Adm_Authname\n";
	    print GXS "Authtype Basic\n";
            unless ($Features{'pam'}) {
            print GXS "AuthUserFile $Htpasswd\n";
	    print GXS "AuthGroupFile $Htgroup\n";
            }
	    close GXS;
	    return MSG_ok("2448");
	}
    }
}

1;
