#
# $Id: Services.pm,v 1.48 1999/10/14 22:20:37 duncan Exp $
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#            Kevin K.M. Chiu (kevin@cobaltnet.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Services;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(service_set_active   service_get_active
	     set_arkeia_client_on get_arkeia_client_on
             set_arkeia_servername get_arkeia_servername
	     set_atalk_server_on  get_atalk_server_on
	     set_dhcp_server_on   get_dhcp_server_on
	     set_dns_server_on    get_dns_server_on
	     set_ftp_server_on    get_ftp_server_on
	     set_fpx_on           get_fpx_on
	     set_gated_on         get_gated_on
             set_gated_interfaces get_gated_interfaces
	     set_mail_server_on   get_mail_server_on
	     set_networker_client_on get_networker_client_on
	     set_nfs_server_on    get_nfs_server_on
	     set_nfs_perms        get_nfs_perms
	     set_ntp_server_on    get_ntp_server_on
	     set_smb_server_on    get_smb_server_on
	     set_smb_workgroup    get_smb_workgroup
	     set_smb_controller   get_smb_controller
	     set_snmp_server_on   get_snmp_server_on
	     set_snmp_community   get_snmp_community
	     set_telnet_server_on get_telnet_server_on
	     set_telnet_server_open get_telnet_server_open
	     service_set_init_on  service_set_init_off
	     service_get_init
	     );

require Cobalt::Language;
import Cobalt::Language qw(:DEFAULT);
require Cobalt::Network;
import Cobalt::Network qw(net_get_ipaddr net_set_ipaddr);
use Cobalt::Util;
use Cobalt::Product;

1;
sub service_get_init
# get the state of the service in the given runlevel
# arguments: state, runlevel (defaults to 3 if not specified)
{
    my ($service,$state) = @_;
    my ($s);

    ($state) ? ($s=$state) : ($s=3);
    $result = `/sbin/chkconfig --list $service`;

    $result =~ /^$service [\s\S]*$s:(\S+)/;

    ($1 eq "on") ? ($state = 1) : ($state = 0);
    return $state;
}

sub service_set_init_on
# set the given service state to on or of
# arguments: service name, list of runlevels
{
    my ($service,@states,$level);
    $service = shift;
    @states = @_;

    if (@states) {
	$level = " --level ";
	$level .= join('',@states);
    }

    system("/sbin/chkconfig $level $service on >> $Sauce_log 2>&1") &&
	return;

    return MSG_ok("service_set_init_on_ok", $service);
}

sub service_set_init_off
# set the given service state to on or of
# arguments: service name, list of runlevels
{
    my ($service,@states,$level);
    $service = shift;
    @states = @_;

    if (@states) {
	$level = " --level ";
	$level .= join('',@states);
    }

    system("/sbin/chkconfig $level $service off >> $Sauce_log 2>&1") &&
	return;

    return MSG_ok("service_set_init_off_ok", $service);
}

sub service_set_active
# Turn the given service on or off
# Arguments: service name, 1 for on/0 for off
# Side effects: sets/breaks symlinks for the boot scripts, starts up
#   the server
# Return value: status string of the form MSG_ok("2821")
# Example: $ret = service_enable("dns",1);
{
  my ($service,$state) = @_;
  if ($service eq "mail") {
    return Cobalt::Services::set_mail_server_on($state);
  }
  if ($service eq "arkeia") {
    return Cobalt::Services::set_arkeia_client_on($state);
  }
  if ($service eq "atalk") {
    return Cobalt::Services::set_atalk_server_on($state);
  }
  if ($service eq "dhcp") {
    return Cobalt::Services::set_dhcp_server_on($state);
  }
  if ($service eq "dns") {
    return Cobalt::Services::set_dns_server_on($state);
  }
  if ($service eq "ftp") {
    return Cobalt::Services::set_ftp_server_on($state);
  }
  if ($service eq "gated_on") {
    return Cobalt::Services::set_gated_on($state);
  }
  if ($service eq "networker") {
    return Cobalt::Services::set_networker_client_on($state);
  }
  if ($service eq "nfs") {
    return Cobalt::Services::set_nfs_server_on($state);
  }
  if ($service eq "ntp") {
    return Cobalt::Services::set_ntp_server_on($state);
  }
  if ($service eq "smb") {
    return Cobalt::Services::set_smb_server_on($state);
  }
  if ($service eq "snmp") {
    return Cobalt::Services::set_snmp_server_on($state);
  }
  if ($service eq "telnet") {
    return Cobalt::Services::set_telnet_server_on($state);
  }
}

sub service_get_active
# Get the state of the service
# Arguments: service name
# Side effects: none
# Return value: 1 for on, 0 for off
# Example: $ret = service_enable("dns",1);
{
  my ($service) = @_;
  if ($service eq "mail") {
    return Cobalt::Services::get_mail_server_on();
  }
  if ($service eq "arkeia") {
    return Cobalt::Services::get_arkeia_client_on();
  }
  if ($service eq "atalk") {
    return Cobalt::Services::get_atalk_server_on();
  }
  if ($service eq "dhcp") {
    return Cobalt::Services::get_dhcp_server_on();
  }
  if ($service eq "dns") {
    return Cobalt::Services::get_dns_server_on();
  }
  if ($service eq "ftp") {
    return Cobalt::Services::get_ftp_server_on();
  }
  if ($service eq "gated_on") {
    return Cobalt::Services::get_gated_on();
  }
  if ($service eq "networker") {
    return Cobalt::Services::get_networker_client_on();
  }
  if ($service eq "nfs") {
    return Cobalt::Services::get_nfs_server_on();
  }
  if ($service eq "ntp") {
    return Cobalt::Services::get_ntp_server_on();
  }
  if ($service eq "smb") {
    return Cobalt::Services::get_smb_server_on();
  }
  if ($service eq "snmp") {
    return Cobalt::Services::get_snmp_server_on();
  }
  if ($service eq "telnet") {
    return Cobalt::Services::get_telnet_server_on();
  }
}

sub set_mail_server_on
# Turn the mail server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "778 can't tapdance"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
    my ($state) = @_;
    return MSG_error("1100")
      unless ($Features{'mail'});

    my $istate = service_get_init("sendmail");
    if ($state && (! $istate))
    {
	service_set_init_on("sendmail")
	    || return MSG_error("4420", $Mailscript);
	return MSG_error("4470", $Inetd_conf)
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return MSG_error("4433", $Lockdir);
	open(I,$Inetd_conf)
	    || return MSG_error("4432", $Inetd_conf);
	while (<I>)
	{
	    if (/^\#(pop-3|imap)/o)
	    {
		$_ =~ s/^\#//o;
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return MSG_error("4434", $Inetd_conf);
	open(PID,$Inetd_pid) || return MSG_error("3458", $!);
	my $line = <PID>;
	my $pid;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	system ("$Mailscript start >> $Sauce_log 2>&1") &&
	    return MSG_error("3455", $!);
	system("$Newaliases_cmd >>$Sauce_log 2>&1");
	system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	return MSG_ok("2450");
    }
    elsif ((! $state) && ($istate))
    {
	service_set_init_off("sendmail")
	    || return MSG_error("3457", $Maillink);
	return MSG_error("4416", $Inetd_conf)
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return MSG_error("4437", $Lockdir);
	open(I,$Inetd_conf)
	    || return MSG_error("4436", $Inetd_conf);
	while (<I>)
	{
	    if (/^(pop-3|imap)/o)
	    {
		$_ =~ s/^/\#/o;
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return MSG_error("4438", $Inetd_conf);
	open(PID,$Inetd_pid) || return MSG_error("3456", $!);
	my ($pid);
	my $line = <PID>;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	system ("$Mailscript stop >> $Sauce_log 2>&1") &&
	    return MSG_error("4421", $!);
	system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	return MSG_ok("2452");
    }
    else
    {
	my $off;
	($state) ? ($off = MSG_get("On_string")) : 
	  ($off = MSG_get("Off_string"));
	return MSG_ok("2451", $off);
    }
}

sub get_mail_server_on
# Is the mail server set to be on?
# Returns 1 if the mail server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    return service_get_init("sendmail");
}

sub set_smb_server_on
# Turn the smb server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "682 insufficient depth perception"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
    my ($state) = @_;

    return MSG_error("1101")
      unless ($Features{'smb'});

    my $istate = service_get_init("smb");
    if ($state && (! $istate))
    {
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to restart samba\n" if ($Debug);
	    return MSG_ok("2493");
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    # Note that we die() instead of return() because we are
	    # a child process, and if we return we'll end up back in
	    # the CGI, executing the same code our parent process
	    # is executing.  That would be Bad.
	    service_set_init_on("smb")
		|| die MSG_error("4423", $Smblink);
	    system ("$Smbscript start >> $Sauce_log 2>&1") &&
	      die MSG_error("3493", $!);
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    exit;
	}
    }
    elsif ((! $state) && ($istate))
    {
	service_set_init_off("smb")
	    || return MSG_error("3497", $Smblink);
	system ("$Smbscript stop >> $Sauce_log 2>&1") &&
	    return MSG_error("4424", $!);
	system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	return MSG_ok("2497");
    }
    else
    {
	my $off;
        ($state) ? ($off = MSG_get("On_string")) : 
		    ($off = MSG_get("Off_string"));
	return MSG_ok("2491", $off);
    }
}

sub get_smb_server_on
# Is the smb server set to be on?
# Returns 1 if the smb server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    return service_get_init("smb");
}

sub set_nfs_server_on
# Turn the nfs server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "882 insufficient depth perception"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
    my ($state) = @_;
    
    return MSG_error("1102")
      unless($Features{'nfs'});
    
    my $istate = service_get_init("nfsfs");
    if ($state && (! $istate))
    {
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to start NFS\n" if ($Debug);
	    return MSG_ok("2494");
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    service_set_init_on("nfsfs")
		|| die MSG_error("4425", $!);
	    service_set_init_on("portmap")
		|| die MSG_error("3434", $!);
	    system("$Portmap_script start >>$Sauce_log 2>&1") &&
	      die MSG_error("3435", $!);
	    system ("$Nfsscript start") &&
	      die MSG_error("3494", $!);
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    exit;
	}
    }
    elsif ((! $state) && ($istate))
    {
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to stop NFS\n" if ($Debug);
	    return MSG_ok("2496");
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    service_set_init_off("nfsfs")
		|| die MSG_error("3496", $!);
	    service_set_init_off("portmap")
		|| die MSG_error("3436", $!);
	    system ("$Nfsscript stop") &&
	      die MSG_error("4426", $!);
	    system("$Portmap_script stop >>$Sauce_log 2>&1") &&
	      die MSG_error("4452", $!);
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    exit;
	}
    }
    else
    {
	my $off;
        ($state) ? ($off = MSG_get("On_string")) : 
		    ($off = MSG_get("Off_string"));
	return MSG_ok("2498", $off);
    }
}

sub get_nfs_server_on
# Is the nfs server set to be on?
# Returns 1 if the nfs server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    return service_get_init("nfsfs");
}

sub set_dns_server_on
# Turn the dns server on or off
# Arguments: 1 for on, 0 for off
# Side effects: creates or removes /etc/named.boot, runs or kills named
{
    my ($state) = @_;

    return MSG_error("1103")
      unless ($Features{'dns'});

    my $istate = service_get_init("named");
    # Supports both Bind 4 and 8 formats
    my($bind_conf)=$Named_boot;
    $bind_conf=$Named_conf if ($Features{'bind8'});

    if ($state)
    {
	service_set_init_on("named")
	    || return MSG_error("4850", $Named_disabled);

        if (-e "$Named_disabled")
	{
            rename($Named_disabled,$bind_conf)
		|| return MSG_error("4850", $Named_disabled);
            unless (system("$Named_script start >>$Sauce_log 2>&1"))
	    {
                return MSG_ok("2850");
	    }
            else
	    {
                return MSG_error("4851", $!);
	    }
	}
        elsif (-e "$bind_conf")
        {
            return MSG_ok("2856");
        }
        else
        {
            # Create fresh new named.boot
            my($named_boot,$srv);
	    if ($Features{'bind8'}) {
		$named_boot = <<EOF;
options { directory "$Named_dir"; };
zone "." { type hint; file "db.cache"; };
EOF
	    } else {
		$named_boot = "directory\t$Named_dir\n\ncache\t.\tdb.cache\n";
            }
            open(NAMED,"> $bind_conf")
		|| return MSG_error("4852", $Named_boot);
            print NAMED $named_boot;
            close(NAMED);
            $srv = system("$Named_script start >>$Sauce_log 2>&1");

            if (!$srv)
	    {
		return MSG_ok("2852");
	    }
            else
	    {
                return MSG_error("4853", $!);
	    }
	}
    }
    else
    {
	service_set_init_off("named") 
	    || return MSG_error("4856", $Named_boot);

        if (-e "$bind_conf")
	{
	    my $srv = system("$Named_script stop >>$Sauce_log 2>&1");
	    unlink( "$Subsys/named" ) if (-f "$Subsys/named" );
	    
	    rename($bind_conf,$Named_disabled)
		|| return MSG_error("4856", $Named_boot);
	    if (!$srv)
	    {
		return MSG_ok("2854");
	    }
	    else
	    {
		return MSG_error("4854", $!);
	    }
	}
        else
	{
	    return MSG_ok("2855");
	}
    }
    return 1;
}

sub get_dns_server_on
# Checks for an active DNS server
# Returns 1 if the dns service is active, 0 if it is disabled
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    # Supports both Bind 4 and 8 formats
    my($bind_conf)=$Named_boot;
    $bind_conf=$Named_conf if ($Features{'bind8'});

    (-e "$bind_conf") ? (return 1) : (return 0);
}

sub set_atalk_server_on
# Turn the atalk server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "582 insufficient depth perception"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
    my ($state) = @_;

    return MSG_error("1104")
      unless ($Features{'atalk'});

    my $istate = service_get_init("atalk");
    if ($state && (! $istate))
    {
	service_set_init_on("atalk")
	    || return MSG_error("4427", $Atalklink);
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to restart appletalk\n" if ($Debug);
	    return MSG_ok("2431");
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    system ("$Atalkscript start >>$Sauce_log 2>&1 ") &&
		warn MSG_error("3431", $!);
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    exit;
	}
    }
    elsif ((! $state) && ($istate))
    {
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to stop appletalk\n" if ($Debug);
	    return MSG_ok("2433");
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    service_set_init_off("atalk")
		|| die MSG_error("3433", $Atalklink);
	    system ("$Atalkscript stop 1>&2") &&
	      die MSG_error("4428", $!);
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    Cobalt::Network::net_set_ipaddr(Cobalt::Network::net_get_ipaddr());
	    exit;
	}
    }
    else
    {
	my $off;
        ($state) ? ($off = MSG_get("On_string")) : 
	  ($off = MSG_get("Off_string"));
	return MSG_ok("2454", $off);	
    }
}

sub get_atalk_server_on
# Is the atalk server set to be on?
# Returns 1 if the atalk server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    return service_get_init("atalk");
}

sub set_dhcp_server_on
# Turn the dhcp server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form MSG_error("4325")
# Side effects: creates or breaks symlink
{
    my ($state) = @_;
    my $istate = service_get_init("dhcpd");

    return MSG_error("1110")
      unless ($Features{'dhcpd'});

    return MSG_error("4333") if ($state && !(-e $Dhcpd_conf));

    if ($state && (! $istate))
    {
	unless (-e $Dhcpd_leases)
	{
	    open(LEES,"> $Dhcpd_leases") || return MSG_error("4334");
	    close(LEES);
	}
	open(DHCPSTART,"$Dhcpdscript start 2>&1 |") ||
	  return MSG_error("dhcpd_cant_runscript");
	open(LOG,">>$Sauce_log") || return MSG_error("dhcpd_cant_log", $Sauce_log);
	# The error message comes right before the "exiting" line...
	# so we remember the error and then return it if we see
	# "exiting"
	my ($line,$line1,$line2);
	while (<DHCPSTART>) {
	  if (/exiting/o) {
	    print LOG;
	    close LOG;
	    close DHCPSTART;
	    $line = "$line1 $line2";
	    return MSG_error("dhcpd_cant_start", $line);
	  }
	  unless (/^[\s\^]*$/o) {  # ignore blank lines
	    $line1 = $line2;
	    chomp($line2 = $_);
	  } 
	  print LOG;
	}
	close DHCPSTART;
	close LOG;
	system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	  if (-x $Lcdshowscript);
	service_set_init_on("dhcpd")
	    || return MSG_error("4335", $Dhcpdlink);
	return MSG_ok("2335");
    }
    elsif ((! $state) && ($istate))
    {
	my $pid;
	if (!defined($pid = fork)) {
	    die "cannot fork: $!";
	} elsif ($pid) {
	    # i'm the parent
	    print STDERR "Forking to stop Dhcpd\n" if ($Debug);
	    return MSG_ok("2332");
	}
	else
	{
	    # otherwise I'm the child
	    open(STDERR,">>$Sauce_log");
	    open(STDOUT,">>$Sauce_log");
	    service_set_init_off("dhcpd")
		|| die MSG_error("3503", $Dhcpdlink);
	    system ("$Dhcpdscript stop 1>&2") &&
	      die MSG_error("4336", $!);
	    system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
	    exit;
	}
    }
    elsif ($state)
    {
      return MSG_ok("2333");	
    }
    else 
    {
      return MSG_ok("2334");	
    }
}

sub get_dhcp_server_on
# Is the dhcp server set to be on?
# Returns 1 if the atalk server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    return service_get_init("dhcpd");
}

sub set_ftp_server_on
# Turn the ftp server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "638 smokey bacon"
# Side effects: modifies inetd.conf
{
    my ($state) = @_;
    if ($state)
    {
	# turn it on!
	return MSG_error("4439", $Inetd_conf)
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return MSG_error("4441", $Lockdir);
	open(I,$Inetd_conf)
	    || return MSG_error("4440", $Inetd_conf);
	while (<I>)
	{
	    if (/^\#ftp/o)
	    {
		$_ =~ s/^\#ftp/ftp/o;
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return MSG_error("4442", $Inetd_conf);
	open(PID,$Inetd_pid) || return MSG_error("3459", $!);
	my ($pid);
	my $line = <PID>;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	return MSG_ok("2453");
    }
    else
    {
	return MSG_error("4443", $Inetd_conf)
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return MSG_error("4445", $Lockdir);
	open(I,$Inetd_conf)
	    || return MSG_error("4444", $Inetd_conf);
	while (<I>)
	{
	    if (/^ftp/o)
	    {
		$_ =~ s/^ftp/\#ftp/o;
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return MSG_error("4446", $Inetd_conf);

	open(PID,$Inetd_pid) || return MSG_error("3464", $!);
	my $pid;
	my $line = <PID>;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	return MSG_ok("2464");
    }
}

sub get_ftp_server_on
# Is the ftp server set to be on?
# Returns 1 if the ftp server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    open(INETD,$Inetd_conf) || return MSG_error("4430", $Inetd_conf);
    while (<INETD>)
    {
	if (/^ftp\s/o)
	{
	    close INETD;
	    return 1;
	}
    }
    close INETD;
    return 0;
}

sub set_nfs_perms
# Define the list of hosts which are allowed to connect to the NFS server.
# Arguments: up to two strings specifying hosts which should be granted
#   access.  Empty arguments will clear the lines from the exports file.
#   The strings may include wilcards (*.foo.com) or IP addresses.
# Side effects: modifies /etc/exports and calls exportfs
# Return value: usual error string
{
    return MSG_error("1105")
      unless ($Features{'nfs'});

    my ($allow1,$allow2) = @_;
    return MSG_error("4449", $allow1)
	unless ($allow1 =~ /^[\w\*\.\-]*$/o);
    return MSG_error("4462", $allow2)
	unless ($allow2 =~ /^[\w\*\.\-]*$/o);
    open(EXPORTS,">$Exports_file") ||
	return MSG_error("4447", $Exports_file);
    if ($allow1)
    {
	print EXPORTS "$Htgroup_dir\t$allow1(rw)\n";
	print EXPORTS "$Htusers_dir\t$allow1(rw)\n";
    }
    if ($allow2)
    {
	print EXPORTS "$Htgroup_dir\t$allow2(rw)\n";
	print EXPORTS "$Htusers_dir\t$allow2(rw)\n";
    }
    close EXPORTS;
    # exportfs sends signals to the NFS processes.  If the processes aren't
    # running, then don't bother with the signal
    if (Cobalt::Services::get_nfs_server_on())
    {
	system($Exports_cmd) && return MSG_error("4448", $!);
    }
    return MSG_ok("2455");
}

sub get_nfs_perms
# Find out who we are exporting to
# Arguments: none
# Return value: a list of (up to two) hosts or host-patterns to which
#   we are exporting NFS
# Side effects: none
{
    my @exportees;
    open(EXPORTS,$Exports_file) ||
	return MSG_error("4463", $Exports_file);
    while (<EXPORTS>)
    {
	# Only count the user lines.  Assume that the group line
	# to match will be there
	if (/users\t(.+)\(rw\)$/o)
	{
	    push (@exportees,$1);
	}
    }
    close EXPORTS;
    return @exportees;
}


sub set_telnet_server_on
# Turn the telnet server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "619 Major Strasser's been shot"
# Side effects: modifies inetd.conf
{
    my ($state) = @_;
    if ($state)
    {
	# turn it on!
	return MSG_error("4453", $Inetd_conf)
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return MSG_error("4455", $Lockdir);
	open(I,$Inetd_conf)
	    || return MSG_error("4454", $Inetd_conf);
	while (<I>)
	{
	    if (/^\#$SecretTelnetPort/o)
	    {
		$_ =~ s/^\#$SecretTelnetPort/$SecretTelnetPort/o;
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return MSG_error("4456", $Inetd_conf);
	open(PID,$Inetd_pid) || return MSG_error("3465", $!);
	my ($pid);
	my $line = <PID>;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	return MSG_ok("2456");
    }
    else
    {
	return MSG_error("4457", $Inetd_conf)
	    if (-e "$Lockdir/inetd.conf");
	open(O,">$Lockdir/inetd.conf")
	    || return MSG_error("4459", $Lockdir);
	open(I,$Inetd_conf)
	    || return MSG_error("4458", $Inetd_conf);
	while (<I>)
	{
	    if (/^$SecretTelnetPort/o)
	    {
		$_ = "#$_";
	    }
	    print O;
	}
	close I;
	close O;
	rename("$Lockdir/inetd.conf",$Inetd_conf)
	    || return MSG_error("4460", $Inetd_conf);

	open(PID,$Inetd_pid) || return MSG_error("3466", $!);
	my $pid;
	my $line = <PID>;
	if ($line =~ /^(\d+)$/o)
	{
	    $pid = $1;
	}
	close PID;
	kill(1,$pid) if ($pid);
	return MSG_ok("2457");
    }
}

sub get_telnet_server_on
# Is the telnet server set to be on?
# Returns 1 if the telnet server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    open(INETD,$Inetd_conf) || return MSG_error("4461", $Inetd_conf);
    while (<INETD>)
    {
	if (/^$SecretTelnetPort\s/o)
	{
	    close INETD;
	    return 1;
	}
    }
    close INETD;
    return 0;
}

sub set_telnet_server_open
# Set wether only root can telnet in, or anyone at all
# Arguments: 1 for anyone, 0 for root only
# Side effects: creates or removes /etc/nologin, modifies
#   /etc/rc.d/rc.sysinit to remove /etc/nologin or not
# Return value: usual status thingy
# Example: Cobalt::Services::set_telnet_server_open(1);
{
    my ($newstate) = @_;
    my ($nowstate) = (! -e $Nologin);
    if (($newstate && $nowstate) ||
	((! $newstate) && (! $nowstate)) )
    {
	return MSG_ok("2458");
    }
    elsif ($newstate)
    {
	# open 'er up!
	unlink($Nologin);

	# fix the boot script so that nologin gets removed at
	# boot time
	return MSG_error("3411")
	  if (-e "$Lockdir/rc.sysinit");
	open(SO,">$Lockdir/rc.sysinit")
	    || return MSG_error("4419", $Sysinit_script);
	open(SI,"$Sysinit_script")
	    || return MSG_error("4418", $Sysinit_script);
	while (<SI>)
	{
	    if (/^rm -f \/etc\/mtab~ \/fastboot/o)
	    {
		chomp $_;
		$_ .= " $Nologin\n";
	    }
	    print SO;
	}
	close SI;
	close SO;
	rename("$Lockdir/rc.sysinit","$Sysinit_script")
	    || return MSG_error("4417", $Sysinit_script);
	chmod(0755,"$Sysinit_script");
	return MSG_ok("2460");
    }
    else
    {
	# close it down
	open(N,">$Nologin") 
	    || return MSG_error("4429", $Nologin);
	close N;

	# fix the boot script so that nologin won't get clobbered
	# on bootup
	return MSG_error("3410")
	  if (-e "$Lockdir/rc.sysinit");
	open(SO,">$Lockdir/rc.sysinit") 
	    || return MSG_error("4435", $Sysinit_script);
	open(SI,"$Sysinit_script") 
	    || return MSG_error("4431", $Sysinit_script);
	while (<SI>)
	{
	    if (/$Nologin/o)
	    {
		$_ =~ s/\s*$Nologin//og;
	    }
	    print SO;
	}
	close SI;
	close SO;
	rename("$Lockdir/rc.sysinit","$Sysinit_script")
	    || return MSG_error("4422", $Sysinit_script);
	chmod(0755,"$Sysinit_script");
	return MSG_ok("2459");
    }
}

sub get_telnet_server_open
# Check wether telnet access is open
# Arguments: none
# Return value: 0 if only root can log in, 1 if anyone can
# Example: print "Anyone can log in\n" if (Cobalt::Services::get_telnet_server_open());
{
    (-e $Nologin) ? (return 0) : (return 1);
}

sub set_smb_workgroup
# Set the workgroup for the SMB server (samba)
# Arguments: the workgroup name
# Side effects: modifies smb.conf and restarts samba
# Return value: usual status thingy
# Example: Cobalt::Services::set_smb_workgroup("workgroup");
{
    return MSG_error("1106")
      unless ($Features{'smb'});

    my ($workgroup) = @_;
    return MSG_error("4464") if (-e "$Lockdir/smb.conf");
    open(SNB,">$Lockdir/smb.conf") 
	|| return MSG_error("4471", $Lockdir);
    open(SMB,"$Smbconf") 
	|| return MSG_error("3475", $Smbconf);
    while (<SMB>)
    {
	# Suppose you're thinking of a plate of shrimp...
	if (/^\s*workgroup = (\w+)/o)
	{
	    # $oldworkgroup = $1;
	    $_ = "   workgroup = $workgroup\n";
	}
	print SNB;
    }
    close SMB;
    close SNB;
    rename("$Lockdir/smb.conf",$Smbconf) ||
	return MSG_error("3467", $Smbconf);

    # Start a child process to restart the samba server
    my $pid;
    if (!defined($pid = fork)) {
	die "cannot fork: $!";
    } elsif ($pid) {
	# i'm the parent
	print STDERR "forking a child to restart daemons\n" if ($Debug);
    }
    else
    {
	# otherwise I'm the child
	open(STDERR,">>$Sauce_log");
	open(STDOUT,">>$Sauce_log");
	# restart samba
	if (-f "$Subsys/smb")
	{
	    system("$Smbscript stop >>$Sauce_log 2>&1 ");
	    system("$Smbscript start >>$Sauce_log 2>&1 ");
	}
	system ("$Lcdshowscript start >>$Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	exit;
    }

    # done!
    return MSG_ok("2413", $workgroup);
}

sub get_smb_workgroup
# Get the workgroup for the SMB server (samba)
# Arguments: none
# Side effects: modifies smb.conf and restarts samba
# Return value: the name of the workgroup
# Example: $workgroup = Cobalt::Services::get_smb_workgroup();
{
    my ($workgroup);
    open(SMB,"$Smbconf") 
	|| return MSG_error("3471", $Smbconf);
    while (<SMB>)
    {
	if (/\s*workgroup = (\w+)/o)
	{
	    $workgroup = $1;
	    last;
	}
    }
    close SMB;
    return $workgroup;
}

sub set_smb_controller
# Set the domain server for the SMB server (samba), or disable domain logins
# Arguments: a boolean of wether to use a domain server (1 means do use it), 
#   the domain server name
# Side effects: modifies smb.conf and restarts samba
# Return value: usual status thingy
# Examples: Cobalt::Services::set_smb_controller(1,"corp-pdc");
#           Cobalt::Services::set_smb_controller(0,"");
{
    return MSG_error("1107")
      unless ($Features{'dns'});

    my ($usecontroller,$server) = @_;
    my $security;
    ($usecontroller) ? ($security = "server") : ($security = "user");
    return MSG_error("4467") if (-e "$Lockdir/smb.conf");
    open(SNB,">$Lockdir/smb.conf") 
	|| return MSG_error("4468", $Lockdir);
    open(SMB,"$Smbconf") 
	|| return MSG_error("3469", $Smbconf);
    while (<SMB>)
    {
	# Joe, you're kind of a Philistine, aren't you?
	if (/^\s*security = (.+)/o)
	{
	    $_ = "   security = $security\n";
	}
	elsif (/^;?\s*password server = (.*)/o)
	{
	    my $oldserver = $1;
	    if ($usecontroller)
	    {
		$_ = "   password server = $server\n";
	    }
	    else
	    {
		$_ = ";   password server = $server\n";  
	    }
	}
	print SNB;
    }
    close SMB;
    close SNB;
    rename("$Lockdir/smb.conf",$Smbconf) ||
	return MSG_error("3470", $Smbconf);

    # Start a child process to restart the samba server
    my $pid;
    if (!defined($pid = fork)) {
	die "cannot fork: $!";
    } elsif ($pid) {
	# i'm the parent
	print STDERR "forking a child to restart daemons\n" if ($Debug);
    }
    else
    {
	# otherwise I'm the child
	open(STDERR,">>$Sauce_log");
	open(STDOUT,">>$Sauce_log");
	# restart samba
	if (-f "$Subsys/smb")
	{
	    system("$Smbscript stop >>$Sauce_log 2>&1 ");
	    system("$Smbscript start >>$Sauce_log 2>&1 ");
	}
	system ("$Lcdshowscript start >>$Sauce_log 2>&1")
	    if (-x $Lcdshowscript);
	exit;
    }

    # done!
    $server ||= "off"; # default so the message makes sense
    return MSG_ok("2414", $server);
}

sub get_smb_controller
# Get the domain controller for the SMB server (samba)
# Arguments: none
# Side effects: modifies smb.conf and restarts samba
# Return value: an array containing a boolean of wether samba is using a
#    server, and the name of the server
# Example: ($using_domain_controller,$server) = Cobalt::Services::get_smb_controller();
#          print "Server is $server\n" if ($using_domain_controller);
{
    my ($security,$server);
    open(SMB,"$Smbconf") 
	|| return MSG_error("3472", $Smbconf);
    while (<SMB>)
    {
	if (/^\s*security = (.+)/o)
	{
	    ($1 eq "user") ? ($security = 0) : ($security = 1);
	}
	elsif (/^\;?\s*password server = (.+)/o)
	{
	    $server = $1;
	}
    }
    close SMB;
    $server = "" if ($server =~ /\</o);
    return ($security,$server);
}

sub set_snmp_server_on
# Turn the snmp server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form "911 Invalid hormone balance."
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
    my ($state) = @_;
    my $istate = service_get_init("snmpd");

    return MSG_error("1403")
      unless ($Features{'snmp'});

    if ($state && (! $istate))
    {
        my $pid;
        if (!defined($pid = fork)) {
            die "cannot fork: $!";
        } elsif ($pid) {
            # i'm the parent
            print STDERR "Forking to restart snmpd\n" if ($Debug);
            return MSG_ok("2461");
        }
        else
        {
            # otherwise I'm the child
            open(STDERR,">>$Sauce_log");
            open(STDOUT,">>$Sauce_log");
	    service_set_init_on("snmpd")
		|| die MSG_error("4487", $SNMPlink);
            system ("$SNMPscript start >> $Sauce_log 2>&1") &&
	      die MSG_error("3498", $!);
            system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
            exit;
        }
    }
    elsif ((! $state) && ($istate))
    {
	service_set_init_off("snmpd")
            || return MSG_error("4488", $SNMPlink);
        system ("$SNMPscript stop >> $Sauce_log 2>&1") &&
            return MSG_error("3499", $!);
        system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
        return MSG_ok("2462");
    }
    else
    {
        my $off;
        ($state) ? ($off = MSG_get("On_string")) : 
	  ($off = MSG_get("Off_string"));
        return MSG_ok("2463", $off);
    }
}

sub get_snmp_server_on
# Is the snmp server set to be on?
# Returns 1 if the snmp server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    return service_get_init("snmpd");
}

sub set_snmp_community
# Sets the community with read and write access
# Arguments: read-only community, read/write community
#   If a community name is set to empty, then disable that community
# Return value: a status string of the form "666 Got a raise. Blow a hole in the account."
# Example: $ret = set_snmp_community("mypublic","");  (this disables the 
#   r/w community and sets the read-only community to "mypublic").
{
    return MSG_error("1404")
	unless ($Features{'snmp'});

    my ( $read_community, $write_community )=@_;

    # The SNMP RFC just says that a community should be an "octet string";
    # but I know that would break the config file, so I'm going to
    # restrict to alphanumeric, underscore, dash, and dots until I find 
    # documenation to say otherwise.
    return MSG_error("services_snmp_invalid_read_community", $read_community)
      if ($read_community !~ /^[\w\.\-]*$/o);
    return MSG_error("services_snmp_invalid_write_community", $write_community)
      if ($write_community !~ /^[\w\.\-]*$/o);

    return MSG_error("4472", $SNMPconf) if ( -e "$Lockdir/snmp.conf" );
    open( SNMPLOCK, ">$Lockdir/snmp.conf" )
	|| return MSG_error("4473", $Lockdir);
    open( SNMP, "$SNMPconf") 
	|| return MSG_error("3477", $SNMPconf);
    my $done=0;
    while( <SNMP> ) {
      if (/^[\#\s]*community\s+(\S+)\s+(xmini.+)/o ) {
	# Change the read-only community
	if ($read_community) {
	  $_ = "community\t$read_community\t$2\n";
	}
	elsif (/^c/o) {
	  # only comment it out if it isn't already
	  $_ = "#$_";
	}
      } 
      elsif (/^[\#\s]*community\s+(\S+)\s+(mini.+)/o) {
	# Change the read/write community (commented out by default)
	if ($write_community) {
	  $_ = "community\t$write_community\t$2\n";
	}
	elsif (/^c/o) {
	  # only comment it out if it isn't already
	  $_ = "#$_";
	}
      }
      elsif (/^trap community:/o) {
	# Trap community is not documented.  We don't want to just
	# leave it as "public" if we changed community names...
	my $trap_community;
	# first try setting it to the write community
	$trap_community = $write_community;
	# if write_community wasn't set, set it to the read community
	$trap_community ||= $read_community;
	# otherwise...well, set it to something.  No communities are
	# set, so we'll set it to the default of "public".
	$trap_community ||= "public";

	$_ = "trap community:\t$trap_community\n";
      }
      print SNMPLOCK;
    }
    close SNMPLOCK;
    close SNMP;
    rename( "$Lockdir/snmp.conf", $SNMPconf ) ||
	return MSG_error("3478", $SNMPconf);
    chmod(0600, "$SNMPconf");

    # Start a child process to restart the SNMP agent
    my $pid;
    if( !defined( $pid=fork ) ) { die "cannot fork: $!"; }
    elsif( $pid ) {
	# i'm the parent
	print STDERR "forking a child to restart daemons\n" if ($Debug);
    }
    else {
	# otherwise I'm the child
	open( STDERR, ">>$Sauce_log" );
	open( STDOUT, ">>$Sauce_log" );
	# restart SNMP
	if( -f "$Subsys/snmpd" ) {
	    system( "$SNMPscript stop >>$Sauce_log 2>&1 " );
	    system( "$SNMPscript start >>$Sauce_log 2>&1 " );
	}
	system( "$Lcdshowscript start >>$Sauce_log 2>&1" )
	    if( -x $Lcdshowscript );
	exit;
    }

    # done!
    return MSG_ok("services_snmp_ok_community_set");
}

sub get_snmp_community
# Gets the names of the SNMP communities
# Arguments: none
# Return value: the read-only community, the read/write community
#   If a community name is not set, then we return 0.
{
    my ($read_community,$write_community);
    $read_community ||= '';   # default in case there is none set
    $write_community ||= '';  # default in case there is none set
    open( SNMP, "<$SNMPconf" )
	|| return MSG_error("3479", $SNMPconf);
    while( <SNMP> ) {
	if( /^\s*community\s+(\S+)\s+xmini/o ) {
	    $read_community = $1;
	    # if we know both, then we're done
	    last if ($write_community);
	}
	elsif( /^\s*community\s+(\S+)\s+mini/o ) {
	    $write_community = $1;
	    # if we know both, then we're done
	    last if ($read_community);
	}
    }
    close SNMP;
    return ($read_community,$write_community);
}

sub get_arkeia_servername
# Gets the name of the Arkeia backup server
# Arguments: none
# Return value: the backup server name 
{
  my $servername;
  $servername ||= '';
  open( ARKEIA, "<$Arkeia_conf" )
      || return MSG_error("arkeia_noconf", $ArkeiaConf);
  $servername = <ARKEIA>;
  close ARKEIA;
  return $servername;

}

sub set_arkeia_servername
# Gets the name of the Arkeia backup server
# Arguments: the backup server 
# Return value: none
{
  my $servername = shift;
  open( ARKEIA, ">$Arkeia_conf" )
      || return MSG_error("arkeia_noconf", $ArkeiaConf);
  print ARKEIA $servername;  
  close ARKEIA;
  return MSG_ok( "services_arkeia_ok_servername_set" );

}

sub set_ntp_server_on
# Turn the ntp server on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form MSG_error("1132")
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
  my ($state) = @_;
  my $istate = service_get_init("xntpd");

  if (! $Features{'ntp'} )
  {
    return MSG_error("1109");
  } 
  else 
  {

    if ($state && (! $istate))
    {
        my $pid;
        if (!defined($pid = fork)) {
            die "cannot fork: $!";
        } elsif ($pid) {
            # i'm the parent
            print STDERR "Forking to restart NTP server\n" if ($Debug);
            return MSG_ok("2502");
        }
        else
        {
            # otherwise I'm the child
            open(STDERR,">>$Sauce_log");
            open(STDOUT,">>$Sauce_log");
	    service_set_init_on("xntpd")
		|| die MSG_error("4489", $NTPlink);
            system ("$NTPscript start >> $Sauce_log 2>&1") &&
	      die MSG_error("3500", $!);
            system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
            exit;
        }
    }
    elsif ((! $state) && ($istate))
    {
        service_set_init_off("xntpd")
            || return MSG_error("4490", $NTPlink);
        system ("$NTPscript stop >> $Sauce_log 2>&1") &&
            return MSG_error("3501", $!);
        system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	  if (-x $Lcdshowscript);
        return MSG_ok("2503");
    }
    else
    {
        my $off;
        ($state) ? ($off = MSG_get("On_string")) : 
	  ($off = MSG_get("Off_string"));
        return MSG_ok("2504", $off);
    }
  }
}

sub get_ntp_server_on
# Is the ntp server set to be on?
# Returns 1 if the ntp server is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    return service_get_init("xntpd");
}

sub set_networker_client_on
# Turn the Legato Networker client on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form MSG_ok("2700")
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
  my ($state) = @_;

  if (! $Features{'networker'} )
  {
    return MSG_error("services_networker_unsupported");
  } 
  else 
  {
    if ($state && (! -e $Networker_link))
    {
        my $pid;
        if (!defined($pid = fork)) {
            die "cannot fork: $!";
        } elsif ($pid) {
            # i'm the parent
            print STDERR "Forking to restart Legato Networker client\n"
	      if ($Debug);
            return MSG_ok("services_networker_ok_started");
        }
        else
        {
            # otherwise I'm the child
            open(STDERR,">>$Sauce_log");
            open(STDOUT,">>$Sauce_log");
            symlink($Networker_script,$Networker_link) ||
	      die MSG_error("services_networker_cant_link", $Networker_link);
            system ("$Networker_script start >> $Sauce_log 2>&1") &&
	      die MSG_error("services_networker_cant_start", $!);
            system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
            exit;
        }
    }
    elsif ((! $state) && (-e $Networker_link))
    {
        unlink($Networker_link) ||
            return MSG_error("services_networker_cant_unlink", $Networker_link);
        system ("$Networker_script stop >> $Sauce_log 2>&1") &&
            return MSG_error("services_networker_cant_stop", $!);
        system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	  if (-x $Lcdshowscript);
        return MSG_ok("services_networker_ok_stopped");
    }
    elsif ($state)
    {
      return MSG_ok("services_networker_ok_alreadyon");
    }
    else
    {
      return MSG_ok("services_networker_ok_alreadyoff");
    }
  }
}

sub get_networker_client_on
# Is the Legato Networker client set to be on?
# Returns 1 if the client is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $Networker_link) ? (return 1) : (return 0);
}

sub set_fpx_on
# Enable or disable FrontPage Server Extensions on the main site
# Arguments: 1 for on/0 for off
# Return value: a status string of the usual form form MSG_ok("2534") 
# Side effects: Calls MS's fpsrvadm.exe, cats /etc/htpasswd
# Examples: $good = Cobalt::Services::set_fpx_on(0);
#            $bad = Cobalt::Services::set_fpx_on(1);
{
  my $state = shift(@_);
  my $fpx_active = get_fpx_on();

  if (! $Features{'fpx'} )
  {
    return MSG_error("services_fpx_unsupported");
  }

  unless (-x "$Fpx_admsrvexe")
  {
    return MSG_error("services_fpx_bad_fpxnotavail");
  } 
  else 
  {

    if ($state && (! $fpx_active))
    {
	# Install

	# Temporarily move .htaccess files
	my(@htax,$hta);
	open(HTAS,"/usr/bin/find $Htgroup_dir/$MainGroup -name .htaccess -print |");
	while(<HTAS>) { chop; push(@htax,$_); }
	foreach $hta (@htax) 
	{
	    system("/bin/mv $hta $hta.FP98") unless ($hta eq "$Htgroup_dir/$MainGroup/.htaccess");
	}

	# Can't return after failure without .htaccess restoer
	my($could_not_install);
	
	# Call fpadmserv.exe (ugh)
	system("$Fpx_admsrvexe -o install -p 80 -t apache -m \"\" -u \"$Adminuser\" -pw \"$$\" -s \"$Htconf/httpd.conf\" -xu \"$WWWuser\" -xg \"$MainGroup\" > /dev/null")
	    && ($could_not_install = 1);

	# Restore .htaccess files asap!
	foreach $hta (@htax) 
	{
	    system("/bin/mv $hta.FP98 $hta") if (-e "$hta.FP98");
	}
	return MSG_error("services_fpx_bad_could_not_install") if ($could_not_install);

	# chmod u+s g+s fpcount.exe
	chmod(06555,"$Htgroup_dir/$MainGroup/_vti_bin/fpcount.exe");

	# qube2/raq2 fpx configurations are different! don't mess w/ fpx
        # if you hate m-soft.
	unless ($Features{'pam'}) {
	  # Use admin's password, only sure user
	  open(HTPAS,"$Htpasswd") 
	    || return MSG_error("services_fpx_bad_nosetpassword");
	  open(MSHT,"> $Htgroup_dir/$MainGroup/_vti_pvt/service.pwd") 
	    || return MSG_error("services_fpx_bad_nosetpassword");
	  print MSHT "# -FrontPage-\n";
	  while(<HTPAS>) {
	    print MSHT if (/^$Adminuser\:/o);
	  }
	  close(MSHT);
	  close(HTPAS);
	  chown ( (getpwnam($WWWuser))[2], (getgrnam($MainGroup))[2], 
		  "$Htgroup_dir/$MainGroup/_vti_pvt/service.pwd" );
	}

	# Configure SMTP sender for fpx cgi files
	if (-w $Fpx_we80cnf && !$could_not_install)
	{ 
		# We first check to see if fpadmsrv could somehow have configured smtp
		# It's never happened before, but who knows how it may behave in the future
		# (feature requested by Kevin 01/11/99 /wdh)
		my($SMTPrelay)=0;
		open(FPCNF,"$Fpx_we80cnf") || return MSG_error("services_fpx_bad_nosmtp");
		while(<FPCNF>) { $SMTPrelay=1 if (/SMTPHost/o); }
		
		unless($SMTPrelay) # Append the SMTPHost: directive
		{
			open(FPCNF,">>$Fpx_we80cnf") || return MSG_error("services_fpx_bad_nosmtp");
			print FPCNF "SMTPHost:127.0.0.1\n";
			close(FPCNF);
		}
	}

        return MSG_ok("services_fpx_ok_installed");
    }
    elsif ((! $state) && $fpx_active)
    {
	# Uninstall
	# Call fpadmserv.exe (ugh)
	system("$Fpx_admsrvexe -o uninstall -p 80 -t apache -m \"\" -s \"$Htconf/httpd.conf\" > /dev/null")
	    && return MSG_error("services_fpx_bad_could_not_uninstall");
	
	# Already done by fpadmsrv unless something went wrong AND it returned 0 anyways
	# (feature requested by Kevin 01/11/99 /wdh)
	unlink($Fpx_we80cnf) if (-e $Fpx_we80cnf); 
        
	return MSG_ok("services_fpx_ok_uninstalled");
    }
    elsif ($state)
    {
      return MSG_ok("services_fpx_ok_alreadyon");
    }
    else
    {
      return MSG_ok("services_fpx_ok_alreadyoff");
    }
  }
}

sub get_fpx_on
# Are FrontPage Server Extensions enabled on the main site?
# Returns 1 if installed, 0 if uninstalled or never installed 
#   Tests for the existance of we80.cnf
# Arguments: none
# Side effects: none
{
    (-e $Fpx_we80cnf) ? (return 1) : (return 0);
}

sub set_gated_on
# Turn the Gate Daemon (gated) on or off
# Arguments: 1 for on/0 for off; optional list of interfaces on 
#    which to listen
# Return value: a status string of the form "2534 The doggy in the 
#    window is $10"
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
# Examples: $ret = Cobalt::Services::set_gated_on(0);
#           $ret = Cobalt::Services::set_gated_on(1,"eth0","eth1");
{
  my $state = shift(@_);

  if (! $Features{'gated'} )
  {
    return MSG_error("services_gated_unsupported");
  } 
  else 
  {
    # If we're given a list of interfaces, fix gated.conf
    if ($#_ >= 0) {
      my $ret = Cobalt::Services::set_gated_interfaces(@_);
      return $ret if ($ret !~ /^2/o);
    }

    if ($state && (! -e $Gated_link))
    {
        my $pid;
        if (!defined($pid = fork)) {
            die "cannot fork: $!";
        } elsif ($pid) {
            # i'm the parent
            print STDERR "Forking to restart gated\n"
	      if ($Debug);
            return MSG_ok("services_gated_ok_started");
        }
        else
        {
            # otherwise I'm the child
            open(STDERR,">>$Sauce_log");
            open(STDOUT,">>$Sauce_log");
            symlink($Gated_script,$Gated_link) ||
	      die MSG_error("services_gated_cant_link", $Gated_link);
            system ("$Gated_script start >> $Sauce_log 2>&1") &&
	      die MSG_error("services_gated_cant_start", $!);
            system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	      if (-x $Lcdshowscript);
            exit;
        }
    }
    elsif ((! $state) && (-e $Gated_link))
    {
        unlink($Gated_link) ||
            return MSG_error("services_gated_cant_unlink", $Gated_link);
        system ("$Gated_script stop >> $Sauce_log 2>&1") &&
            return MSG_error("services_gated_cant_stop", $!);
        system ("$Lcdshowscript start >> $Sauce_log 2>&1")
	  if (-x $Lcdshowscript);
        return MSG_ok("services_gated_ok_stopped");
    }
    elsif ($state)
    {
      return MSG_ok("services_gated_ok_alreadyon");
    }
    else
    {
      return MSG_ok("services_gated_ok_alreadyoff");
    }
  }
}

sub get_gated_on
# Is gated set to be on?
# Returns 1 if the daemon is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $Gated_link) ? (return 1) : (return 0);
}

sub set_gated_interfaces
# Set the list of interfaces on which gated will listen
# Arguments: list of interfaces
# Side effects: modifies gated.conf
# Return value: return string of the form MSG_ok("2149")
# Example: $ret = Cobalt::Services::set_gated_interfaces("eth0","eth1");
{
  my %netifs = map { $_, 1 } @_;   # make a hash of the interface list
  return MSG_error("services_gated_cant_locked")
    if (-e "$Lockdir/gated.conf");
  open(OUT,">$Lockdir/gated.conf") || return MSG_error("services_gated_cant_lock");
  unless (open(IN,$Gated_conf)) {
    unlink("$Lockdir/gated.conf");
    return MSG_error("services_gated_cant_read_config", $Gated_conf);
  }
  my $inrip;
  while (<IN>) {
    if (/^rip yes/o) {
      $inrip = 1;
      print OUT;
    }
    elsif ($inrip && /\};/o) {
      my $netif;
      foreach $netif (sort keys %netifs) {
	print OUT "        interface $netif ripin ripout version 2 broadcast;\n"
	  if ($netifs{$netif} == 1);
      }
      $inrip = 0;
      print OUT;
    }
    elsif ($inrip && /^\s*interface all/o) {
      # the default interface entry
      print OUT;
    }
    elsif ($inrip && /^\s*interface (\S+)\s/o) {
      print OUT if ($netifs{$1} == 1);
      $netifs{$1} = 0;
    }
    else {
      print OUT;
    }
  }
  close IN;
  close OUT;
  rename("$Lockdir/gated.conf",$Gated_conf);
  return MSG_ok("services_gated_ok_setifs", $Gated_conf);
}

sub get_gated_interfaces
# Get the list of interfaces on which gated will listen
# Arguments: none
# Side effects: reads gated.conf
# Return value: list of interfaces ("eth0","eth1")
# Example: @netifs = Cobalt::Services::get_gated_interfaces();
{
  open(IN,$Gated_conf) || 
    return MSG_error("services_gated_cant_read_config", $Gated_conf);
  my $inrip;
  my @netifs;
  while (<IN>) {
    if (/^rip yes/o) {
      $inrip = 1;
    }
    elsif ($inrip && /\};/o) {
      last;
    }
    elsif ($inrip && /^\s*interface all/o) {
      # the default interface entry
      next;
    }
    elsif ($inrip && /^\s*interface (\S+)/o) {
      push(@netifs,$1);
    }
  }
  close IN;
  return @netifs;
}

sub set_arkeia_client_on
# Turn the Arkeia client on or off
# Arguments: 1 for on, 0 for off
# Return value: a status string of the form MSG_ok("2700")
# Side effects: creates or breaks symlinks in /etc/rc.d/rc3.d
{
  my ($state) = @_;

  if (! $Features{'arkeia'} )
  {
    return MSG_error("services_arkeia_unsupported");
  }
  else
  {
    if ($state && (! -e $Arkeia_link))
    {
        my $pid;
        if (!defined($pid = fork)) {
            die "cannot fork: $!";
        } elsif ($pid) {
            # i'm the parent
            print STDERR "Forking to restart Arkeia client\n"
              if ($Debug);
            return MSG_ok("services_arkeia_ok_started");
        }
        else
        {
            # otherwise I'm the child
            open(STDERR,">>$Sauce_log");
            open(STDOUT,">>$Sauce_log");
            symlink($Arkeia_script,$Arkeia_link) ||
              die MSG_error("services_arkeia_cant_link", $Arkeia_link);
            system ("$Arkeia_script start >> $Sauce_log 2>&1") &&
              die MSG_error("services_arkeia_cant_start", $!);
            system ("$Lcdshowscript start >> $Sauce_log 2>&1")
              if (-x $Lcdshowscript);
            exit;
        }
    }
    elsif ((! $state) && (-e $Arkeia_link))
    {
        unlink($Arkeia_link) ||
            return MSG_error("services_arkeia_cant_unlink", $Arkeia_link);
        system ("$Arkeia_script stop >> $Sauce_log 2>&1") &&
            return MSG_error("services_arkeia_cant_stop", $!);
        system ("$Lcdshowscript start >> $Sauce_log 2>&1")
          if (-x $Lcdshowscript);
        return MSG_ok("services_arkeia_ok_stopped");
    }
    elsif ($state)
    {
      return MSG_ok("services_arkeia_ok_alreadyon");
    }
    else
    {
      return MSG_ok("services_arkeia_ok_alreadyoff");
    }
  }
}

sub get_arkeia_client_on
# Is the Arkeia client set to be on?
# Returns 1 if the client is activated, 0 if deactivated
#   This reflects what ought to be, *not* what is
# Arguments: none
# Side effects: none
{
    (-e $Arkeia_link) ? (return 1) : (return 0);
}


1;
