#  ___________________________________________________________
# /\                                                          \
# \_| Ssl.pm: Functions for manging SSL certificates.         |
#   |                                                         |
#   | $Id: Ssl.pm,v 1.20 1999/10/04 11:58:56 harris Exp $      |
#   |                                                         |
#   | Written by                                              |
#   |   Harris Vaegan-Lloyd <harris@cobaltnet.com>            |
#   |                                                         |
#   | Copyright 1999 Cobalt Networks http://www.cobaltnet.com |
#   |   ______________________________________________________|_
#    \_/________________________________________________________/


package Cobalt::Ssl;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
@ISA	= qw(Exporter SelfLoader);
@EXPORT = qw( ssl_set_cert
			ssl_get_cert
			ssl_set_identity
			ssl_get_cert_info
			ssl_cert_check
			ssl_check_valid_cert
			ssl_clear
			);

require Cobalt::Language;
import Cobalt::Language qw(:DEFAULT);
use Cobalt::Util;
use Cobalt::Product;

1;
__DATA__

sub ssl_set_cert
#  _____________________________________________________________________
# /\                                                                    \
# \_| Arguments: A certificate directory root.                          |
#   |            A string containing the new certificate.               |
#   |   ________________________________________________________________|_
#    \_/__________________________________________________________________/
{
	my $cert = shift;
	my $cert_root = shift;

	$cert =~ s/\r//g;

	$cert =~ m#(-----BEGIN RSA PRIVATE KEY-----.*-----END RSA PRIVATE KEY-----)#s;
	my $key = $1;
	$cert =~ m#(-----BEGIN CERTIFICATE-----.*-----END CERTIFICATE-----)#s;
	$cert = $1;

	print STDERR "Cert: $cert\n Key: $key\n" if $Debug;

	# First off we must chack if this is a valid certificate. The cgi may
	# have passed up just anything.
	if ( ssl_check_valid_cert($cert) !~ /^2/ ) {
		return MSG_error("vsite_invalid_cert");
	}

	# We make a backup of a certificate we're about to remove.
	if ( -e "$cert_root/certificate" ) {
		rename ("$cert_root/certificate",
				"$cert_root/certificate.bak") ||
			return MSG_error("vsite_cant_backup_cert");
	}

	if ( $key ) {
		if ( -e "$cert_root/key" ) {
			rename ("$cert_root/key",
					"$cert_root/key.bak") ||
				return MSG_error("vsite_cant_backup_cert");
		}

		open (KEY, ">$cert_root/key") ||
			return MSG_error("vsite_cant_write_cert");

		print KEY $key;

		close KEY;
		chmod 0660, "$cert_root/key";
	}

	open( CERTIFICATE, ">$cert_root/certificate") ||
		return MSG_error("vsite_cant_write_cert");

	print CERTIFICATE $cert;

	close CERTIFICATE;
	chmod 0660, "$cert_root/certificate";

	return MSG_ok("vsite_ok_set_cert");
}

sub ssl_get_cert
#  ________________________________________________________________
# /\                                                               \
# \_| Arguments: A optional root directory from which to fetch the |
#   |                        certificate, req and key.             |
#   | Returns: ( $cert, $req, $key )                               |
#   |                   Null strings for non-present files.        |
#   |   ___________________________________________________________|_
#    \_/_____________________________________________________________/
{
	my $cert_root = shift;
	# Default to /etc/httpd/ssl/
	$cert_root ||= $Htssldir;

	my @certificate;

    if ( open(CERTIFICATE, "$cert_root/certificate") ) {
        # Simply return a blank string if no request has yet been
        # generated;
        $certificate[0] = join('',<CERTIFICATE>);
		close CERTIFICATE;
    }
    if ( open(REQUEST, "$cert_root/request") ) {
        # Simply return a blank string if no request has yet been
        # generated;
        $certificate[1] = join('',<REQUEST>);
		close REQUEST;
    }
    if ( open(KEY, "$cert_root/key") ) {
        # Simply return a blank string if no request has yet been
        # generated;
        $certificate[2] = join('',<KEY>);
		close KEY;
    }
	return @certificate;
}

sub ssl_set_identity
#  _____________________________________________________________
# /\                                                            \
# \_| Arguments: Number of days the certificate is valid for.   |
#   |            A country.                                     |
#   |            State or province name.                        |
#   |            Locality Name (City.)                          |
#   |            Organisation Name.                             |
#   |            Organisation Unit.                             |
#   |            Fully Qualified Domain Name.                   |
#   |            Email address of maintainer.                   |
#   |           An optional root certificate dir.               |
#   | Returns: The usual. But with mustard this time.           |
#   |   ________________________________________________________|_
#    \_/__________________________________________________________/
{
	my %subject;

	my $days = shift;
	$subject{'C'} = shift;
	$subject{'ST'} = shift;
	$subject{'L'} = shift;
	$subject{'O'} = shift;
	$subject{'OU'} = shift;
	$subject{'CN'} = shift;
	$subject{'Email'} = shift;
	my $cert_dir = shift;
	# Again, default to /etc/httpd/ssl if no root given.
	$cert_dir ||= $Htssl_dir;

	if ( $subject{'Email'} !~ /\@/ ) {
		return MSG_error('invalid_emailAddr');
	}

	# We're starting from scratch here.. Make backups if we're going
	# to be overwriting something.
	for my $file ( qw(request certificate key) ) {
		if( -f "$cert_dir/$file") {
			rename("$cert_dir/$file","$cert_dir/$file.bak");
		}
	}

	# Ugh. I hate having to do system calls.
	if ( system("$Openssl_cmd genrsa -out $cert_dir/key 1024 1>&2") ) {
		return MSG_error("ssl_cant_gen_new_key");
	}

	open (REQ, "|$Openssl_cmd req -new -config /usr/lib/openssl.cnf "
		. "-key $cert_dir/key -days $days -out $cert_dir/request 1>&2") ||
		return MSG_error("ssl_cant_gen_request");

	print REQ <<USER_INFO;
$subject{'C'}
$subject{'ST'}
$subject{'L'}
$subject{'O'}
$subject{'OU'}
$subject{'CN'}
$subject{'Email'}
USER_INFO
	;
	close REQ;	

	if( system("$Openssl_cmd x509 -days $days -req -signkey $cert_dir/key "
		  ."-in $cert_dir/request -out $cert_dir/certificate 1>&2") ) {
		return MSG_error("ssl_cant_gen_cert");
	}

	chmod 0660, "$cert_dir/key";
	chmod 0660, "$cert_dir/certificate";
	chmod 0660, "$cert_dir/request";

	unless( ssl_cert_check($cert_dir) =~ /^2/) {
		return MSG_error("ssl_cant_created_invalid");
	}

	return MSG_ok("ssl_ok_cert_created");
}

sub ssl_get_cert_info
#  ___________________________________________________________
# /\                                                          \
# \_| Arguments: An optional certificate root directory       |
#   | Returns: Refrence to hash of subject information.       |
#   |          Refrence to hash of issuer information.        |
#   |          Array of ( year, monty, day ) for expiry date. |
#   | e.g. ( $issuer, $subject @expiry) = ssl_get_cert_info   |
#   |   ______________________________________________________|_
#    \_/________________________________________________________/
{
	my $cert_root = shift;
	$cert_root ||= $Htssldir;
	my %subject;
	my %issuer;
	my @date;

	# Ugh, backticks.. No other options unfortunately..
	open(SSL_CMD, "$Openssl_cmd x509 -text -noout  < $cert_root/certificate 2>&1 |") ||
		die MSG_error("vsite_cant_run_openssl_x509");

	while (<SSL_CMD>) {
		my $line = $_;
		if (/^\s+Subject:/o) {
			# Strip off the subject;
			$line =~ s/^\s+Subject://;
			while ( m:(\w+)=([^\,/]+)[,/]*:go) {
				$subject{$1} = $2;
			}
		} elsif (/^\s+Issuer:/o) {
			$line =~ s/^\s+Issuer://;
			while ( m:(\w+)=([^\,/]+)[,/]*:go) {
				$issuer{$1} = $2;
			}
		} elsif (/Not After : (\S+)\s(\d+)\s+\S+\s+(\d+)/) {
			@date = ( $3, $1, $2); # year month day;
		}
	}
	close SSL_CMD;
	return \%subject, \%issuer, @date;
}

sub ssl_check_valid_cert
# Arguemnts: A certificate in PEM form.
# Returns: True for a valid certificate. False otherwise.
{
	my $cert = shift;
	# We're not intrested in output here, just the return value.
	open(X509,"|$Openssl_cmd x509 2> /dev/null > /dev/null")
		|| die MSG_error("ssl_cant_run_openssl_x509");
	
	print X509 $cert;
	if ( close X509 ) {
		# Closed without errors, that's a valid certificate allright.
		return MSG_ok("ssl_ok_certificate_valid");
	} else {
		return MSG_error("ssl_cant_certificate_invalid");
	}
}

sub ssl_cert_check
# Arguments: An options certificate root.
# Returns: True for a valid certificate/key pair. False otherwise.
# Encrypted keys are considered invalid.
{

	my $cert_dir=shift;
	# First check that all the necessary files exist.
	if (! -f "$cert_dir/certificate" || ! -f "$cert_dir/key" ) {
		return MSG_error("ssl_cant_files_missing");
	}

	# Next check if the key is encrypted.
	open(KEY,"$cert_dir/key") ||
		return MSG_error("ssl_cant_read_key");

	while (<KEY>) {
		if (/ENCRYPTED/o) {
			close KEY;
			return MSG_error("ssl_cant_encrypted_key");
		}
	}
	close KEY;

	open(TMP_KEY,">$Lockdir/temp.key.$$") ||
		return MSG_error("ssl_cant_write_temp_cert");

	unless ( open(CERT,"<$cert_dir/certificate") ) {
		close TMP_KEY;
		unlink("$Lockdir/temp.key.$$");
		return MSG_error("ssl_cant_read_certificate");
	}

	unless ( open(KEY,"<$cert_dir/key") ) {
		close CERT;
		close TMP_KEY;
		unlink("$Lockdir/temp.key.$$");
		return MSG_error("ssl_cant_read_key");
	}

	while(<CERT>) { print TMP_KEY; }
	# Need a new line between the,
	print TMP_KEY "\n";
	while(<KEY>) { print TMP_KEY; }

	close CERT;
	close KEY;
	close TMP_KEY;

	# FIXME: Once I put this into an RPM we should replace this with something
	#        in Product.pm
	if ( system("/usr/sbin/ssltest -cert $Lockdir/temp.key.$$ > /dev/null 2>&1") ) {
		unlink("$Lockdir/temp.key.$$");
		return MSG_error("ssl_non_matching_keys_cert");
	}

	unlink("$Lockdir/temp.key.$$");
	return MSG_ok("ssl_ok_certificate");
}

sub ssl_clear
# Arguments: An option certificate root directory.
# Returns: The usual mess.
{
	my $cert_dir = shift;
	$cert_dir ||= $Htssldir;

	foreach my $file ( qw(certificate key request) ) {
		$file = "$cert_dir/$file";
		if ( -f $file ) {
			unlink($file) ||
				return MSG_error("ssl_cant_delete",$file,$!);
		}
		if ( -f $file.'.bak' ) {
			unlink($file.'.bak') ||
				return MSG_error("ssl_cant_delete", $file.'.bak',$!);
		}
	}
	return MSG_ok("ssl_ok_deleted_files");
}

1;
