# $Id: Stats.pm,v 1.20 1999/10/02 16:44:27 duncan Exp $
#
# Written by Will DeHaan <will@cobaltnet.com>
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Stats;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(stats_get_www_report stats_set_www_report);

use Cobalt::Product;
use Cobalt::Util;
use strict;

1;
__DATA__

sub stats_set_www_report
# Sets periodic www report generation
# Arguments: interval, site-associated group name(s)
#   interval is one of (never,hourly,daily,weekly,monthly)
# Return value: string, MSG_error("4040")
# Example: $ret = Cobalt::Stats::stats_set_www_report(24,"cobaltnet-com","cobaltmicro-com");
{
    my($Interval,@SiteGroups) = @_;
    $Interval = 0 if ($Interval eq "never");
    return MSG_error("4560") unless (!$Interval || (-d "/etc/cron.$Interval"));

    my($siteGroup,$crondir);
    foreach $siteGroup (@SiteGroups)
    {
	next if (-e "/etc/cron.$Interval/genWebUsage.$siteGroup");
	foreach $crondir (@Cron_Intervals)
	{
	    if ($crondir eq $Interval)
	    {
		Cobalt::Stats::stats_write_webusage($siteGroup,$crondir) || return MSG_error("3510");
	    } else {
		unlink("/etc/cron.$crondir/genWebUsage.$siteGroup") if (-e "/etc/cron.$crondir/genWebUsage.$siteGroup");
	    }
	}
    }
    return MSG_ok("2510");
}

sub stats_get_www_report
# Gets periodic www report generation
# Arguments: site-associated group name
# Return value: interval
#   interval is one of (never,hourly,daily,weekly,monthly)
# Example: $ret = Cobalt::Stats::stats_get_www_report("cobaltnet-com");
{
    my($siteGroup) = $_[0];

    my($crondir);
    foreach $crondir (@Cron_Intervals)
    {
       return($crondir) if (-e "/etc/cron.$crondir/genWebUsage.$siteGroup"); 
    }
    return ('never');
}


# --------------------------------------- Subs ---------------------

# Cobalt::Stats::stats_write_webusage($siteGroup,$crondir);
sub stats_write_webusage
{
    my($siteGroup,$crondir) = @_;
    my($cronFile) = "/etc/cron.$crondir/genWebUsage.$siteGroup";
    open(CRON,"> $cronFile") ||
      return MSG_error("4561", $cronFile);

    if( !$Features{ 'autoGenWebUsage' } ) {
	print CRON <<EOF;
#!/usr/bin/perl

# Web log generation
# Vitual Machine compatible
# Will DeHaan <will\@cobaltnet.com>
# Copyright 1998-1999 Cobalt Networks

my \$log="$Htgroup_dir/$siteGroup/$Logdir/access";
my \$analogReport="$Big_tmp/analog-$siteGroup.dat";

# analog generate report
system( "$Analog_cmd \$log >\$analogReport 2>>$Sauce_log" );

# generate web output
system( "$AdminCGIs_dir/webUsage/parseReport.pl -l \$log -a \$analogReport $AdminPages_dir/siteManage/$siteGroup" );

# clean up
unlink \$analogReport;
EOF
    }
    else {
	print CRON <<EOF;
#!/usr/bin/perl

# Web log generation
# Will DeHaan <will\@cobaltnet.com>
# Copyright 1998-1999 Cobalt Networks

my \$log="$Weblog_dir/$siteGroup-access";
my \$analogReport="$Big_tmp/analog-$siteGroup.dat";

# analog generate report
system( "$Analog_cmd \$log >\$analogReport 2>>$Sauce_log" );

# generate web output
system( "$AdminCGIs_dir/webUsage/parseReport.pl -l \$log -a \$analogReport $AdminPages_dir/webUsage" );

# clean up
unlink \$analogReport;
EOF
    }

    close(CRON);
    chmod(0755,$cronFile) || 
      return MSG_error("4562", $cronFile);
    return 1;
}



1;
