#
# $Id: Time.pm,v 1.46 1999/10/02 16:44:27 duncan Exp $
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
# ntp-werks by Will DeHaan (will@cobaltnet.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Time;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(set_time
	     time_set_zone
	     time_get_zone
	     time_getregions
	     time_getcountries
	     time_getzones
	     time_zone_js
	     ntp_set_servers
	     ntp_get_servers
	     ntpc_set
	     ntpc_get);

require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);
use Cobalt::Util;
use Cobalt::Product;

#use Cobalt::Paths;
#use strict;
use Locale::gettext;
Locale::gettext::textdomain("time");

1;
__DATA__

sub ntpc_get
# Checks whether a ntpdate fetch is scheduled, and what server it queries
# Arguments: none
# Return value: two values, the first is 1 or 0 for on or off,
#    the second is the target server or null
{
	my($server);

	if (-e "$NtpcConf")
	{
		open(NTPC,"$NtpcConf");
		chomp($server = <NTPC>);
		close(NTPC);
	} else
	{
		return(0,'');
	}
	if (-e "$NtpcCron")
	{
		return(1,$server);
	} else
	{
		return(0,$server);
	}
}

sub ntpc_set
# Schedules a ntpdate update, remembers the ntp server if updates are disabled\
# Arguments: 1 or 0 for enable updates or disable, ntp server name
# Return value: String, of the form MSG_error("4324")
# Calling this with status of 1 ntpdates immediately to test the server
{
	my($status,$server) = @_;

	# Use old server if none specified but enabled updates
	if ($status && !$server && (-e "$NtpcConf"))
	{
                open(NTPC,"$NtpcConf");
                chomp($server = <NTPC>);
                close(NTPC);
	}

	if (Cobalt::Netutil::netutil_test_netaddr("$server")) 
	{
		open(NTPC,"> $NtpcConf");
		print NTPC "$server\n";
		close(NTPC);
	} elsif ($status) 
	{
		return MSG_error("4633");
	}

	if ($status)
	{
		if( Cobalt::Netutil::netutil_ping( $server )!~/^2/ ) {
		    return MSG_error("4659");
		}

		my($ret) = `$NtpDate $server 2>&1`;
		if ($ret =~ /no server suitable for synchronization/)
		{
			return MSG_error("4657");
		} elsif ($ret =~ /t adjust the time of day: Invalid/)
		{
			return MSG_error("4658");
		}
		return MSG_error("4632", $server) unless ($ret =~ /[adjust|step] time server/);

		`$Clock_cmd -w -u >>$Sauce_log 2>&1`;
		# Fix the timestamp in /etc/issue, because monitor uses it
		# to tell how long ago we finished booted up
		utime($^T,$^T,$Issuefile);
		# Fix the timestamp on the aliases db, because we auto-rebuild 
		# based on it being older than /etc/aliases
		utime($^T,$^T,$Alias_db) if ($Features{'mail'});

		# Avoid a network slam when 200 RaQs all ntp fetch at the same
		# time.  Now they'll do it over a ten minute window
		my($ntpdelay) = int (rand(600)) + 1;

		# Now make the cute little shell script that keeps us synced
		open(NTCRON,"> $NtpcCron");
		print NTCRON <<EOF;
#!/bin/sh

# NTPdate cron event
# Configured through the administrative web interface
# Copyright (C) 1998, Cobalt Networks, Inc
# All rights reserved

sleep $ntpdelay
$NtpDate $server >>$Sauce_log 2>&1
$Clock_cmd -w -u >>$Sauce_log 2>&1
EOF
		close(NTCRON);
		chmod(0700,"$NtpcCron");

		return MSG_ok("2714");
	} else
	{
		unlink("$NtpcCron") if (-e "$NtpcCron");
		unlink("$NtpcConf") unless ($server);
		return MSG_ok("2715");
	}
}

sub ntp_set_servers
# Define a list of ntp servers
# Arguments: array of fully qualified time server names or ip addresses
# Return value: string of the form MSG_ok("2123")
# Side effects: edits /etc/ntp.conf
{
    my(@NtpServers) = reverse(@_);
    return MSG_error("4625") unless ($Features{'ntp'});
    return MSG_error("4626") unless (-e "$NtpConf");
    return MSG_error("4627") if (-e "$Lockdir/ntp.conf");

    my($ntpServer,$NewServers,%ExistServer);
    while ($#NtpServers >= 0) 
    {
	$ntpServer = pop(@NtpServers); # foreach $ntpServer (@NtpServers

	next if ($ExistServer{"$ntpServer"});

	if ($ntpServer =~ /[a-zA-Z]/)
	{
		return MSG_error("4628", $ntpServer) unless (Cobalt::Netutil::netutil_test_domainname($ntpServer));
		$NewServers .= "server\t$ntpServer\n";
	} elsif ($ntpServer =~ /\d/) {
		return MSG_error("4629", $ntpServer) unless (Cobalt::Netutil::netutil_test_ipaddr($ntpServer));
		$NewServers .= "server\t$ntpServer\n";
	}
	$ExistServer{"$ntpServer"} = 1; # Don't want duplicate entries
    }
    return MSG_error("4630") unless ($NewServers);

    open(NTPLCK,"> $Lockdir/ntp.conf");
    open(NTPCNF,"$NtpConf");
    while(<NTPCNF>)
    {
	if (/^server/)
	{
	    print NTPLCK $NewServers;
	    $NewServers = '';  
	} else {
	    print NTPLCK;
	}
    }
    close(NTPCNF);
    close(NTPLCK);
    rename("$Lockdir/ntp.conf","$NtpConf") || return MSG_error("4631");

    return MSG_ok("2630");
}

sub ntp_get_servers
{
    return "0" unless ($Features{'ntp'});
    return "0" unless (-e "$NtpConf");
    my(@NtpServers);
    open(NTPCF,"$NtpConf");
    while(<NTPCF>) 
    {
	push(@NtpServers,$1) if (/^server\s+([a-zA-Z\d\.]+)/o);
    }
    return(@NtpServers);
    close NTPCF;
}

sub local_settime
{
    my ($month, $date, $year, $hour, $minute) = @_;
    my $now;

    system("$Date_cmd --set='$month/$date/$year $hour:$minute' >>$Sauce_log 2>&1");
    $now = time();
    # Fix the timestamp in /etc/issue, because monitor uses it
    # to tell how long ago we finished booted up
    utime($now,$now,$Issuefile);
    # Fix the timestamp on the aliases db, because we auto-rebuild 
    # based on it being older than /etc/aliases
    utime($now,$now,$Alias_db) if ($Features{'mail'});
}

sub set_time
# Set the time and date
# Arguments: An array containing the month (0-11), the date (1-31), the
#   year (19xx or 20xx or whatever), the hour (0-23) and the minute (0-59)
# Side effects: sets the system clock
# Return value: Returns a string of the form "889 Not enough string"
{
    my ($month,$date,$year,$hour,$minute, $wait) = @_;
    return MSG_error("4486") if (grep(/[^\d]/o, 
	($month, $date, $year, $hour, $minute)));
    $month++;

    local_settime($month, $date, $year, $hour, $minute) unless $wait;

    my $pid;
    if (!defined($pid = fork)) {
	die "cannot fork: $!";
    } elsif ($pid) {
	my $fork_ok;

	# i'm the parent. we do everything here so that we can
	# wait on the child.
        close(\*STDERR);
        close(\*STDOUT);
        close(\*STDIN);
	if ($wait) { # work around some nastiness in apache/perl
	    waitpid($pid, 0);
	    my $newpid = fork;
	    if ($newpid) {
		$fork_ok = 1;
	    } elsif (defined($newpid)) {
		sleep $wait + 5;
		system("$Cron_script stop >>$Sauce_log 2>&1");
		system("$Cron_script start >>$Sauce_log 2>&1");
		exit 0;
	    }
	    sleep($wait);
	    local_settime($month, $date, $year, $hour, $minute);
	}
	system("$Clock_cmd -w -u >>$Sauce_log 2>&1");
	unless ($fork_ok) {
	    system("$Cron_script stop >>$Sauce_log 2>&1");
	    system("$Cron_script start >>$Sauce_log 2>&1");
	}
	exit 0;    
    } else {
	# return the child process.
	return MSG_ok("2039");
    }
}

sub time_get_zone
# Get our timezone
# Arguments: none
# Return value: ($zone, $country, $region)
# Side effects: none
{
    my ($region, $country, $zone);      

    # see if we have some info already.     
    if (open(FILE, "$Time_DBdir/cobalt/timezone")) {
	while (<FILE>) {
	    $region = $1 if /^REGION=(\S+)/;
	    $country = $1 if /^COUNTRY=(\S+)/;
	    $zone = $1 if /^TIMEZONE=(\S+)/;
	}
	close(FILE);
    }

    my $link = readlink($Localtime_link);
    $link =~ s/^$Zone_dir\///o; # strip off leading bits.
    unless ($link eq $zone) {
	# uh oh. the link doesn't match our actual zone. clear out 
	# the current values and go searching for the real timezone.
	($region, $country, $zone) = ();

	return undef unless open(FILE, "$Time_DBdir/cobalt/zonemap");
	while (<FILE>) {
	    next if /^#/;
	    next unless /\b$link\b/;
	    $_ =~ /:([A-Z][A-Z]):(\S+)/;
	    $country = $1;
	    $region = $2;
	    $zone = $link;
	}
	close(FILE);
    }

    return ($zone, $country, $region);
}


sub time_set_zone
# Set our timezone
# Arguments: A string representing the timezone. 
#            we'll just look in /usr/share/zoneinfo for everything.
# Side effects: sets our local time (using the /etc/localtime link)
# Return value: the usual string
{
    my ($zone, $country, $region) = @_;
      
    return MSG_error("4483", $zone) unless (-f "$Zone_dir/$zone");
    unlink($Localtime_link);
    symlink("$Zone_dir/$zone",$Localtime_link) ||
	return MSG_error("4485", $!);

    # write out the timezone in $Timedir
    if ($region and $country and 
	open(FILE, ">$Time_DBdir/cobalt/timezone")) {
	print FILE "REGION=$region\n";
	print FILE "COUNTRY=$country\n";
	print FILE "TIMEZONE=$zone\n";
	close(FILE);
    }
    return MSG_ok("2040", $zone);
}

sub time_getregions
# get regions around the world
{
    my @regions;
    my $region;

    opendir(DIR, $Time_DBdir) or return undef;
    while ($_ = readdir(DIR)) {
	next unless /^[A-Z]/;
	push @regions, $_;
    }
    closedir(DIR);

    return sort @regions;
}

sub time_getcountries
# get countries for a particular region
{
    my $region = shift;
    my @countries;
	
    opendir(DIR, "$Time_DBdir/$region") or return undef;

    # first see if there are a bunch of countries in a single file
    if (open(FILE, "$Time_DBdir/$region/$region")) {
	while (<FILE>) {
	    next unless /^([A-Z][A-Z])\s+/;
	    push @countries, $1;
	}
	close(FILE);
    }

    # now see if there are other countries
    while ($_ = readdir(DIR)) {
	next unless /^[A-Z][A-Z]$/;
	push @countries, $_;
    }
    closedir(DIR);

    return sort @countries;
}

sub time_getzones
# description: get timezones within a country. 
{
    my ($region, $country) = @_;
    my @zones;
    
    # we have two strategies. first, test for a region/country
    # file. then, look for a region/region file.
    if (open(FILE, "$Time_DBdir/$region/$country")) {
	while (<FILE>) {
	    next if /^#/;
	    next unless /^(\S+)/;
	    push @zones, $1;
	}
	close(FILE);
    } elsif (open(FILE, "$Time_DBdir/$region/$region")) {
	while (<FILE>) {
	    next unless /^([A-Z][A-Z])\s+(\S+)/;
	    next unless ($1 eq $country);
	    push @zones, $2;
	}
    }

    return @zones;
}


sub time_zone_js
# spit out timezone info as a javascript data structure.
# NOTE: explorer doesn't like trailing commas, hence the hokeyness.
{
    # regions
    my (@regions, @countries, @zones);
    my ($r, $c, $z);
    my ($nr, $nc, $nz);
    my ($i, $j, $k);

    # regions
    @regions = time_getregions;
    $nr = scalar(@regions);
    my $list = "[";
    while ($r < $nr) {
	$i = $regions[$r];
	$list .= "[\"$i\",\"";
	$list .= Locale::gettext::dgettext("time", $i);
	$list .= "\",[";
	
	# country
	@countries = time_getcountries($i);
	$nc = scalar(@countries);
        $c = 0;
	while ($c < $nc) {
	    $j = $countries[$c];
	    $list .= "[\"$j\",\"";
	    $list .= Locale::gettext::dgettext("time", "$i/$j");
	    $list .= "\",[";

	    @zones = time_getzones($i, $j);
	    $nz = scalar(@zones);
	    $z = 0;
	    while ($z < $nz) {
		$k = $zones[$z];
		$list .= "[\"$k\",\"";
		$list .=  Locale::gettext::dgettext("time", $k);
		$list .= "\"]";
		$z++;
		$list .= "," if ($z < $nz); 
	    }
	    $c++;
	    $list .= "]]";
	    $list .= "," if ($c < $nc);
	}
	$r++;
	$list .= "]]";
	$list .= "," if ($r < $nr);
	$list .= "\n";
    }
    $list .= "]";

    return $list;
}
