#
# $Id: User.pm,v 1.96 1999/10/06 15:13:27 duncan Exp $
#
# User.pm: Module for managing users
#
# Written by Brion Moss (brion@cobaltnet.com, brion@nda.com)
#            Kevin K.M. Chiu (kevin@cobaltnet.com)
#	     Will DeHaan (will@cobaltnet.com)
#            John D. Blair (jdblair@cobaltnet.com) (SMB & ATALK hooks)
#            Duncan Laurie <duncan@cobaltnet.com>
#
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::User;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(user_add user_mod user_del user_show user_list user_exist
	     user_list_groups user_list_site user_info_sorted user_get_xml
	     user_disable user_enable user_issuspend site_user_add
	     user_authok user_fullname_enc);

require Cobalt::Admserv;
import Cobalt::Admserv qw(admserv_set_access);
require Cobalt::Email;
import Cobalt::Email qw(:DEFAULT);
require Cobalt::Fpx;
import Cobalt::Fpx qw(:DEFAULT);
require Cobalt::Group;
import Cobalt::Group qw(group_del_user group_show);
require Cobalt::List;
import Cobalt::List qw(:DEFAULT);
require Cobalt::Quota;
import Cobalt::Quota qw(:DEFAULT);
require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);
require Cobalt::Network;
import Cobalt::Network qw(:DEFAULT);
require Cobalt::Vacation;
import Cobalt::Vacation qw(:DEFAULT);
require Cobalt::Utility;
import Cobalt::Utility qw(:DEFAULT);
require Cobalt::Samba;
import Cobalt::Samba qw(:DEFAULT);
require Cobalt::Atalk;
import Cobalt::Atalk qw(:DEFAULT);
use Cobalt::Util;
use Cobalt::Product;
use Cobalt::Vsite;
use Cobalt::Meta;

use integer;

1;
__DATA__

sub user_authok
# description: check the access rights of a user. 
# input: $user, "admin|site-adm|group", [$group if "group" selected]
# output: 0 on denial, 1 on access.
{
    my ($user, $access, $group) = @_;

    return 0 unless $user;
    return 1 if ($user eq $AdminUser);
    return 1 if (($access =~ /\badmin\b/) and ($user eq $AdminUser));
    return 1 if (($access =~ /\bsite-adm\b/) and
                 grep(/\b$user\b/, group_show($Sites_AdminGroup)));
    return 1 if (($access =~ /\bgroup\b/) and
                 grep(/\b$group\b/, user_list_groups($user)));
    return 0;
}
    
sub user_issuspend
# user_issuspend: true/false depending on user status
# arguments: username
# returns: 0/1
{
    my $user = shift;
    my $shell = (getpwnam($user))[8];
    return 1 if ($shell =~ /suspendsh/);
    return 0;
}

sub user_disable
# user_disable: disable/suspend a user
#  - changes their shell to /bin/false
#  - changes their home direcotry permissions to 000
# arguments: array of usernames
# example: $ret = user_disable("doug","tim")
{
    my %u = map { $_, 1 } @_;
    my ($name,$pass,$uid,$gid,$desc,$dir,$shell);
    my $mode = oct '0000';
    my ($siteadmin,$uobj,$vsite);

    $uobj = Cobalt::Meta->create("type" => "users");

    setpwent;  # start over in passwd file

    FILE_lock_rw("$Passwd",*IN,*OUT)
	|| return MSG_error("4515", $Passwd);

    while (<IN>) {
	($name,$pass,$uid,$gid,$desc,$dir,$shell) = split(":",$_);
	if (exists($u{$name})) {

	    # remove site admin privlidge if present
	    $uobj->retrieve($name);
	    $siteadmin = $uobj->get("admin");
	    $vsite = $uobj->get("vsite");
	    if ($siteadmin eq 'on') {
		Cobalt::Vsite::vsite_admin_del($vsite, $name);
	    }

	    if ($name eq "root" || $name eq $Adminuser) {
		print OUT;
		next;
	    }
	    print OUT "$name:$pass:$uid:$gid:$desc:$dir:/bin/suspendsh\n";
	    chmod $mode, $dir;
	} else {
	    print OUT;
	}
    }

    FILE_unlock_rw("$Passwd",*IN,*OUT)
	|| return MSG_error("4515", $Passwd);

    return MSG_ok("user_disable_ok");
}

sub user_enable
# user_enable: enable/activate a user
#  - changes their shell to /bin/false
#  - changes their home direcotry permissions to 000
# arguments: array of usernames
# example: $ret = user_enable("doug","tim")
{
    my %u = map { $_, 1 } @_;
    my ($name,$pass,$uid,$gid,$desc,$dir,$shell);
    my ($uobj,$vsite,$siteadmin);
    my $mode = oct '2771';
    
    $uobj = Cobalt::Meta->create("type" => "users");

    setpwent;  # start over in passwd file

    FILE_lock_rw("/etc/passwd",*IN,*OUT)
	|| return MSG_error("4515", $Ptmp);

    while (<IN>) {
	($name,$pass,$uid,$gid,$desc,$dir,$shell) = split(":",$_);
	if (exists($u{$name})) {

	    # grant site admin privlidge if present
	    $uobj->retrieve($name);
	    $siteadmin = $uobj->get("admin");
	    $vsite = $uobj->get("vsite");
	    if ($siteadmin eq 'on') {
		Cobalt::Vsite::vsite_admin_add($vsite, $name);
	    }

	    if ($name eq "root" || $name eq $Adminuser) {
		print OUT;
		next;
	    }
	    print OUT "$name:$pass:$uid:$gid:$desc:$dir:/bin/bash\n";
	    chmod $mode, $dir;
	} else {
	    print OUT;
	}
    }

    FILE_unlock_rw("/etc/passwd",*IN,*OUT)
	|| return MSG_error("4515", $Ptmp);

    return MSG_ok("user_enable_ok");
}

sub user_exist
# user_exist: does a user exist?
# Arguments: login name
# Side effects: none
# Return value: true if they do, false if they don't
# Example: $return = Cobalt::User::user_exist("name");
{
    my ($user) = @_;
    setpwent;   # flush
    ((getpwnam($user))[0]) ? (return 1) : (return 0);
}

sub user_add
# user_add: Create a new user
# Arguments: login, full name, cleartext password, quota, shell
#            optional group (necessary for RaQ),
#            optional home directory (if specified, don't create or touch)
# Side effects: adds the user to the password file and the htpassword file, 
#   and creates a home directory.
# Return value: a string of the form "526 The wicked witch is dead"
# Example: Cobalt::User::user_add("rblaine","Rick Blaine","SeCrEt",2048,$Usershell);
{
    my ($username,$descr,$passwd,$quota,$shell,$base_group,$user_home) = @_;
    my ($group) = $base_group;
    my ($id,$uid,$return,$cryptpw,$htmlfile);
    my ($user_group,$user_dirgroup);
    my ($homedir);

    return MSG_error("4543", $username) if ($username !~ /^[\w\-\.]+$/o);
    return MSG_error("4554", $MaxNameLen)
	if (length($username) > $MaxNameLen);
    # Print out the max in megs, not kilobytes
    return MSG_error("4544")
      if ($quota !~ /^\d+$/o);
    my $warn_quota = ($MaxQuota / 1024);
    return MSG_error("4555", $warn_quota)
	if ($quota > $MaxQuota);
    return MSG_error("4546", $username) if ($System_users{$username});
    return MSG_error("4500", $username) if (Cobalt::User::user_exist($username));
    return MSG_error("4501")
	if ($descr =~ /:/o);
    return MSG_error("4557", $username)
	if ($System_groups{$username} && $username ne $Adminuser);
    return MSG_error("4530", $username)
	if (getgrnam($username));
    return MSG_error("4545", $username)
	if (Cobalt::List::alias_exist($username));
    return $cryptpw                    # error messages contain spaces
	if (($cryptpw = Cobalt::User::verify_password($passwd)) =~ /\s/o); #// vim syntax.

    my $localeObj = Cobalt::Locale->new();
    my $locale = $localeObj->getLocale();

    if ($base_group && $Features{'vmachine'}) 
    {
 	unless (-d "$Htgroup_dir/$base_group/users")
	{
	    mkdir("$Htgroup_dir/$base_group/users",02775);
	    chown( (getpwnam($WWWuser))[2], (getgrnam($base_group))[2], "$Htgroup_dir/$base_group/users");
	    chmod(02775,"$Htgroup_dir/$base_group/users"); # the mkdir above doesn't work.
	}
	# all the files should be owned by the site group
	$user_group = (getgrnam($base_group))[2];
	$user_dirgroup = $user_group;
	$base_group = "/$base_group/users";
    }
    else {
	# the files should be owned by the generic users group
	$user_group = $WWWgid;
	$user_dirgroup = $WWWgid;
    }
    print STDERR "$username in group $user_group\n" if ($Debug);

    if ($user_home) { $homedir = $user_home; }
    else {
	if( $base_group ) {
	    $homedir = "$Userdir$base_group/$username";

	    # Make sure the parent directory exists
	    if( ! -d "$Userdir$base_group" ) { mkdir( "$Userdir$base_group", 02775 ); }
	}
	else {
	    $homedir = "$Userdir/$username";

	    # Make sure the parent directory exists
	    if( ! -d $Userdir ) { mkdir( $Userdir, 02775 ); }
	}
    }

    if ( !$user_home && -e $homedir) {
	return MSG_error("4503", $username);
    }

    setpwent();

    # set default shell
    if( !$shell ) { $shell=$Usershell; }

    # Truncate the GCOS if it's too long
    $descr = substr($descr,0,$MaxGcosLen) if (length($descr) > $MaxGcosLen);
    $descr =~ s/>/&gt;/og;  # No HTML!
    $descr =~ s/</&lt;/og;
    $descr =~ s/\'/&\#039;/og;  # Replace quotes with HTML

    return $return unless (($return = Cobalt::User::lock_passwd()) =~ /^2/o);

    ### Add the entry to the password file

    # Keep track of what UIDs we've given and don't reassign them
    # (and don't forget to wipe this in sysreset if we use it)
    if (open(UID,$Uid_file))
    {
	# UID file exists
	$id = <UID>;
	close UID;
    }
    $id ||= $Firstuid;
    #setpwent();
    #for($id; getpwuid($id); $id++) {;}  # Get next UID
    #if ($id =~ /^(\d+)$/o)  { $uid = $1; }
    #else                    { return Cobalt::User::user_abort(MSG_error("4514"));  }
    $uid = Cobalt::User::user_get_first_available_uid($id);
    setpwent();
    if (getpwuid($uid)) {
      return Cobalt::User::user_abort(MSG_error("4506", $uid));
    }

    open(UID,">$Uid_file")
      || return Cobalt::User::user_abort(MSG_error("4556", $Uid_file));
    print UID $uid+1;
    close UID;
    
    my $line = "$username:$cryptpw:$uid:$Usergroup:$descr:$homedir:$shell\n";
    print STDERR "Adding to $Ptmp:\n$line" if ($Debug);
    open(PTMP,">>$Ptmp") 
	|| return Cobalt::User::user_abort(MSG_error("4515", $Ptmp));
    print PTMP $line;
    close PTMP;
    
    unless ($Features{'pam'}) {
      # Appends to the htpasswd file
      open(HTP,">>$Htptmp") 
	|| return Cobalt::User::user_abort(MSG_error("4516", $Htptmp));
      print HTP "$username:$cryptpw\n";
      close HTP;
    }
    
    unless ($user_home) {
      ### Create the directory and files for the user
      print STDERR "Creating directories under $homedir\n" if ($Debug);

      mkdir($homedir,$UserDirPermissions) 
	|| return Cobalt::User::user_abort(MSG_error("user_cant_create_homedir", $homedir));
      chown($uid,$user_dirgroup,$homedir)
	|| return Cobalt::User::user_abort(MSG_error("user_cant_set_ownership", $homedir));
      chmod($UserDirPermissions,$homedir);

      # Add public web directory
      mkdir("$homedir/$Webdir",$UserWebPermissions) 
	|| return Cobalt::User::user_abort(MSG_error("user_cant_create_webdir", $homedir));
      chown($uid,$user_dirgroup,"$homedir/$Webdir")
	|| return Cobalt::User::user_abort(MSG_error("user_cant_set_ownership", $homedir));
      chmod($UserWebPermissions,"$homedir/$Webdir");
    
      my $useridx="$User_skel/$locale/$Webdir/index.html";
      my $useridxt="$homedir/$Webdir/index.html";
      my $domain;
      if ( -d "$User_skel/$locale" ) {
	  system("cp -r $User_skel/$locale/\* $homedir > /dev/null 2>&1");
      }
      if ($group) {
	  if ($group eq $MainGroup) { $domain = Cobalt::Netutil::netutil_get_fulldomain(); }
	  else { $domain = (Cobalt::Vsite::vsite_get_bygroup($group))[1]; }

	  my $siteskel = "$Htgroup_dir/$group/skel";
	  if ( -d $siteskel ) {
	      system("cp -r $siteskel/\* $homedir > /dev/null 2>&1");
	  }
	  if ( -f "$siteskel/$Webdir/index.html" ) {
	      $useridx = "$siteskel/$Webdir/index.html";
	  }
      }
      if ( -d "$User_skel/common" ) {
	  system("cp -r $User_skel/common/\* $homedir > /dev/null 2>&1");
      }

      if ( -f $useridx ) {
	  open(IN, $useridx);
	  if (open(OUT, "> $useridxt")) {
	      while (<IN>) {
		  s/\[DOMAIN\]/$domain/g;
		  s/\[USER\]/$descr/g;
		  s/\[USERNAME\]/$username/g;
		  
		  print OUT;
	      }
	      close(OUT);
	      chmod(0664,"$useridxt");
	  }
	  close(IN);
	  chmod(0664,$useridxt);
      }
      system("chown -R $uid.$user_group $homedir > /dev/null 2>&1");
      
      if ($Features{'privatedirs'})
      {
	mkdir("$homedir/$Webdir/$Private_html",$UserPrivateDirPermissions)
	  || return Cobalt::User::user_abort(MSG_error("4528", $homedir));
	chown($uid,$user_dirgroup,"$homedir/$Webdir/$Private_html");
	chmod(02750,"$homedir/$Webdir/$Private_html");
	Cobalt::Admserv::admserv_set_access("$homedir/$Webdir/$Private_html","require","user",$username);
	
	# Now do index.html for the private directory
	if (-e "$User_skel/private.tmpl")
	{
	  print STDERR "Copying private.tmpl to $homedir/$Webdir/$Private_html/index.html\n"
	    if ($Debug);
	  open(SRC,"$User_skel/private/private.tmpl") 
	    || return Cobalt::User::user_abort(MSG_error("4570", $User_skel));
	  open(DEST,">$homedir/$Webdir/$Private_html/index.html") 
	    || return Cobalt::User::user_abort(MSG_error("4571", $homedir));
	  while (<SRC>)
	  {
	    if (/\[/o)
	    {
	      s/\[USER\]/$descr/;
	      s/\[PATH\]/$homedir\/$Private_html\/index.html/;
	    }
	    print DEST;
	  }
	  close DEST;
	  close SRC;
	  chown($uid,$user_group,"$homedir/$Private_html/index.html");
	}
      }

      # AppleTalk seems to feel some odd need for this directory.
      # Let the baby have its way.
      if ($Features{'atalk'}) {
	mkdir("$homedir/Network Trash Folder",$UserDirPermissions)
	  || return Cobalt::User::user_abort(MSG_error("user_cant_create_trash", $!));
	chown($uid,$user_dirgroup,"$homedir/Network Trash Folder");
	chmod($UserDirPermissions,"$homedir/Network Trash Folder");
	mkdir("$homedir/$Webdir/Network Trash Folder",$UserDirPermissions)
	  || return Cobalt::User::user_abort(MSG_error("user_cant_create_trash", $!));
	chown($uid,$user_dirgroup,"$homedir/$Webdir/Network Trash Folder");
	chmod($UserDirPermissions,"$homedir/$Webdir/Network Trash Folder");
      }
    }
    ### End dir creation

    # Touch the user mail spool and set ownership. the links are created
    # in a funky way to avoid having to actually reference the disk 
    # label.
    my $spoolfile = "$Htgroup_dir/" . ($group ? $group : $MainGroup) . 
	"/../../spool/mail/$username";
    open(MLSPL,"> $spoolfile")
      || return Cobalt::User::user_abort(MSG_error("4596", $username)); 
    close(MLSPL);
    chmod(0600, $spoolfile);
    chown($uid, $user_group, $spoolfile);

    symlink($spoolfile, "$Mailspool/$username");
    `$Lchown_cmd $uid.$user_group $Mailspool/$username`;

    # update the netatalk passwd file (will they ever end?)
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_set_passwd($username, $passwd);
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    # lastly, add an entry to the smbpasswd file
    if ($Features{'smb'}) {
      $ENV{'SMBPASSWD'} = $passwd;
      open(GETHASH, "$Smb_gethash|")
	|| return Cobalt::User::user_abort(MSG_error("4584", $Smb_gethash));
      my $smbhashes = <GETHASH>;
      close(GETHASH);
      open(SMBPTMP, ">>$Smbptmp") or return MSG_error("4575", $Smbptmp);
      print SMBPTMP "$username:$uid:$smbhashes:$descr:$homedir:$shell\n";
      close(SMBPTMP);
    }
    
    return $return unless (($return = Cobalt::User::commit_passwd()) =~ /^2/o);
    Cobalt::Quota::setqlim($homedir,$uid,$quota,($quota + $Quota_harder));
    #Cobalt::Quota::setqlim($Varpart,$uid,$quota,($quota + $Quota_harder));
    
    print STDERR "User $username complete\n" if ($Debug);
    return MSG_ok("2530", $username);
}

sub user_del
# user_del: Delete a user
# Arguments: the login name of the user, optional boolean value to not delete home directory
# Side effects: removes the password file entry, and deletes the home
#   directory
# Return value: a string of the form "902 nut crunchies exploded"
# Example: Wipes user and home directory
#          Cobalt::User::user_del("joey"); 
# Example: Deletes user but leaves the user home directory as-is
#          Cobalt::User::user_del("anonftpuser",1); 
# Example: Deletes user but leaves the user home directory as-is
#          Cobalt::User::user_del("anonftpuser","/home/sites/home/ftp/");
{
    my ($username,$userhome) = @_;
    my ($return);
    my ($login,$home) = (getpwnam($username))[0,7];
    return MSG_ok("2516", $username)
	unless ($login);
    return MSG_error("4502", $Adminuser) if ($username eq $Adminuser);
    return MSG_error("4522", $username)
	if ($System_users{$username});
    
    return $return unless (($return = Cobalt::User::lock_passwd()) =~ /^2/o);

    #   clean the smbpasswd file
    if ($Features{'smb'}) {
      print STDERR "Cleaning from $Smbptmp\n" if ($Debug);
      open(SMBPTMP,$Smbptmp) 
	|| return Cobalt::User::user_abort(MSG_error("4576", $Smbptmp));
      open(SMBPASSWDLOCK,">$Lockdir/smbpasswd") 
	|| return Cobalt::User::user_abort(MSG_error("4577", $Lockdir));
      while (<SMBPTMP>)
	{
	  print SMBPASSWDLOCK unless (/^$username/o);
	}
      close SMBPASSWDLOCK;
      close SMBPTMP;
      rename("$Lockdir/smbpasswd",$Smbptmp) 
	|| return Cobalt::User::user_abort(MSG_error("4578", $Smbptmp));
    }

    # clean the netatalk passwd file
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_del_user($username);
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    ### clean the password file
    print STDERR "Cleaning from $Ptmp\n" if ($Debug);
    open(PASSWD,$Ptmp) 
	|| return Cobalt::User::user_abort(MSG_error("4517", $Ptmp));
    open(PTMP,">$Lockdir/passwd") 
	|| return Cobalt::User::user_abort(MSG_error("4518", $Lockdir));
    "$username:" =~ /^$username:/; # pre-compile our pattern
    while (<PASSWD>)
    {
	print PTMP unless (//);
    }
    close PTMP;
    close PASSWD;
    rename("$Lockdir/passwd",$Ptmp) 
	|| return Cobalt::User::user_abort(MSG_error("4519", $Ptmp));
    
    unless ($Features{'pam'}) {
      # Clean the htpasswd file
      open(HTPASSWD,$Htptmp) 
	|| return Cobalt::User::user_abort(MSG_error("4520", $Htpasswd));
      open(HTP,">$Lockdir/htpasswd") 
	|| return Cobalt::User::user_abort(MSG_error("4521", $Htptmp));
      "$username:" =~ /^$username:/; # pre-compile
      while (<HTPASSWD>)
      {
	print HTP unless (//);
      }
      close HTP;
      close HTPASSWD;
      rename("$Lockdir/htpasswd",$Htptmp) 
	|| return Cobalt::User::user_abort(MSG_error("4540", $Htptmp));
    }
    
    ### get rid of the directory
    unless ($userhome) 
    {
	if ($home && (-d $home)) {
	    print STDERR "Deleting $home...\n" if ($Debug);
	    Cobalt::Utility::sauce_deltree($home);
	} else {
	    print STDERR MSG_error("4531", $username) if ($Debug);
	}
    } else
    {
        print STDERR "Deleting user but leaving directories $userhome alone...\n" if ($Debug);
    }
    my $spoolfile = readlink "$Mailspool/$username";
    unlink("$Mailspool/$username") if (-e "$Mailspool/$username");
    unlink($spoolfile) if (-e $spoolfile);

    ### Commit
    return $return unless (($return = Cobalt::User::commit_passwd()) =~ /^2/o);
    
    ### Clean the group file
    my ($gr,$gp,$gd,$members,@ingroups);
    setgrent();
    $username =~ /\b$username\b/;  # pre-compile
    # We have to do this in two steps because the getgrent
    # stepping through the group list gets confused when we
    # do the getgrnam call within Cobalt::Group::group_del_user()
    while (($gr,$gp,$gd,$members) = getgrent)
    {
	push(@ingroups,$gr)
	    if ($members =~ //);
    }
    endgrent();
    foreach $gr (@ingroups)
    {
	Cobalt::Group::group_del_user($gr,$username);
    }

    # Make sure that we don't leave any vacation or forwarding stuff
    # around
    # I'm not going to sweat the return values here
    Cobalt::List::alias_delete($username); # forwarding alias
    Cobalt::Vacation::vacation_set_message($username,"")
	if (Cobalt::Vacation::vacation_get_message($username));
    Cobalt::Vacation::vacation_set_on($username,0) if (Cobalt::Vacation::vacation_get_on($username));
    Cobalt::Vacation::vacation_del_db($username);

    if ($Features{'vmachine'}) {
	# Remove the user from all virtual mailing lists here
	my ($fqdn, $group) = user_list_site($username);
        Cobalt::List::list_lists_del_mem($group, $fqdn, "${username}\@${fqdn}");
        Cobalt::List::list_del_mem("", "", "${group}", "${username}\@${fqdn}");
    }
    else {
	# delete unqualified:-) user entries from all the implicit mailing lists
	my $fqdn = Cobalt::Netutil::netutil_get_fulldomain();
        Cobalt::List::list_lists_del_mem("", "", "${username}\@${fqdn}");
    }

    # Make sure email aliases are removed
    Cobalt::Email::mail_virtuser_del_byuser( $username );
    my @aliases = Cobalt::List::alias_get_bytarget($username);
    return $aliases[0] if ($aliases[0] =~ /^[134]\S*\s/);
    $return = Cobalt::List::alias_delete(@aliases);
    return $return if ($return !~ /^2/);

    print STDERR "Deletion of user $username complete\n" if ($Debug);
    return MSG_ok("2535", $username);
}

sub user_show
# user_show: Get information about a user
# Arguments: login name
# Return value: an array containing 
#   ("login", "full name", quota, usage, shell)
# Side effects: none
# Example: ($login,$name,$quota,$use,$shell) = Cobalt::User::user_show("mick");
{
    my ($login) = @_;
    return (MSG_error("4504")) unless (Cobalt::User::user_exist($login));
    my ($fullname,$shell) = (getpwnam($login))[6,8];
    my ($usage,$quota) = (Cobalt::Quota::repquota($login))[0,1];
    return ($login,$fullname,$quota,$usage,$shell);
}

sub user_list
# user_list: Return a list of users
#   Skip capitalized users, they're special.
# Arguments: none
# Return value: a sorted list of users, with admin on top
# Side effects: none
# Example: @users = Cobalt::User::user_list();
{
    my (@users,@name);
    setpwent();
    while (@name = getpwent)
    {
	# Generalized system users
	next if ($name[2] < $Firstuid);
	# The admin user is hidden from the UI 
	next if ($name[0] eq $Adminuser);
	# capitalized names are reserved (anonymous FTP users)
	next if ($name[0] =~ /^[A-Z]/o);  

	# System_users are explicitly specified reserved usernames
	push (@users,$name[0]) unless ($System_users{$name[0]});
    }
    endpwent();

    @users = ($Adminuser,sort(@users));
    return @users;
}

sub user_mod
# user_mod: modify the settings for a user.  An argument of 0 or ""
#   for any given field means no change.
# Arguments: ("login","new description","new cleartext password","new quota","new shell")
# Side effects: modifies the password file
#   If we change the password for admin, we also do it for root.
# Return value: a string of the form "895 Out of paper"
# Example: Cobalt::User::user_mod("randy","Randy the Killer Clown",0,0);
{
    my ($name,$newdescr,$newpasswd,$newquota,$newshell,$newgroup) = @_;
    my ($return);
    return MSG_error("4523", $name)
	unless (Cobalt::User::user_exist($name));
    my $warn_quota = ($MaxQuota / 1024);
    return MSG_error("4525", $warn_quota) 
	if ($newquota > $MaxQuota);
    return MSG_error("4524", $name)
	if ($System_users{$name});
    return MSG_error("4505")
	if ($newdescr =~ /:/o);
    return $return unless (($return = Cobalt::User::lock_passwd()) =~ /^2/o);
    # Truncate the GCOS if it's too long
    $newdescr = substr($newdescr,0,$MaxGcosLen) 
	if (length($newdescr) > $MaxGcosLen);
    $newdescr =~ s/>/&gt;/og;  # No HTML!
    $newdescr =~ s/</&lt;/og;
    $newdescr =~ s/\'/&\#039;/og;

    ### modify the password file
    my ($name,$pass,$uid,$gid,$j1,$j2,$desc,$dir,$shell) = getpwnam($name);
    if ($newpasswd)
    {
	$pass = Cobalt::User::verify_password($newpasswd);
	if ($pass =~ /\s/o)
	{
	    # we got an error message, not a password
	    return Cobalt::User::user_abort("$pass");
	}
	
	print STDERR "Setting password for $name to $pass\n" if ($Debug);
    }
    if ($newdescr)
    {
	$desc = $newdescr;
    }
    if ($newshell)
    {
	$shell = $newshell;
    }
    if (($newdescr || $newpasswd || $newshell || $newgroup) && ($name eq $Adminuser))
    {
        if ($newgroup and ($newgroup = (getgrnam($newgroup))[2])) {
           $gid = $newgroup;
        }
       
	open(PASSWD,$Ptmp) || return Cobalt::User::user_abort(MSG_error("4538", $Ptmp));
	open(PTMP,">$Lockdir/passwd") 
	    || return Cobalt::User::user_abort(MSG_error("4541", $Lockdir));
	while (<PASSWD>)
	{
	    if (/^$Adminuser:/o)
	    {
		print PTMP "$name:$pass:$uid:$gid:$desc:$dir:$shell\n";
	    }
	    elsif (/^root:/o)
	    {
		print PTMP "root:$pass:0:0:Root:/root:/bin/sh\n";
	    }
	    else
	    {
		print PTMP;
	    }
	}
	close PASSWD;
	close PTMP;
	rename("$Lockdir/passwd",$Ptmp);
    }
    elsif ($newdescr || $newpasswd || $newshell)  # not the admin user
    {
	open(PASSWD,$Ptmp) || return Cobalt::User::user_abort(MSG_error("4532", $Ptmp));
	open(PTMP,">$Lockdir/passwd") 
	    || return Cobalt::User::user_abort(MSG_error("4533", $Lockdir));
	"$name:" =~ /^$name:/;
	while (<PASSWD>)
	{
	    if (//)
	    {
		# don't chomp
		print PTMP "$name:$pass:$uid:$gid:$desc:$dir:$shell\n";
	    }
	    else
	    {
		print PTMP;
	    }
	}
	close PASSWD;
	close PTMP;
	rename("$Lockdir/passwd",$Ptmp);
    }

    # modify the netatalk password entry
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_set_passwd($name, $newpasswd);
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    # modify the smbpasswd entry
    if ($Features{'smb'}) {
      # start by getting the new password hash
      $ENV{'SMBPASSWD'} = $newpasswd;
      open(GETHASH, "$Smb_gethash|")
	|| return Cobalt::User::user_abort(MSG_error("4579", $Smb_gethash));
      my $smbhashes = <GETHASH>;
      close(GETHASH);
      
      # modify smbpasswd
      open(SMBPTMP,$Smbptmp) 
	|| return Cobalt::User::user_abort(MSG_error("4580", $Smbptmp));
      open(SMBPASSWDLOCK,">$Lockdir/smbpasswd") 
	|| return Cobalt::User::user_abort(MSG_error("4582", $Lockdir));
      "$name:" =~ /^$name:/; # pre-compile our pattern
      while (<SMBPTMP>)
	{
	  if (//) {
	    print SMBPASSWDLOCK "$name:$uid:$smbhashes:$desc:$dir:$shell\n";
	  } else {
	    print SMBPASSWDLOCK;
	  }
	}
      close SMBPASSWDLOCK;
      close SMBPTMP;
      rename("$Lockdir/smbpasswd",$Smbptmp) 
	|| return Cobalt::User::user_abort(MSG_error("4583", $Smbptmp));
      # done modifying smbpasswd
    }
    
    unless ($Features{'pam'}) {
      if ($newpasswd)
      {
	# Fix the htpasswd file
	open(HTPASSWD,$Htptmp) 
	    || return Cobalt::User::user_abort(MSG_error("4549", $Htpasswd));
	open(HTP,">$Lockdir/htpasswd") 
	    || return Cobalt::User::user_abort(MSG_error("4547", $Htptmp));
	"$name:" =~ /^$name:/; # pre-compile
	while (<HTPASSWD>)
	{
	    if (//)
	    {
		print HTP "$name:$pass\n";
	    }
	    else
	    {
		print HTP;
	    }
	}
	close HTP;
	close HTPASSWD;
	rename("$Lockdir/htpasswd",$Htptmp) 
	    || return Cobalt::User::user_abort(MSG_error("4548", $Htptmp));
      }
    }
    return $return unless (($return = Cobalt::User::commit_passwd()) =~ /^2/o);
    
    ### Set the quota
    if ($newquota)
    {
	Cobalt::Quota::setqlim($dir,(getpwnam($name))[2],
		$newquota,($newquota + $Quota_harder));
	#Cobalt::Quota::setqlim($Varpart,(getpwnam($name))[2],
	#	$newquota,($newquota + $Quota_harder));
    }
    
    print STDERR "Modification of user $name complete\n" if ($Debug);
    return MSG_ok("2533", $name);
}

sub user_list_groups
# Find out what groups a user is in.
# Arguments: username
# Return value: a list of groups
# Side effects: none
# Example: @debbies_groups = Cobalt::User::user_list_groups("debbie");
{
    my ($username) = @_;
    my ($gr,$members,@grouplist);
    setgrent();
    $username =~ /\b$username\b/;  # pre-compile
    while (($gr,$members) = (getgrent)[0,3])
    {
	next if ($System_groups{$gr});
	push(@grouplist,$gr) if ($members =~ //);
    }
    endgrent();
    return sort(@grouplist);
}

sub user_list_site
# Find out what site a user is in.
# Assuming the list groupName-users contains a list of users of group groupName
#   and a user can only belong to one groupName-users list
# Arguments: username
# Return value: fully qualified domain name of the site, group name
#   Nothing if nothing found
# Side effects: none
# Example: ( $domain, $group ) = Cobalt::User::user_list_site( 'kevin' );
{
    my ($user) = @_;
    my ($fqdn, $group, $obj, $vobj);

    $obj = Cobalt::Meta->create("type" => "users");
    $obj->retrieve($user);
    $group = $obj->get("vsite");

    $vobj = Cobalt::Meta->create("type" => "vsite");
    $vobj->retrieve($group);
    $fqdn = $vobj->get("fqdn");

    return ($fqdn, $group);
}

sub user_info_sorted
# Get a sorted list of users, sorted by full name
# Arguments: noParseFullName
# Return value: a two dimensional array: each element of the array
#   is an array containing (full name, '', user name) if noParseFullName is true or (first name, last name, user name) otherwise
# Side effects: none
{
    my ( $noParseFullName )=@_;

    my ($login,@users,@admin);
    foreach $login (Cobalt::User::user_list())
    {
	#next if ($login eq $Adminuser);   # Don't change the admin user
	my $descr = (getpwnam($login))[6];
	my ($firstName,$lastName);

        if( !$noParseFullName ) {
	    # This way will split the first and last name by the last whitespace:
	    # @name = split(/\s+/o,$descr);
	    # $lastname = pop(@name);
	    # $firstname = join(' ',@name);

	    # This way will split the first and last name by the first whitespace:
	    if ($descr =~ /^\s*(\S+\s+)?(\S?.*\S)\s*$/o)
	    {
		($firstName,$lastName) = ($1,$2);
		# get rid of trailing whitespace
		$firstName =~ s/\s+$//og;
	    }
	}
        else {
	    $firstName=$descr;
	    $lastName='';
        }

	if ($login eq $Adminuser)
	{
	    @admin = ($firstName,$lastName,$login);
	}
	else
	{
	    my @user = ($firstName,$lastName,$login);
	    push(@users, [ @user ]);
	}
	print STDERR "My name is $firstName/$lastName/\n" if ($Debug);
    }

    # sort users 
    @users=sort { 
	# order by last name, first name, then user name 
	lc ( ${ $a }[ 1 ] ) cmp lc( ${ $b }[ 1 ] ) 
        or 
	lc( $ { $a }[ 0 ] ) cmp lc( $ { $b }[ 0 ] ) 
	or 
        lc( ${ $a }[ 2 ] ) cmp lc( ${ $b }[ 2 ] ); 
    } 
    @users; 

    # admin goes first
    #my @admin = ("",(getpwnam($Adminuser))[6],$Adminuser);
    unshift(@users, [ @admin ]);
    return @users;
}

##############################

sub lock_passwd
# Lock the password file so that nobody can collide with us.
# Arguments: none
# Side effects: copies $Passwd to $Ptmp and $Htpasswd to $Htptmp
# Return value: message of the form MSG_ok("2671")
{
    if (-e $Ptmp)
    {
      return Cobalt::User::user_abort(MSG_error("4507"));
    }
    else
    {
	# Lock the password file by creating the ptmp file as a
	# copy of the password file.
	open(PASSWD,$Passwd) || return MSG_error("4508", $Passwd);
	open(PTMP,">$Ptmp") 
	    || return MSG_error("4509", $Ptmp);
	print PTMP while (<PASSWD>);
	close PTMP;
	close PASSWD;

	# Lock the smbpasswd file by creating the smbptmp file as a
	# copy of the password file.
	if ($Features{'smb'}) {
	  open(SMBPASSWD,$Smbpasswd) || return MSG_error("4572", $Smbpasswd);
	  open(SMBPTMP,">$Smbptmp") 
	    || return MSG_error("4573", $Smbptmp);
	  print SMBPTMP while (<SMBPASSWD>);
	  close SMBPTMP;
	  close SMBPASSWD;
	}

	# Lock the netatalk passwd file by calling atalk_lock_passwd
	# (which does the same thing as the others above)
	if ($Features{'atalk'}) {
	  my $atalk_ret = Cobalt::Atalk::atalk_lock_passwd();
	  return $atalk_ret if ($atalk_ret !~ /^2/);
	}

	# we need to lock this now in case the next opens abort
	open(LOCK,">$Lockdir/user-$$") || return Cobalt::User::user_abort(MSG_error("4599", $!));
	print LOCK "$$\n$0";
	close LOCK;

    	unless ($Features{'pam'}) {
	  print STDERR "Copying htpassword file $Htpasswd to $Htptmp\n" 
	    if ($Debug);
	  if (! -e $Htpasswd)
	  {
	    # this looks like a fresh system
	    print STDERR "Creating new htpassword file $Htpasswd\n" 
	      if ($Debug);
	    open(HTPASS,">$Htpasswd")
	      || return MSG_error("4574", $Htpasswd);
	    close HTPASS;
	  }
	  Cobalt::Utility::sauce_cp($Htpasswd,$Htptmp);
	}
    }

    print STDERR "Locked $Ptmp\n" if ($Debug);
    return MSG_ok("2511");
}

sub commit_passwd
# Unlocks the password file by moving ptmp into the place of passwd
# Arguments: none
# Side effects: replaces password with ptmp and htpasswd with htptmp
{
    rename($Ptmp,$Passwd) 
	|| return MSG_error("4510", $Passwd);
    chmod(0644,$Passwd);
    if ($Features{'pam'}) {
        # currently sauce supports shadow passwd only
        # support for radius/nis/etc requires libpwdb
	system("$Pwconv 2>/dev/null") &&
	    return (MSG_error("user_cant_pwconv"));
    }
    else {
    rename($Htptmp,$Htpasswd) 
	|| return MSG_error("4511", $Htpasswd);
    chmod(0644,$Htpasswd);
    }
    if ($Features{'smb'})
    {
	rename($Smbptmp,$Smbpasswd)
	  || return MSG_error("4581", $Smbpasswd);
	chmod(0600,$Smbpasswd);
    }
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_commit_passwd();
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    unlink("$Lockdir/user-$$");
    endpwent();  # flush the cache
    return MSG_ok("2512");
}

sub unlock_passwd
# Unlocks the password file by wiping ptmp
# Arguments: none
# Side effects: wipes ptmp, htptmp
{
    unlink($Ptmp) || return MSG_error("4512", $Ptmp);
    print STDERR "Removed $Ptmp\n" if ($Debug);

    unless ($Features{'pam'}) {
      unlink($Htptmp) || return MSG_error("4513", $Htptmp);
      print STDERR "Removed $Htptmp\n" if ($Debug);
    }

    if ($Features{'smb'}) {
      unlink($Smbptmp) || return "smb_cant_unlink_smbptmp Cannot unlink $Smbptmp: $!";
    }
    if ($Features{'atalk'}) {
      my $atalk_ret = Cobalt::Atalk::atalk_unlock_passwd();
      return $atalk_ret if ($atalk_ret !~ /^2/);
    }

    unlink("$Lockdir/user-$$");
    endpwent();  # flush the cache
    return MSG_ok("2513");
}

sub verify_password
# Make sure the password is okay
# Arguments: the password
#   If the password is "*" then return a locked password entry
# Return value: the encrypted version of the password, or an error message.
#   Encrypted passwords will always be 13 characters; error messages
#   never will.
#TBD# SECURITY ALERT
# If we wanted to enforce strong passwords, this is where we would
# do it.  Add tests to this function to see if a password is good enough,
# and return an error if it isn't...
{
    my ($userpass) = @_;
    
    return "*" if ($userpass eq "*");  # Locked entry

    # If the password is of the form 'crypt:<crypted_password>' then just return that.
    if ( $userpass =~ /^crypt:(.{13})$/ ) {
	return $1;
    }

    if ($userpass =~ /[\000-\037\177]/o)
    {
	return MSG_error("4534");
    }
    if (length($userpass) > $MaxPassLen)
    {
	return MSG_error("4535", $MaxPassLen);
    }
    elsif (length($userpass) < $MinPassLen)
    {
	return MSG_error("4536", $MinPassLen);
    }

    # Generate the encrypted password
    srand();
    my $salt = ('a'..'z','A'..'Z',0..9)[rand(62)] . 
	('a'..'z','A'..'Z',0..9)[rand(62)];
    my $cryptpass = crypt($userpass, $salt);
    return $cryptpass;
}

sub user_abort
# leave in a (more or less) graceful manner
# Arguments: a string to print
# Side effects: unlocks anything we've locked
{
    if (-e "$Lockdir/user-$$")
    {
	my $return = Cobalt::User::unlock_passwd();
	return $return unless ($return =~ /^2/o);
    }
    return join('',@_);
}

sub user_get_first_available_uid
# Read in the password file and figure out what the lowest
# unallocated UID is.
# Arguments: UID to start from
# Side effects: none
# Return value: a nice fresh UID to use
{
  my $lowest_id = $_[0];
  $lowest_id = $Firstuid if ($lowest_id < $Firstuid);
  
  my %ids = ();
  open(PW, "$Passwd") || return MSG_error("4569", $Passwd);
  while (<PW>) {
    chomp;
    my ($user, $pw, $id, $rest) = split(/\:/o, $_, 4);

    # Be careful and fill in the UID holes
    if ($id == $lowest_id) {
      $lowest_id++;
    }
    elsif ($id > $lowest_id) {
      $ids{$id} = 1;
    }
  }
  close(PW);

  # Incrementing for what?
#  $lowest_id++;
  $lowest_id++ while ($ids{$lowest_id});
  return $lowest_id;
}

#
# set the encoding of the users fullname in the password file
#
sub user_fullname_enc
{
    my $name = shift;
    my $enc = shift;

    if (!$enc) {
	return;
    } else {
	my $uObj = Cobalt::Meta->new("type" => "users");
	$ret = Cobalt::Meta::set_encoding($enc);
	$uObj->retrieve($name);
	my $FN_ENC = $uObj->get("fullname");
	if (!$FN_ENC) {
	    return MSG_error("fullname_enc_error");
	}

	setpwent();
	my ($name,$pass,$uid,$gid,$j1,$j2,$desc,$dir,$shell) = getpwnam($name);
	endpwent();

	FILE_lock_rw("$Passwd", *PWIN, *PWOUT) ||
	    return MSG_error("fullname_enc_error");
	while (<PWIN>) {
	    if (/^$name:/) {
		print PWOUT "$name:$pass:$uid:$gid:$FN_ENC:$dir:$shell\n";
	    } else {
		print PWOUT;
	    }
	}
	FILE_unlock_rw("$Passwd", *PWIN, *PWOUT) ||
	    return MSG_error("fullname_enc_error");
    }
    return MSG_ok("fullname_enc_ok");
}

sub site_user_add 
{
    my $obj = shift;
    my $err=0;
    my $info;
    
    # read form input
    my $name      = $obj->get('name');
    my $group     = $obj->get('group');
    my $fullname  = $obj->get('fullname');
    my $password1 = $obj->get('password1');
    my $password2 = $obj->get('password2');
    my $quota     = $obj->get('quota');
    my $shell     = $obj->get('shell');
    my $admin     = $obj->get('admin');
    my $aliasArea = $obj->get('aliases');
    my $group     = $obj->get('group');
    my $fpx       = $obj->get('fpx');
    my $forward   = $obj->get('forward');
    my $apop      = $obj->get('apop');
    my $suspend   = $obj->get('suspend');
    my $vacation  = $obj->get('vacation');
    my $vacationMsg = $obj->get('vacationmsg');

    my $vsiteobj = Cobalt::Meta->create("type" => "vsite");
    $vsiteobj->retrieve($group);
    my $quota_site = $vsiteobj->get("quota");
    my $fqdn = $vsiteobj->get("fqdn");

    my $userobj = Cobalt::Meta->create("type" => "users");
    $userobj->put("vsite"    => $group,
		  "name"     => $name,
		  "fullname" => $fullname,
		  "quota"    => $quota,
		  "uid"      => '999', # Changed later.
		  "shell"    => $shell,
		  "apop"     => $apop,
		  "suspend"  => $suspend,
		  "admin"    => $admin,
		  "fpx"      => $fpx,
		  "forward"  => $forward,
		  "vacation" => $vacation,
		  "vacationmsg" => $vacationMsg);


    my @passwordInBytes=unpack ('C*', $password1);
    my $passwordBadChar=0;
    my $byte;
    foreach $byte ( @passwordInBytes ) {
	if ($byte>127) {
	    $passwordBadChar=1;
	    last;
	}
    }

    my (@resultAliases,$alias,@badalii);
    if ($aliasArea) {
	# get individual aliases
	my @aliases=split /[\s,]+/m, $aliasArea;
	my %aliasHash;
	map { map { $aliasHash{$_} = 1; }
	      split(' ', $_); } Cobalt::Meta::query("type"  => "users",
						    "keys"  => ["aliases"],
						    "where" => ["vsite", "=", "$group"]);
	foreach $alias (@aliases) {
	    next unless ($alias);
	    $alias =~ s/^([^@]+)\@.*/$1/;
	    if (exists($aliasHash{$alias}) ||
		$alias !~ /^[a-zA-Z0-9\@\.\-\_]+$/o) {
		push @badalii, $alias;
		$err=1;
	    } else {
		push @resultAliases, $alias;
	    }
	}
    }
    if ($err) {
	$info.=MSG_get_ui("emailAliasNotValid");
	$info.=join(', ', @badalii);
    }
    elsif (!$name || !$fullname || !$password1) {
	$info.=MSG_get_ui("noUserInfo");
	$err=1;
    }
    elsif ($password1 ne $password2) {
	$info.=MSG_get_ui("passwordsNotMatch");
	$err=1;
    }
    elsif ($passwordBadChar) {
	$info.=MSG_get_ui("passwordBadChar");
	$err=1;
    }
    elsif ($name!~/^[a-z0-9][a-z0-9\.\-\_]*$/o) {
	$info.=MSG_get_ui("userNameNotValid");
	$err=1;
    }
	# Comment below places to stop vim highlighting getting confused.
    elsif ($fullname=~/\"/o) { # "
	$info.=MSG_get_ui("noQuotesInFullName");
	$err=1;
    }
    elsif ($quota!~/^\d+$/o || $quota==0) {
	$info.=MSG_get_ui("diskQuotaNotPositive");
	$err=1;
    }
    elsif ($quota>$quota_site) {
	# for the message
	$info.=MSG_get_ui("userDiskQuotaTooBig", $quota_site);
	$err=1;
    }
    else {
	# everything is fine

	my $shell = ($shell eq 'on') ? $Usershell : $Badshell;

	# convert from megabyte to block
	$quota*=1048576/$BytesPerBlock;

	my $ret=user_add ($name, $fullname, $password1, $quota, $shell, $group);
	if ($ret!~/^2/o) {
	    $info.=substr ($ret, 4);
	    $err=1;
	}
	# Now we need to get the UID of the user to update the database with.
	$userobj->put("uid", (getpwnam($name))[2] );

	if (!$err) {
	    # create default vacation reply
	    $ret=vacation_set_message ($name, $vacationMsg);
	    if ($ret!~/^2/o) {
		$info.=substr ($ret, 4);
		$err=1;
	    }

	    # set vacation to proper state
		if ( $vacation eq 'on' ) {
		    $ret=vacation_set_on($name, $vacation);
		    if ($ret!~/^2/o) {
				$info.=substr ($ret, 4);
				$err=1;
		    }
		}

	    # set mail aliases
	    my @alii;
	    my $a = join(' ', @resultAliases);
	    $userobj->put("aliases" => $a);
	    foreach $_ (@resultAliases) {
		push @alii, "$_\@$fqdn";
	    }
	    $ret=mail_virtuser_set_byuser ($name, @alii);
	    if ($ret!~/^2/o) {
		$info.=substr ($ret, 4) . '<BR>';
	    }

	    # save the user in the database
	    # don't want a stupid error from apop/fpx to make the
	    # user disappear from the database
	    $userobj->save();

	    # assign site admin if necessary
	    if ($admin eq 'on') {
		$ret=vsite_admin_add ($group, $name);
		if ($ret!~/^2/o) {
		    $info.=substr ($ret, 4);
		    $err=1;
		}
	    }

	    # set apop on/off
	    if (!$err && $apop eq 'on') {
		$ret=mail_apop_adduser ($name, $password1);
		if ($ret !~ /^2/) {
		    $info.=substr ($ret, 4) . '<BR>';
		    $err=1;
		}
	    }

	    # set frontpage on/off
	    if (!$err && $fpx eq 'on') {
	        $ret=fpx_set_web ($name, 1);
	    	if ($ret !~ /^2/) {
                    $info.=substr ($ret, 4) . '<BR>';
                    $err=1;
	    	}
		# Update web server configuration file for fpx
	        Cobalt::Http::http_restart(1);
	    }
	}
    }

    if ($err) {
	return ($info);
    } else {
	return;
    }
}

sub user_get_xml
# Arguments: A Vsite name
# Returns: The site as an XML object.
#
# Only used inside this module for the purpose of dumping Vsite objs.
{
	my $user = shift;
	my $obj = Cobalt::Meta->create("type" => "users");
	$obj->retrieve($user);
	return $obj->as_xml();
}

1;
