# Utility.pm: General functions used throughout the sauce
#
# $Id: Utility.pm,v 1.13 1999/10/02 16:44:27 duncan Exp $
#
# Written by
#   Brion Moss (brion@cobaltnet.com, brion@nda.com)
#
# Copyright 1998 Cobalt Networks http://www.cobaltnet.com/

package Cobalt::Utility;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(sauce_cp sauce_deltree sauce_booted);

use Cobalt::Product;
use Cobalt::Util;

1;
__DATA__

sub sauce_cp
# Copies a file
# Arguments: Source, dest
# Side effect: creates (or clobbers) dest as a copy of src
{
    my ($src,$dest) = @_;
    open(SRC,$src) || return MSG_error("4551", $src);
    open(DEST,">$dest") || return MSG_error("4552", $dest);
    print DEST while (<SRC>);
    close DEST;
    close SRC;
    return MSG_ok("2515");
}

sub sauce_deltree
# Recursively deletes a directory tree
# Arguments: the directory to start at
# Side effects: kills a tree of files
{
    my ($top) = @_;
    my (@files,$filename,$file);

    if ($top =~ /^([^;\|]+)$/o)  {$top = $1; }  # untaint ourselves

    print STDERR "Deleting $top\n" if ($Debug);
    if ((-d $top) && (! -l $top))
    {
	# It's a directory, recurse
	opendir(DIR,"$top") || return MSG_error("4537", $top);
	@files = grep(!/^\.\.?$/o,readdir(DIR));
	closedir DIR;
	if (@files)
	{
	    foreach $filename (@files)
	    {
		$file = "$top/$filename";
		if ((-d $file) && (! -l $file))
		{
		    Cobalt::Utility::sauce_deltree($file);
		}
		else
		{
		    unlink($file);
		}
	    }
	}
	rmdir($top);
    }
    else
    {
	# it's just a file...
	unlink($top);
    }
    return MSG_ok("2514");
}

sub sauce_booted
# Are we booted up?
# The last thing done in the boot sequence is to re-create /etc/issue;
# so if that file is older than our current uptime, then we're in the
# early stages of bootup and should wait before doing anything.
# Arguments: none
# Return value: 1 if we're all up and going, 0 if we're still in the
#   process of booting
{
  my ($runtime,$boottime);

  # First check to make sure we aren't in the middle of boot-up or shutdown
  open(RL,"$Runlevel_cmd|") || die "Can't get runlevel: $!\n";
  my $rl = (split(/\s/o,<RL>))[1];
  close RL;
  return 0 if ($rl != 3);

  # Now check to see if we're all the way up
  open(UPTIME,"/proc/uptime") || die "Cannot read /proc/uptime: $!\n";
  $boottime = (split(/\s/o,<UPTIME>))[0];
  close UPTIME;

  # No way should we ever take more than an hour to boot.  If uptime is that 
  # big,assume we're booted (and let Active Monitor sort it out...)
  return 1 if ($boottime > 3600);  

  # Otherwise we're in that first hour of uptime.  The last thing that should
  # happen in the boot sequence is the issue file getting updated, so check 
  # that.
  $boottime = time() - $boottime;   # convert into time since the epoch
  $runtime = (stat($Issuefile))[9];
  print STDERR "Up since $boottime, running since $runtime\n" if ($Debug);
  ($runtime < $boottime) ? return 0 : return 1;
}

1;
