# Vsite.pm: Functions for virtual hosting
#
# $Id: Vsite.pm,v 1.92 1999/10/09 08:47:29 asun Exp $
#
# Written by
#   Will DeHaan (will@cobaltnet.com)
#   Brion Moss (brion@cobaltnet.com, brion@nda.com)
#   Duncan Laurie <duncan@cobaltnet.com>
#   Harris Vaegan-Lloyd <harris@cobaltnet.com>
#
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Vsite;
use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
require SelfLoader;
@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(vsite_add vsite_del
	     vsite_list vsite_get_bygroup
	     vsite_gen_name
	     vsite_admin_add vsite_admin_del vsite_admin_set vsite_admin_list
	     vsite_add_alias vsite_del_alias
	     vsite_set_cgis vsite_get_cgis vsite_getgroup
	     vsite_set_ssi vsite_get_ssi
	     vsite_set_fpx vsite_get_fpx
	     vsite_sync vsite_migrate_ipaddr
	     vsite_set_logrotate
	     vsite_disable vsite_enable vsite_issuspend
	     vsite_get_users
	     vsite_ssl_get_cert_info
	     vsite_ssl_set_identity
	     vsite_ssl_get_cert
	     vsite_ssl_set_cert
	     vsite_check_cert
	     vsite_ssl_clear
	     vsite_get_ssl
	     vsite_set_sslconf
	     vsite_get_xml
	     site_del
	     vsite_obj_add
	     );

require Cobalt::Group;
import Cobalt::Group qw(:DEFAULT);
require Cobalt::Language;
import Cobalt::Language qw(:DEFAULT);
require Cobalt::List;
import Cobalt::List qw(:DEFAULT);
require Cobalt::Netutil;
import Cobalt::Netutil qw(:DEFAULT);
require Cobalt::Network;
import Cobalt::Network qw(:DEFAULT);
require Cobalt::Ftp;
import Cobalt::Ftp qw(:DEFAULT);
require Cobalt::Fpx;
import Cobalt::Fpx qw(:DEFAULT);
require Cobalt::Http;
import Cobalt::Http qw(:DEFAULT);
require Cobalt::Ssl;
import Cobalt::Ssl qw(:DEFAULT);
use Cobalt::Util;
use Cobalt::Product;
use Cobalt::Email;
use Cobalt::Stats;
use Cobalt::User;
use Cobalt::Stats;
use Cobalt::BandWidth;
use Cobalt::Meta;
use Cobalt::Layout;
use Cobalt::Admserv;

use DB_File;
use Fcntl qw(O_RDWR O_CREAT);
use integer;

1;
__DATA__

sub vsite_add
# Adds a virtual web server
# Arguments: the IP address, server name, associated group name, 
#            admin email address, optional subnet mask, 
# Return value: string of the form MSG_error("3310")
# Side effects: Modifies httpd.conf, may add a virtual network interface
{
    my ($addr,$name,$group,$serveradmin,$mask) = @_;
    my($found_interface,$found_namev,$conf,$new_conf,$found_group);
    my $ret;

    # This is a new site, enforce a maximum site count
    #   "Not with RaQ 2" /wdh  
    # my (@siteslist) = Cobalt::Vsite::vsite_list();
    # return MSG_error("4597")
    #   if (($#siteslist + 1) >= $Sites_Max); # +1 to un-zero-index

    $serveradmin ||= $Adminuser; # Give it a default..
    # hack to prevent perl from joining the $ at the end of a here
    # document line to the text on the next line and do variable
    # interpolation on it...perl _is_ brain dead!!
    my $my = '$';		
    $new_conf .= <<EOF;
<VirtualHost $addr>
ServerName $name
ServerAdmin $serveradmin
DocumentRoot $Htgroup_dir/$group/$Webdir
RewriteEngine on
RewriteCond %{HTTP_HOST}                !^$addr(:80)?$my
RewriteCond %{HTTP_HOST}                !^$name(:80)?$my
RewriteRule ^/(.*)                      http://$name/\$1 [L,R]
RewriteOptions inherit
AliasMatch ^/~([^/]+)(/(.*))? $Htgroup_dir/$group/users/\$1/web/\$3
AliasMatch ^/users/([^/]+)(/(.*))? $Htgroup_dir/$group/users/\$1/web/\$3
</VirtualHost>
EOF

    # 'update the current configuration
    open(HTCNF,"$Htconf/httpd.conf") ||
      return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (LOCK, ">$Lockdir/httpd.conf") ||
      return MSG_error("vsite_wicked");

    $found_interface = 0;
    while(<HTCNF>)
    {
	if (!$found_interface and
	    /^<VirtualHost $addr>$/ ... /^<\/VirtualHost>$/) {
	    # sanity check
	    return vsite_abort(MSG_error("vsite_cant_hostnameinuse", $name))
		if (/^DocumentRoot\s+$Userdir\/$group\/$Webdir$/ or
		    /^ServerName\s+$name$/);

	    # slurp!
	    $conf .= $_;

	    # end of section; throw up the slurped goo..
	    if (/^<\/VirtualHost>$/) {
		# first entry for $addr
		$found_interface = 1;
		print LOCK "\nNameVirtualHost $addr\n";

		# hack to get the maingroup conf first
		if ($group eq $MainGroup) {
		    print LOCK "${new_conf}${conf}";
		} else {
		    print LOCK "${conf}${new_conf}";
		}
		$conf = '';
	    }
	} else {
	    # spit it unless..
	    print LOCK unless (/NameVirtualHost $addr$/);
	}
    }
    print LOCK "\nNameVirtualHost $addr\n$new_conf" unless ($found_interface);
    close(HTCNF);
    close(LOCK);

    # Create web subdir
    unless(-d "$Htgroup_dir/$group/$Webdir")
    {
	mkdir("$Htgroup_dir/$group/$Webdir",775);
	chown((getpwnam($WWWuser))[2],(getgrnam($group))[2],
	      "$Htgroup_dir/$group/$Webdir");
	chmod(02775,"$Htgroup_dir/$group/$Webdir");
    }

    # Create logs subdir
    unless(-d "$Htgroup_dir/$group/$Logdir")
    {
	mkdir("$Htgroup_dir/$group/$Logdir",775);
	chown((getpwnam($WWWuser))[2],(getgrnam($group))[2],
	      "$Htgroup_dir/$group/$Logdir");
	chmod(02750,"$Htgroup_dir/$group/$Logdir");
    }

    # Create certs subdir
    unless(-d "$Htgroup_dir/$group/$Certdir")
    {
	mkdir("$Htgroup_dir/$group/$Certdir",775);
	chown((getpwnam($WWWuser))[2],(getgrnam($group))[2],
	      "$Htgroup_dir/$group/$Certdir");
	chmod(02750,"$Htgroup_dir/$group/$Certdir");
    }

    # Add the network interface if necessary
    unless($group eq $MainGroup) # Special case handled by Network.pm
    {
	my(@all_IPs) = Cobalt::Network::net_get_alladdrs();
	my($ip_instance,%active_IPs);
	foreach $ip_instance (@all_IPs) { $active_IPs{"$ip_instance"}=1; }
	unless ($active_IPs{$addr})
	{
	    print STDERR "Vsite::vsite_add found IP address $addr not to be in use. Adding interface..\n" if ($Debug);
	    $mask ||= (net_get_ipaddr())[1];
	    # Insert subnet mask validation here..
	    ($ret)=net_set_vinterface($addr,$mask);
	    return vsite_abort($ret) unless ($ret =~ /^2/);
	    if ($Features{'proftpd'})
	    {
		print STDERR "Vsite::vsite_add configuring ftp server for new ip address $addr\n" if ($Debug);
		($ret)=Cobalt::Ftp::ftp_set_vinterface($addr);
		return vsite_abort($ret) unless ($ret =~ /^2/);
	    }
	}
    }

    # Add mod_rewrite friendly symlink into the siteManage ui
    unlink("$AdminPages_dir/siteManage/$name") if (-l "$AdminPages_dir/siteManage/$name");
    symlink("$AdminPages_dir/siteManage/$group",
	    "$AdminPages_dir/siteManage/$name");

    # Add filesystem friendly symlink into the data partition
    #   necessary with ambiguous site group names
    unlink("$Userdir/$name") if (-l "$Userdir/$name");
    symlink("$Htgroup_dir/$group","$Userdir/$name");

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("2660"); 
}

sub vsite_del
# Removes a name-based virtual web server
# Arguments: The site group name
# Return value: string of the form MSG_error("3311")
# Side effects: Modifies httpd.conf, may delete a virtual network interface
{
    my ($group) = @_;
    my($foundit,$found_web,$conf,$num_interface,$ret);
    my ($addr, $name);
    my @vsites = vsite_list();
    grep { ($addr, $name) = @$_ if ($group eq $$_[2]) } @vsites;
    $num_interface = scalar (grep { $addr eq $$_[0] } @vsites);
    return vsite_abort(MSG_error("vsite_cant_webnotfound", $name)) unless ($num_interface);

    # update the current configuration
    open(HTCNF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (LOCK, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    $found_web = 0;
    while(<HTCNF>) 
    {
	if (/^<VirtualHost $addr>$/ ... /^<\/VirtualHost>$/)
	{
	    $found_web = 1 if (/^DocumentRoot $Userdir\/$group\/$Webdir$/);
	    # slurp!
	    $conf .= $_;

	    if (/^<\/VirtualHost>$/)
	    {
		# don't spit if we got a match
		if ($found_web) {
		    $foundit=1;
		} else {
		    print LOCK $conf;
		}
		$conf = '';
		$found_web = 0;
	    }
	} elsif (/^NameVirtualHost $addr$/) {
	    print LOCK if ($num_interface > 1);
	} else {
	    # spit it..
	    print LOCK;
	}
    }
    close(HTCNF);
    close(LOCK);

    # remove the network interface if necessary
    unless($group eq $MainGroup) # Special case handled by Network.pm
    {
	if ($num_interface == 1)
	{
	    print STDERR "Vsite::vsite_del removing virtual network support for the ip address $addr\n" if ($Debug);
	    net_del_vinterface($addr);
	    if ($Features{'proftpd'})
	    {
		print STDERR "Vsite::vsite_del removing ftp server support for ip address $addr\n" if ($Debug);
		Cobalt::Ftp::ftp_del_vinterface($addr);
	    }
	}
    }

    # Remove mod_rewrite friendly symlink from the siteManage ui
    unlink("$AdminPages_dir/siteManage/$name")
	if (-l "$AdminPages_dir/siteManage/$name");

    # Remove filesystem friendly symlink from the data partition
    #   necessary with ambiguous site group names
    unlink("$Userdir/$name") if (-l "$Userdir/$name");

    vsite_set_sslconf($name, "off");
    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_webdeleted"); 
}

sub vsite_gen_name
# Generate a back-end name that is associated with a virtual site
# Arguments: none
# Return value: an indexed group name (site0, site1, ...)
# Example: $siteGroup = vsite_gen_name();
{
    my(@allGroups)=group_list();
    my(%activeGroups,$groupInst);
    foreach $groupInst (@allGroups) { $activeGroups{$groupInst}=1; }

    my($group,$nameIndex)=('site',1);
    while($activeGroups{$group.$nameIndex}) { $nameIndex++; }
    
    return($group.$nameIndex);
}

sub vsite_get_bygroup
# Get the information about a virtual interface by group name
# A wrapper to vsite_list, which is why $group is a parameter and a return value
# Arguments: group name
# Return value: array of (ip,hostname,group,webmaster email,subnet) or 0 
# Side effects: calls Cobalt::Vsite::vsite_list() to read the httpd.conf file
# Example: ($ip,$hostname,$group,$webmaster_email) = Cobalt::Vsite::vsite_get_bygroup("ratdogs");
{
    my ($group) = @_;
    my ($vhost);
    foreach $vhost (Cobalt::Vsite::vsite_list()) {
      if ($group eq $$vhost[2]) {
	# return $$vhost[1];
	return @$vhost;
      }
    }
    return 0;
}

sub vsite_list
# Finds all virtual web interfaces
# Arguments: none
# Return value: nested array of:
#    (ip,hostname,group,webmaster email)
#    sorted by fully qualified domain name (aka "hostname" for vsite_* functions)
# Example: @vhosts = Cobalt::Vsite::vsite_list();
#          ..OR.. 
#          foreach $vhost (@vhosts) { &use( @$vhost ); }
# Parses httpd.conf
{
    my(@vnames,$nic,$ip,$name,$adminemail,$group);
    open(HTCNF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);
    while(<HTCNF>) 
    {
	if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/)
	{
	    $ip = $1 if (/^<VirtualHost ([\d\.]+)>/);
	    $name = $1 if (/^ServerName ([a-z\d\.\-]+)/);
	    $adminemail = $1 if (/^ServerAdmin ([\w\@\-\.]+)/);
	    $group = $1 if (/^DocumentRoot $Userdir\/([^\/]+)/o);

	    if ($group && /^<\/VirtualHost>$/) {
		chomp($group);
		push( @vnames, ([$ip,$name,$group,$adminemail]) ); 
		$ip = $name = $adminemail = $group = '';
	    }
	}
    }
    close(HTCNF);

    # sort by hostname
    @vnames=sort { ${ $a }[ 1 ] cmp ${ $b }[ 1 ] } @vnames;

    return (@vnames);
}

sub vsite_admin_list
# Lists the site admins for the given site
# Arguments: site group name
#   With no arguments, lists *all* site administrators
# Return value: the list of admins for that site
# Side effects: none
# Example: (@admins) = Cobalt::Vsite::vsite_admin_list("eth1:12");
{
    my $group = shift;
    
    my @where;
    if ($group) {
	@where = ["vsite", "=", "$group", "and", "admin", "=", "on"];
    } else {
	@where = ["admin", "=", "on", "and", "name", "<>", "default"];
    }
    my @admins = Cobalt::Meta::query("type"  => "users",
				     "keys"  => ["name"],
				     "where" => @where);
    return @admins;
}

sub vsite_admin_set
# Set the list of site administrators for a virtual site
# Arguments: site group, list of users
# Side effects: sets the membership of the group for that site
#   to the given list, adds the site admins to the global
#   site admins group
# Return value: the usual status string of the form "3667 You are in
#   a dark place.  You are likely to be eaten by a grue."
{
    my ($group,@admins) = @_;
    my @current_admins = Cobalt::Group::group_show($group);
    Cobalt::Group::group_del_user($Sites_AdminGroup,@current_admins);

    Cobalt::Vsite::vsite_get_siteadmin_ftphelp(@current_admins);
    
    my $ret = Cobalt::Group::group_add_user($Sites_AdminGroup,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group

    $ret = Cobalt::Group::group_set_user($group,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group

    Cobalt::Vsite::vsite_set_siteadmin_ftphelp($group,@admins);
    return MSG_ok("vsite_admin_set_ok");
}

sub vsite_admin_add
# Add a site administrator to a virtual site
# Arguments: site group, list of users to add
# Side effects: adds the user(s) to the group for that site
#   and to the global site admins group
# Return value: the usual status string of the form MSG_error("4210")
{
    my ($group,@admins) = @_;

    my $ret;

    $ret = Cobalt::Group::group_add_user($Sites_AdminGroup,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group
    
    $ret = Cobalt::Group::group_add_user($group,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group
    
    # Add assorted $FtpAdminHelp files
    Cobalt::Vsite::vsite_set_siteadmin_ftphelp($group,@admins);

    return MSG_ok("vsite_admin_add_ok");
}

sub vsite_admin_del
# Remove a site administrator from a virtual site
# Arguments: site group, list of users to remove
#   from the admin list
# Side effects: removes the user(s) from the group for that site
#   and the global site admins group
# Return value: the usual status string of the form "2600 Game over, insert
#   coin"
# Example: Cobalt::Vsite::vsite_admin_del("eth10:1",@doomedusers);
{
    my ($group,@admins) = @_;
    my $ret = Cobalt::Group::group_del_user($Sites_AdminGroup,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group

    $ret = Cobalt::Group::group_del_user($group,@admins);
    return $ret if ($ret !~ /^2/o);  # abort if we can't fix the group

    # Delete assorted $FtpAdminHelp files
    Cobalt::Vsite::vsite_get_siteadmin_ftphelp(@admins);

    return MSG_ok("vsite_admin_del_ok");
}

sub vsite_add_alias
# Set alias names for a vhost
# Arguments: site group name, list of aliases
# Return value: string of the form MSG_error("3311")
# Note: does not remove any old server aliases
# Example: vsite_set_alias($group, $alias0, $alias1);
{
    my ($vsite, @aliases) = @_;
    my ($ret, $found);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    $found = 0;
    while (<HTCONF>) {
	if (!$found && /^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
	    print STDERR "vsite_add_alias found $vsite\n" if ($Debug);
	    print HTCTMP;  # don't forget to include the DocumentRoot
	    print HTCTMP "ServerAlias ", join(" ", @aliases), "\n";
	    $found = 1;
	}
	else {
	    # just spit it..
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    return vsite_abort(MSG_error("vsite_invalid_noentry", $vsite)) unless ($found);
    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_alias");
}

sub vsite_del_alias
# Delete alias names for a vhost
# Arguments: site group name, list of aliases
# Return value: string of the form MSG_error("3311")
# Note: does not remove any old server aliases
# Example: vsite_set_alias($group, $alias0, $alias1);
{
    my ($vsite, $alias) = @_;
    my ($ret, $found);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    $found = 0;
    while (<HTCONF>) {
	if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
	    print STDERR "vsite_add_alias found $vsite\n" if ($Debug);
	    print HTCTMP;  # don't forget to include the DocumentRoot
	    while (<HTCONF>) {
		unless (/^ServerAlias $alias/) {
		    print HTCTMP;  # don't forget to include the DocumentRoot
		}
		if (/<\/VirtualHost>/) {
		    break;
		}
	    }
	}
	else {
	    # just spit it..
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_alias");
}

sub vsite_set_php3
# Enable or disable the use of php3 for a virtual site
# This affects the entire site
# Arguments: site group name, 1 to enable/0 to disable
# Return value: status string of the form MSG_error("4400")
# Side effects: modifies the virtual host entry in httpd.conf
# Example: Cobalt::Vsite::vsite_set_ssi("home",1);
{
    my ($vsite,$_enabled) = @_;
    my ($inentry, $found_web,$conf);
    my ($on,$ret);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    my $php3_was = "";
    $found_web = $inentry = 0;
    # fix the httpd.conf file
    while (<HTCONF>) {
	if (!$inentry and
	    /^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
		$found_web = 1;
		print STDERR "vsite_set_php3 found $vsite\n" if ($Debug);
	    }

	    if (/^<\/VirtualHost>$/)
	    {
		if ($found_web) {
		    print HTCTMP $conf;
		    $inentry = 1;
		    if ($php3_enabled) {
			print HTCTMP "AddType application/x-httpd-php3 .php3\n";
			print HTCTMP "AddType application/x-httpd-php3-source .phps\n";
			$on = MSG_get("On_string");
		    }
		    else {
			$on = MSG_get("Off_string");
		    }
		} else {
		    print HTCTMP $conf;
		}
		print HTCTMP;
		$conf = '';
	    } elsif ($found_web) {

		if (/^AddType application\/x-httpd-php3 .php3/) {
		    # clear the existing AddHandler .shtml entries
		    print STDERR "vsite_set_php3 found AddType .php3\n"
			if ($Debug);
		    $php3_was = "on";
		} elsif (/^AddType application\/x-httpd-php3-source .phps/o) {
		    # clear the existing AddType .shtml entries
		    print STDERR "vsite_set_php3 found AddType .phps\n"
			if ($Debug);
		} else {
		    # within vhost section but not a php3 line! slurp!
		    $conf .= $_;
		}
	    } else {
		# within vhost section but not the right one! slurp!
		$conf .= $_;
	    }
	} else {
	    # outside vhost section! spit it..
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    return vsite_abort(MSG_error("vsite_invalid_noentry", $vsite)) unless ($inentry);
    return vsite_abort(MSG_ok("vsite_ok_php3_nochange"))
	if ($php3_was eq $php3_enabled);

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_php3");
}

sub vsite_set_ssi
# Enable or disable the use of SSIs for a virtual site
# This affects the entire site
# Arguments: site group name, 1 to enable/0 to disable
# Return value: status string of the form MSG_error("4400")
# Side effects: modifies the virtual host entry in httpd.conf
# Example: Cobalt::Vsite::vsite_set_ssi("home",1);
{
    my ($vsite,$ssi_enabled) = @_;
    my ($inentry, $found_web,$conf);
    my ($on,$ret);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    my $ssi_was = "";
    $found_web = $inentry = 0;
    # fix the httpd.conf file
    while (<HTCONF>) {
	if (!$inentry and
	    /^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
		$found_web = 1;
		print STDERR "vsite_set_ssi found $vsite\n" if ($Debug);
	    }

	    if (/^<\/VirtualHost>$/)
	    {
		if ($found_web) {
		    print HTCTMP $conf;
		    $inentry = 1;
		    if ($ssi_enabled) {
			print HTCTMP "AddHandler server-parsed .shtml\n";
			print HTCTMP "AddType    text/html     .shtml\n";
			$on = MSG_get("On_string");
		    }
		    else {
			# httpd.conf gets big enough without this bloat /wdh
			# print HTCTMP "#AddHandler server-parsed .shtml\n";
			# print HTCTMP "#AddType    text/html     .shtml\n";
			$on = MSG_get("Off_string");
		    }
		} else {
		    print HTCTMP $conf;
		}
		print HTCTMP;
		$conf = '';
	    } elsif ($found_web) {

		if (/^AddHandler server-parsed .shtml/) {
		    # clear the existing AddHandler .shtml entries
		    print STDERR "vsite_set_ssi found AddHandler .shtml\n"
			if ($Debug);
		    $ssi_was = "on";
		} elsif (/^\#AddHandler server-parsed .shtml/) {
		    # clear the existing AddHandler .shtml entries
		    print STDERR "vsite_set_ssi found AddHandler .shtml\n"
			if ($Debug);
		    $ssi_was = "";
		} elsif (/^(\#)?AddType\s+text\/html\s+.shtml/o) {
		    # clear the existing AddType .shtml entries
		    print STDERR "vsite_set_ssi found AddType .shtml\n"
			if ($Debug);
		} else {
		    # within vhost section but not a ssi line! slurp!
		    $conf .= $_;
		}
	    } else {
		# within vhost section but not the right one! slurp!
		$conf .= $_;
	    }
	} else {
	    # outside vhost section! spit it..
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    return vsite_abort(MSG_error("vsite_invalid_noentry", $vsite)) unless ($inentry);
    return vsite_abort(MSG_ok("vsite_ok_ssi_nochange"))
	if ($ssi_was eq $ssi_enabled);

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_ssi", $on);
}

sub vsite_get_php3
# Get the current PHP3 settings for a virtual site
# Arguments: the site group name
# Return value: 1 for on, 0 for off
# Side effects: none
# Example: Cobalt::Vsite::vsite_get_php3("home");
{
    my ($vsite) = @_;
    my ($found_web);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);
    
    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    while (<HTCONF>) {
	if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    $found_web = 1 if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/);
	    if ($found_web) {
		if (/^AddType application\/x-httpd-php3 .php3/) {
		    close HTCONF;
		    return 1;
		}

		last if (/^<\/VirtualHost>$/);
	    }
	}
    }
    close HTCONF;
    return 0;
}

sub vsite_get_ssi
# Get the current SSI settings for a virtual site
# Arguments: the site group name
# Return value: 1 for on, 0 for off
# Side effects: none
# Example: Cobalt::Vsite::vsite_get_ssi("home");
{
    my ($vsite) = @_;
    my ($found_web);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);
    
    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    while (<HTCONF>) {
	if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    $found_web = 1 if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/);
	    if ($found_web) {
		if (/^AddHandler server-parsed .shtml/) {
		    close HTCONF;
		    return 1;
		} elsif (/^\#AddHandler server-parsed .shtml/) {
		    close HTCONF;
		    return 0;
		}

		last if (/^<\/VirtualHost>$/);
	    }
	}
    }
    close HTCONF;
    return 0;
}

sub vsite_set_cgis
# Enable or disable the use of CGIs for a virtual site
# This affects the entire site
# Arguments: site group name, 1 to enable/0 to disable
# Return value: status string of the form "8750 I can't hear you, 
#   there's a bannana in my ear!"
# Side effects: modifies the virtual host entry in httpd.conf.
# Example: Cobalt::Vsite::vsite_set_cgis("eth0:0",1);
{
    my ($vsite,$cgis_enabled) = @_;
    my ($inentry, $found_web,$conf);
    my ($on,$ret);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    $found_web = $inentry = 0;
    # fix the httpd.conf file
    while (<HTCONF>) {
	if (!$inentry and
	    /^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
		$found_web = 1;
		print STDERR "vsite_set_cgis found $vsite\n" if ($Debug);
	    }

	    if (/^<\/VirtualHost>$/)
	    {
		if ($found_web) {
		    print HTCTMP $conf;
		    $inentry = 1;
		    # Put in the correct new AddHandler entries
		    if ($cgis_enabled) {
			print HTCTMP "AddHandler cgi-wrapper .cgi\n";
			print HTCTMP "AddHandler cgi-wrapper .pl\n";
			$on = MSG_get("On_string");
		    }
		    else {
			# httpd.conf gets big enough without this bloat /wdh
			# print HTCTMP "#AddHandler cgi-wrapper .cgi\n";
			# print HTCTMP "#AddHandler cgi-wrapper .pl\n";
			$on = MSG_get("Off_string");
		    }
		} else {
		    # end of vhost section but not the right one! spit it..
		    print HTCTMP $conf;
		}
		print HTCTMP;
		$conf = '';
	    } elsif ($found_web) {

		if (/^(\#)?AddHandler cgi-wrapper/) {
		    # clear the existing AddHandler entries
		    print STDERR "vsite_set_cgis found AddHandler\n"
			if ($Debug);
		} else {
		    # within vhost section but not a cgi line! slurp!
		    $conf .= $_;
		}
	    } else {
		# within vhost section but not the right one! slurp!
		$conf .= $_;
	    }
	} else {
	    # outside vhost section! spit it..
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    return vsite_abort(MSG_error("vsite_invalid_noentry", $vsite)) unless ($inentry);

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("2144", $on);
}

sub vsite_get_cgis
# Get the current CGI settings for a virtual site
# Arguments: the site group name
# Return value: 1 for on, 0 for off
# Side effects: none
# Example: Cobalt::Vsite::vsite_get_cgis("eth0:0");
{
    my ($vsite) = @_;
    my ($found_web);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);
    
    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    while (<HTCONF>) {
	if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    $found_web = 1 if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/);
	    if ($found_web) {
		if (/^AddHandler cgi-wrapper/) {
		    close HTCONF;
		    return 1;
		} elsif (/^\#AddHandler cgi-wrapper/) {
		    close HTCONF;
		    return 0;
		}

		last if (/^<\/VirtualHost>$/);
	    }
	}
    }
    close HTCONF;
    return 0;
}

sub vsite_getgroup
# Given a user name, figure out what site they are associated with
# Arguments: user name
# Return value: site group name
# Side effects: none
{
    my $user = shift;
    my $obj = Cobalt::Meta->create("type" => "users");
    $obj->retrieve($user);
    return ($obj->get("vsite"));
}

sub inform_BigBrother
# If $Dns_notification is readable and defined with an email address,
# send snitch-mail of any activity that requires an informed 3rd party
#
# Feature added by SimpleNet request 08/98 /wdh
#
# Arguments: subject string, text body string
{
    my($subject,$body) = @_;
    my($notifaddr);

    if (-r $Dns_notification)
    {
	open(NOTIF,$Dns_notification) || return 0;
	chomp($notifaddr = <NOTIF>);
	close(NOTIF);

	# recipient sanity check
	return 0 if ($notifaddr !~ /^\w+\@\w+/);

	open(MALR,"| $Sendmail -oi -t") || return 0;
	print MALR <<EOF;
To: $notifaddr
From: admin
Subject: $subject

$body

EOF
        close(MALR);
    }

    # Intended to run without $Dns_notification set as well
    return 1;
}

sub vsite_set_siteadmin_ftphelp
{
    my($group,@admins) = @_;

    # Add site admin ftp instructions
    if ($Features{'proftpd'})
    {
	if (-r $FtpAdminHelpSrc)
	{
	    my($ftphelpfile);
	    open(FTPH,"$FtpAdminHelpSrc") || return;
	    while(<FTPH>) { $ftphelpfile .= $_; }
	    close(FTPH);

	    my($siteadmin,$adminroot);
	    foreach $siteadmin (@admins)
	    {
		$adminroot = (getpwnam($siteadmin))[7];
		unless (-e "$adminroot/$FtpAdminHelp")
		{
		    open(FHLP,"> $adminroot/.ftphelp") || return;
		    print FHLP $ftphelpfile;
		    close(FHLP);
		    chown( (getpwnam($siteadmin))[2], (getgrnam($group))[2],
			   "$adminroot/$FtpAdminHelp");
		    chmod(0664,"$adminroot/$FtpAdminHelp");
		}
	    }
	}
    }
}

sub vsite_get_siteadmin_ftphelp
{
    my(@admins) = @_;
    
    # Remove site admin ftp instructions
    if ($Features{'proftpd'})
    {
	my($siteadmin,$adminroot);
	foreach $siteadmin (@admins)
	{
	    $adminroot = (getpwnam($siteadmin))[7];
	    unlink("$adminroot/$FtpAdminHelp")
		if (-e "$adminroot/$FtpAdminHelp");
	}
    }
}

sub vsite_set_fpx
# Enable or disable the use of FPX webs for a virtual site
# MS's fpadmsrv.exe can't be trusted to do anything right
#
# Arguments: site group name, true/false to enable/disable, 
#   optional array of user name (users with fpx webs)
# Return value: status string of the form MSG_error("MS slop incapable")  
# Side effects: modifies the virtual host entry in httpd.conf.
# fpadmsrv.exe is set to modify dummy file fpx.httpd.conf instead.
# Example: $ret=vsite_set_fpx($group,@userwebs);
{
    my ($vsite,$status,@userwebs) = @_;
    my ($inentry, $found_web,$conf);
    my ($on,$ret);

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);

    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    # Uniquify arguments
    my(%userwebs) = map { $_, 1 } @userwebs;
    @userwebs = keys ( %userwebs );

    # first, get the lock
    return $ret if (($ret = vsite_lock()) !~ /^2/);
    open (HTCTMP, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");

    $found_web = $inentry = 0;
    # fix the httpd.conf file
    while (<HTCONF>) {
	if (!$inentry and
	    /^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/) {
		$found_web = 1;
		print STDERR "vsite_set_fpx found $vsite\n" if ($Debug);
	    }

	    if (/^<\/VirtualHost>$/)
	    {
		if ($found_web) {
		    print HTCTMP $conf;
		    $inentry = 1;

		    # Put in the correct new ScriptAlias entries
		    if ($status) {
			print HTCTMP "ScriptAlias /_vti_bin/_vti_adm/ $Userdir/$vsite/$Webdir/_vti_bin/_vti_adm/\n";
			print HTCTMP "ScriptAlias /_vti_bin/_vti_aut/ $Userdir/$vsite/$Webdir/_vti_bin/_vti_aut/\n";
			print HTCTMP "ScriptAlias /_vti_bin/ $Userdir/$vsite/$Webdir/_vti_bin/\n";

			# User webs
			my($userInstance);
			foreach $userInstance (@userwebs)
			{
                            unless($userInstance eq $vsite)
 		  	    {
			        print HTCTMP "ScriptAlias /~$userInstance/_vti_bin/_vti_adm/ $Userdir/$vsite/users/$userInstance/$Webdir/_vti_bin/_vti_adm/\n";
			        print HTCTMP "ScriptAlias /~$userInstance/_vti_bin/_vti_aut/ $Userdir/$vsite/users/$userInstance/$Webdir/_vti_bin/_vti_aut/\n";
			        print HTCTMP "ScriptAlias /~$userInstance/_vti_bin/ $Userdir/$vsite/users/$userInstance/$Webdir/_vti_bin/\n";
			    }
			}
			$on = MSG_get("On_string");
		    }
		    else {
			$on = MSG_get("Off_string");
		    }
		    print HTCTMP <<EOF;
AliasMatch ^/~([^/]+)(/(.*))? $Userdir/$vsite/users/\$1/web/\$3
EOF
		} else {
		    # end of vhost section but not the right one! spit it..
		    print HTCTMP $conf;
		}
		print HTCTMP;
		$conf = '';
	    } elsif ($found_web) {

		if (/^ScriptAlias / && /\/_vti_bin/) {
		    # clear the existing ScriptAlias entries 
		    print STDERR "vsite_set_fpx found FPX ScriptAlias\n"
			if ($Debug);
		} elsif (/^AliasMatch \^\/~/) {
		    print STDERR "found aliasmatch and web\n" if ($Debug);
		} else {
		    # within vhost section but not an fpx line! slurp!
		    $conf .= $_;
		}
	    } else {
		# within vhost section but not the right one! slurp!
		$conf .= $_;
	    }
	} else {
	    # outside vhost section! spit it..
	    print HTCTMP;
	}
    }
    close HTCONF;
    close HTCTMP;
    return vsite_abort(MSG_error("vsite_invalid_noentry", $vsite)) unless ($inentry);

    return $ret if (($ret = vsite_commit()) !~ /^2/);
    return MSG_ok("vsite_ok_fpxset", $on); # Uses $on
}

sub vsite_get_fpx
# Get the current FPX settings for a virtual site
# Arguments: site group name
# Return value: array of user webs if enabled
#	        null if fpx disabled
# Side effects: parses httpd.conf
# Example: Cobalt::Vsite::vsite_get_fpx($MainGroup);
{
    my ($vsite) = @_;
    my ($found_web);
    my(@ret,%beenthere); # init return value, singularity check

    # special case for the primary network interface
    $vsite=$MainGroup if ($vsite eq $Netif);
    
    # update the current configuration
    open(HTCONF,"$Htconf/httpd.conf") || 
	return MSG_error("vsite_cant_read_httpd", $Htconf);

    while (<HTCONF>) {
	if (/^<VirtualHost [\d\.]+>$/ ... /^<\/VirtualHost>$/) {
	    
	    $found_web = 1 if (/^DocumentRoot $Userdir\/$vsite\/$Webdir$/);
	    if ($found_web) {
		if (/^ScriptAlias\s+\/([^\/]+)\// && /_vti_bin/) {
		    my $user;

		    $user = $1 if (/^ScriptAlias\s+\/([^\/]+)\//); # user web
		    if ($user =~ /^\~/)
		    {
			$user =~ s/^\~//; # Strip ~ from ~user
			push(@ret,$user) unless ($beenthere{$user}); # user web
			$beenthere{$user}=1;
		    } else
		    {
			# $user eq _vti_bin, check algorithm
			push(@ret,$vsite) unless ($beenthere{$vsite});
			$beenthere{$vsite}=1;
		    }
		}

		last if (/^<\/VirtualHost>$/);
	    }
	}
    }
    close HTCONF;
    return @ret;
}

sub vsite_sync
# Updates virtualhost configuration to match non-virtual network configuration
# Critical for Netif-bound web url rewrites, sharing the Netif ip address
# Arguments: none
# Return value: true for success, false for failure
{
    # Obtain stale configuration to remove symlinks, etc..
    my($origip,$orighost,$origgroup,$webmaster) = vsite_get_bygroup($MainGroup);

    # Fetch old site configuration, to be re-added
    my($fpx_state)=fpx_get_web($MainGroup);
    my(@fpx_webs)=vsite_get_fpx($MainGroup) if ($fpx_state);
    my($site_ssi)=vsite_get_ssi($MainGroup);
    my($site_cgi)=vsite_get_cgis($MainGroup);
    my($ftpAnon,$ftpMaxUsers,$ftpQuota)=Cobalt::Ftp::ftp_get_anonymous($MainGroup);

    vsite_del($MainGroup) if ($origip); # Return value insignificant

    # Symlink management to facilitate /siteadmin/ redirects
    unlink("$AdminPages_dir/siteManage/$orighost")
	if (-l "$AdminPages_dir/siteManage/$orighost");
    unlink("$AdminPages_dir/siteManage/$origip")
	if (-l "$AdminPages_dir/siteManage/$origip");
    
    # Determine our real net settings
    my $hostname = Cobalt::Network::net_get_hostname();

    # The first time the system is booted it has no hostname; the name
    # defaults to "localhost".  If the main web server is configured as
    # a virtual host, and you try to access the system by name
    # before the system knows its own name, the web server returns a
    # "unknown host" error.  So, in the special case that we don't have
    # a real hostname, we don't add the virtual host entry.
    return 1 if ($hostname eq "localhost");

    my $domainname = (Cobalt::Network::net_get_dnsclient())[0];
    my $fqdn = "$hostname.$domainname";

    my($ip,$netmask,$mac)=Cobalt::Network::net_get_ipaddr();

    # Migrate virtual sites
    if ($ip ne $origip)
    {
      # Delete the superflous virtual network interface that results from
      # changing a real network interface's IP address to one that was in
      # use as a virtual network interface.
      my(@all_sites)=vsite_list();
      my($site_instance);
      foreach $site_instance (@all_sites)
      {
        net_del_vinterface( $ip ) if ( @$site_instance[0] eq $ip );
        # We ignore the return value as $ip could already be in use by a
        # real network interface
      }
      vsite_migrate_ipaddr($origip,$ip); 
    }

    # Re-enable the Netif's vsite
    my($return)=vsite_add($ip,$fqdn,$MainGroup,$webmaster,$netmask);
    return 0 if ($return !~ /^2/);

    # Restore site services (httpd configuration only)
    # Do not return on a failed service configuration, move on to the next
    vsite_set_fpx($MainGroup,1,@fpx_webs) if ($fpx_state);
    vsite_set_ssi($MainGroup,1) if ($site_ssi);
    vsite_set_cgis($MainGroup,1) if ($site_cgi);
    my($return)=Cobalt::Ftp::ftp_config($ip,1,$ftpAnon,$MainGroup,$ftpMaxUsers,$ftpQuota);

    # Recreate symlinks
    symlink("$AdminPages_dir/siteManage/$MainGroup",
	    "$AdminPages_dir/siteManage/$fqdn");
    symlink("$AdminPages_dir/siteManage/$MainGroup",
	    "$AdminPages_dir/siteManage/$ip");

    Cobalt::Http::http_restart();

    return 0 if ($return !~ /^2/);
    return 1;
}

sub vsite_migrate_ipaddr
# Moves virtual sites from an old IP address to a new one en mass
# Useful when changing the IP of a real network interface
#
# Does not restart daemons
#
# Arguments: Old IP address, New IP address
# Return value: 1 for success, 0 for failure
{
    my($oldip,$ip)=@_;

    sleep 1 if (-e "$Lockdir/httpd.conf"); # Avoid race
    return 0 if (vsite_lock() !~ /^2/);
    # update the current configuration
    open(HTCNF,"$Htconf/httpd.conf") || return 0;

    open (HTLOCK, ">$Lockdir/httpd.conf") ||
	return MSG_error("vsite_wicked");
    while(<HTCNF>)
    {
	# subsitute the ip in apache <.> type directives
	s/(\s+)$oldip(>*)$/$1$ip$2/;
	# substitute the ip in the rewritecond..
	s/^(RewriteCond\s*!\^)$oldip\(:80\)\?\$$/$1$ip\$/;
	print HTLOCK;
    }
    close(HTLOCK);
    close(HTCNF);
    return 0 if (vsite_commit() !~ /^2/);
    return 1;
}

sub vsite_set_logrotate
# Defines logrotate entries in /etc/logrotate.d and /etc/logrotate.onboot
# for individual site log access fragments (/home/sites/<group>/logs/access
# Arguments: Site group, Rotate size in megabytes
#            If log rotate size is set to zero, disable logrotations
# Return Value: Return code and descriptive string
# Example: $return=Cobalt::Vsite::vsite_set_logrotate($MainGroup,5)
{
    my($group,$size)=@_;

    unless($size) # Disable if asked
    {
	unlink("$Logrotate_dir/$group");
	unlink("$Logrotate_boot/$group");
	return MSG_ok("vsite_ok_nologrotate");
    }

    # Minor sanity check if going to enable rotates
    return MSG_error("vsite_cant_nogroup") unless (-e "$Htgroup_dir/$group/$Logdir");

    $size.='M';
    my($rotate)=<<EOF;
$Htgroup_dir/$group/$Logdir/mail.log {
   missingok
   compress
   size $size
}

$Htgroup_dir/$group/$Logdir/ftp.log {
   missingok
   compress
   size $size
}

$Htgroup_dir/$group/$Logdir/web.log {
   missingok
   compress
   size $size
}
EOF

    open(DOTD,"> $Logrotate_dir/$group") ||
	return MSG_error("vsite_cant_writerotate");
    print DOTD $rotate;
    close(DOTD);

    open(BOOT,"> $Logrotate_boot/$group") ||
	return MSG_error("vsite_cant_writerotate");
    print BOOT $rotate;
    close(BOOT);

    return MSG_ok("vsite_ok_logrotate");
}

sub vsite_lock
# lock the httpd.conf file
# Arguments: none
# Return value: a nicely stated message string
# Side effects: trix for kids!
{
    # verify and grab our lock
    return(MSG_error("vsite_cant_locked"))
	if (-e "$Lockdir/httpd.conf");

    # Lock the file by creating a temporary file in lockdir
    open(LOCK,"> $Lockdir/httpd.conf") || 
	return(MSG_error("4593"));
    close LOCK;

    return MSG_ok("2421", $Htconf);
}

sub vsite_commit
# commit to the httpd.conf file
# Arguments: none
# Return value: a nicely stated message string
# Side effects: oops! all berries!
{
    rename("$Lockdir/httpd.conf", "$Htconf/httpd.conf") ||
	return(MSG_error("4585"));
    return MSG_ok("2422", $Htconf);
}

sub vsite_abort
# leave in a graceful manner
# Arguments: anything under the sun!
# Return value: garbage in, garbage out.
# Side effects: unlocks httpd.conf
{
    unlink("$Lockdir/httpd.conf") if (-e "$Lockdir/httpd.conf");
    return join('',@_);
}

sub vsite_disable
# disable a virtual site
#   1. disable all virtual site users
#   2. chmod 0000 site home directory
# arguments: vsite groupname
# return value: std
{
    my $group = shift;
    my @users = vsite_get_users($group);
    foreach (@users) {
	Cobalt::User::user_disable($_);
    }
    
    my $mode = oct '0000';
    my $dir = $Userdir.'/'.$group;
    
    chmod $mode, $dir;
    
    admserv_set_access("$AdminPages_dir/siteManage/$group", 'admin');
    
    return MSG_ok("vsite_disable_ok");
}

sub vsite_enable
# enable a virtual site
#   1. re-enable all virtual site users
#   2. chmod site home directory back to normal
# arguments: vsite groupname
# return value: std
{
    my $group = shift;
    my @users = vsite_get_users($group);
    foreach (@users) {
	Cobalt::User::user_enable($_);
    }

    my $mode = oct '2775';
    my $dir = $Userdir.'/'.$group;

    chmod $mode, $dir;

    admserv_set_access("$AdminPages_dir/siteManage/$group", 'require', 'group', $group);
    
    return MSG_ok("vsite_enable_ok");
}

sub vsite_issuspend
# return true/false if a site is suspended
{
    my $group = shift;
    my $dir = $Userdir.'/'.$group;

    return -1 if (-l $dir and not -e readlink($dir));      

    my $m = (stat($dir))[2];
    my $mode = $m & 07777;
    return 0 if ($mode ne oct("0000"));
    return 1;
}

sub vsite_get_users
# given a vsite group query the database
# and return an array of all usernames
{
    my $group = shift;
    my (@users);
    return (Cobalt::Meta::query("type"  => "users",
				"keys"  => ["name"],
				"where" => ["vsite", "=", "$group"]));
}

sub vsite_ssl_get_cert_info
# Arguments: A group name.
# Returns: Refrence to a hash of subject information.
#		   Refrence to a hash of issuer information.
#          Array of ( year, monty, day ) for expiry date.
# e.g. ( $issuer, $subject @expiry) = ssl_get_cert_info
{
	my $group = shift;
	return ssl_get_cert_info("$Htgroup_dir/$group/$Certdir");
}

sub vsite_ssl_set_identity
# Arguments: A group name.
#			 Number of days the certificate is valid for.
#            A country.
#            State or province name.
#			 Locality Name (City.)
#			 Organisation Name.
#            Organisation Unit.
#			 Fully Qualified Domain Name.
#			 Email address of maintainer.
# Returns: The usual. But with mustard this time.
{
	my $group = shift;
	my $days = shift;
	my @cert_info = @_;

	return ssl_set_identity($days, @cert_info, "$Htgroup_dir/$group/$Certdir");
}

sub vsite_ssl_get_cert
# Arguments: A site group name.
# Returns: The base64 encoded site certificate request as a single string;
{
	my $group = shift;
	return ssl_get_cert("$Htgroup_dir/$group/$Certdir");
}

sub vsite_ssl_set_cert
# Arguments: A site name.
# 		     A string that contains the full certificate in PEM format.
{
	my $group = shift;
	my $cert = shift;
	return ssl_set_cert($cert,"$Htgroup_dir/$group/$Certdir");
}

sub vsite_get_ssl
# Arguments: A site group name.
# Returns: 1 for on, 0 for off..
{
	my $group = shift;
	my $o = Cobalt::Meta->new("type" => "vsite");
	$o->retrieve($group);
	$group = $o->get("ssl");
	$group = ($group eq 'on') ? 1 : 0;
	return $group;
}

sub vsite_set_sslconf
# Arguments: an object
# sets the values
{
    my ($group, $ssl) = @_;

    if ($ssl eq 'on') {
	open(FILE, ">$Htssldir/$group");
	close(FILE);
    } else {
	unlink("$Htssldir/$group") if (-e "$Htssldir/$group");
    }
}

sub vsite_check_cert
# Arguments: A group name.
# Returns: The usual, bartender.
{
	my $group = shift;
	return ssl_cert_check("$Htgroup_dir/$group/$Certdir");
}

sub vsite_ssl_clear
# Arguments: A group name
# Returns: The usual, nah, make it a double.
{
	my $group = shift;
	return ssl_clear("$Htgroup_dir/$group/$Certdir");
}

sub site_del
# Description: delete everything associated with a site.
# Input: group name
# Output: error string.
{
    my $group = shift;
    my $groupName ||= $group;
    my ($info, $err, $ret);

    my @users=vsite_get_users($groupName);
    if ($users[0] =~ /^[134]\S*\s/) {
	$info.=substr( $users[0], 4 ).'<BR>';
	$err=1;
    } else {
	if($groupName eq $MainGroup) {
	    $info.='The primary site cannot be deleted.';
	    $err=1;
	}
    }

    # Perform basic locking
    my $lock_denied=0;
    if (-e $Sites_DelLock) {
	sleep 1; # Avoid race
	if (-e $Sites_DelLock) {
	    $info .= MSG_get_ui("vsite_cant_deleting_other_vsite");
	    $err=1;
	    $lock_denied=1;
	}
    }
    unless ( $lock_denied ) {
	# touch
	open( LCK, ">$Sites_DelLock") || die "Could not open lockfile $lock_denied\n"; 
	print LCK $$; 
	close( LCK ); 
    }

    return $info if $err; # don't bail on anything else.

    my $obj = Cobalt::Meta->create("type" => "vsite");
    $obj->retrieve($group);
    my $hostName = $obj->get('hostname') . '.' . $obj->get('domain');
    my $ipAddr = $obj->get('ipaddr');

    # delete bw management if we're the only site using this
    # ip address.
    my @sharedip = Cobalt::Meta::query("type"  => "vsite",
				       "keys"  => ["name"],
				       "where" => ["name", "<>", "default",
						   "AND", "name", "<>", "$group",
						   "AND", "ipaddr", "=", "$ipAddr"]);
    if (!scalar(@sharedip)) {
	$ret = bw_delip($ipAddr);
	unless ($ret =~ /^2/) {
	    $info.=substr($ret,4).'<BR>';
	}
    }

    # delete all mailing lists belonging to this site
    my @lists;
    if (@lists = list_lists($groupName, $hostName)) {
	if ($lists[0] =~ /^[134]\S*\s/) {
	    $info.=substr( $lists[0], 4 ).'<BR>';
	} 
	$ret = list_delete($groupName, $hostName, @lists);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	}
    }

    # delete the virtual list server
    $ret=list_delete_virtual ($groupName, $hostName);
    if( $ret!~/^2/ ) {
	$info.=substr( $ret, 4 ).'<BR>';
    }

    # delete all site users
    my $user;
    foreach $user (@users) {
	if($user ne $AdminUser) {
	    $ret=user_del($user);
	    if($ret!~/^2/) {
		$info.=substr($ret, 4).'<BR>';
	    }
	    my $obj = Cobalt::Meta->new("type" => "users");
	    $obj->retrieve($user);
	    $obj->delete;
	}
    }

    # disable anonymous FTP iff it was enabled
    if(Cobalt::Ftp::ftp_get_anonymous($groupName)) {
	$ret=Cobalt::Ftp::ftp_config($ipAddr,1,0,$groupName);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	}
    }

    # delete virtual host
    $ret=vsite_del( $groupName );
    if( $ret!~/^2/ ) {
	$info.=substr( $ret, 4 ).'<BR>';
    }

    # disable logfile rotation
    $ret=vsite_set_logrotate($groupName,0);
    if( $ret!~/^2/ ) {
	$info.=substr( $ret, 4 ).'<BR>';
    }

    # restart the public server after deleting the vsite
    http_restart();

    # find out if an email route exists (therefore if accept-at-domain was on)
    my($r,$olddomain);
    my(%routes)=Cobalt::Email::mail_virtuser_list_route();
    foreach $r (%routes) {
	if ($routes{$r} eq $hostName) {
	    $olddomain = $r;
	    last;
	}
    }

    # remove entry from sendmail.cw, email route from virtusertable
    if ( $olddomain )
    {
	$ret=Cobalt::Email::mail_virtuser_del_route($olddomain, $hostName);
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	}
	$ret=Cobalt::Email::mail_del_relay( $olddomain );
	if( $ret!~/^2/ ) {
	    $info.=substr( $ret, 4 ).'<BR>';
	}
	$ret=Cobalt::Email::mail_del_cw( $hostName, $olddomain );
    } else {
	$ret=Cobalt::Email::mail_del_cw( $hostName );
    }
    if( $ret!~/^2/ ) {
	$info.=substr( $ret, 4 ).'<BR>';
    }

    # remove entry from relay list
    $ret=Cobalt::Email::mail_del_relay( $ipAddr );
    if( $ret!~/^2/ ) {
	$info.=substr( $ret, 4 ).'<BR>';
    }

    Cobalt::Email::mail_sendmail_hup();

    # remove cron job if any
    $ret=stats_set_www_report( 'never', $groupName );
    if( $ret!~/^2/ ) {
	$info.=substr( $ret, 4 ).'<BR>';
    }

    if( $groupName) {
	# delete site management site
	system( "/bin/rm -fr $AdminPages_dir/siteManage/$groupName" );

	# delete data for applet
	my $appletData="$AdminPages_dir/appletData/webUsage-$groupName.dat";
	if( -f $appletData ) { unlink $appletData; }
    }

    # delete group
    $ret=group_del( "$groupName" );
    if( $ret!~/^2/ ) {
	$info.=substr( $ret, 4 ).'<BR>';
    }

    $obj->delete;
    unlink $Sites_DelLock unless ( $lock_denied ); 
    
    return $info;
}

sub vsite_get_xml
# Arguments: A Vsite name
# Returns: The site as an XML object.
#
# Only used inside this module for the purpose of dumping Vsite objs.
{
	my $group = shift;
	my $obj = Cobalt::Meta->create("type" => "vsite");
	$obj->retrieve($group);
	return $obj->as_xml();
}

sub vsite_obj_add
# Arguments: A vsite meta object.
# Returns: 1 for success, 0 for failure, 2 for success but with a warning.
{
    my $obj = shift;
	my $err;
	my @info;
    my $ret;

	# Some parameters must be set manually as they are either specific to
	# a singel RaQ and thus not relocateable or are not useful to set
	# manually. At the moment this is just the group name.

	if ( $$obj{'type'} != "vsite" ) {
		return (0, MSG_error("vsite_cant_not_a_vsite_obj"));
	}

	my $groupName=$obj->getname();
	my $hostName=$obj->get('hostname');
    my $domain=$obj->get('domain');
    my $ipAddr=$obj->get('ipaddr');
    my $diskQuota=$obj->get('diskquota');
    my $cgi=$obj->get('cgi');
    my $shell=$obj->get('shell');
    my $ssi=$obj->get('ssi');
    my $ssl=$obj->get('ssl');
    my $fpx=$obj->get('fpx');
    my $bw_setip=$obj->get('bw_setip');
    my $bwlimit=$obj->get('bwlimit');
    my $ftp=$obj->get('ftp');
    my $ftpQuota=$obj->get('ftpquota');
    my $ftpUsers=$obj->get('ftpusers');
    my $maxUsers=$obj->get('maxusers');
    my $emailDomain=$obj->get('emaildomain');
    my $webDomain=$obj->get('webdomain');
    my $apop=$obj->get('apop');
    my $suspend=$obj->get('suspend');

	$hostName = $hostName.'.'.$domain;

    $ret=group_add( $groupName, $diskQuota, $mountpoint );
    if( $ret!~/^2/ ) {
		push @info, substr($ret, 4);
		return (0, @info);
    }

	$obj->put("gid", (getgrnam("home"))[2]);

	    # substitute variables in index.html
    my $original="$Htgroup_dir/$groupName/$Webdir/index.html";
    my $tmpFile="$Lockdir/groupIndex-$$.html";

	open(IN,"$original") || die "Could not read file $original: $!";
    open(OUT,">$tmpFile") || die "Could not write to /tmp: $!";

	while( <IN> ) {
		s/\[DOMAIN\]/$hostName/go;
		print OUT;
	}

	close( IN );
    close( OUT );

	rename($tmpFile, $original);

	# Restore index permissions
	chown($WWWuid,(getgrnam($groupName))[2],"$Htgroup_dir/$groupName/$Webdir/index.html");
	chmod(0664,"$Htgroup_dir/$groupName/$Webdir/index.html"); 

	# Setup a virtual list server.
	$ret = list_create_virtual( $groupName, $hostName);
	if ( $ret !~ /^2/ ) {
		$err=1;
		push @info, substr($ret, 4);
	}

        Cobalt::Layout::layout_siteManage($groupName, $hostName);

	# access control
	$ret=admserv_set_access( "$AdminPages_dir/siteManage/$groupName",
							 'require', 'group', $groupName );
	if( $ret!~/^2/ ) {
		$err=1;
		push @info, substr( $ret, 4 );
	}

	$ret = vsite_add($ipAddr, $hostName, $groupName );
	if( $ret!~/^2/ ) {
		push @info, substr( $ret, 4 );
		group_del( $groupName );
		return(0, @info);
	}

	$ret = vsite_set_logrotate($groupName,$logRotateLimit);
	if ( $ret !~ /^2/ ) {
		$err = 1;
		push @info, substr($err, 4);
	}

	if ($webDomain eq 'on') {
		$ret = vsite_add_alias($groupName, $domain);
		if ( $ret !~ /^2/ ) {
			$err = 1;
			push @info, substr($err, 4);
		}
	}


	# If email is on for this domain..
	if ($emailDomain eq 'on') {
		$ret=Cobalt::Email::mail_virtuser_add_route($domain, $hostName);
		if( $ret!~/^2/ ) {
			 $err = 1; push @info, substr($err, 4);
		}
		$ret=Cobalt::Email::mail_add_relay( $domain );
		if( $ret!~/^2/ ) {
			 $err = 1; push @info, substr($err, 4);
		}
		$ret=Cobalt::Email::mail_add_cw( $hostName, $domain );
		if( $ret!~/^2/ ) {
			 $err = 1; push @info, substr($err, 4);
		}
	} else {
		$ret=Cobalt::Email::mail_add_cw($hostName);
		if( $ret!~/^2/ ) {
			 $err = 1; push @info, substr($err, 4);
		}
	}

	# add IP address to relay list
	$ret=Cobalt::Email::mail_add_relay( $ipAddr );
	if( $ret!~/^2/ ) {
		 $err = 1; push @info, substr($err, 4);
	}

	Cobalt::Email::mail_sendmail_hup();

	# config ftp
	if ($ftp eq 'on') {
		$ret=Cobalt::Ftp::ftp_config($ipAddr,1,1,$groupName,$ftpUsers,$ftpQuota);
	} else {
		$ret=Cobalt::Ftp::ftp_config($ipAddr,1,0,$groupName);
	}
	if( $ret!~/^2/ ) {
		 $err = 1; push @info, substr($err, 4);
	}

	# config cgi
	if ( $cgi eq 'on' ) {
		$ret=vsite_set_cgis( $groupName, 1 );
	} else {
		$ret=vsite_set_cgis( $groupName, 0 );
	}
	if( $ret!~/^2/ ) {
		 $err = 1; push @info, substr($err, 4);
	}

	# config ssi
	if ( $ssi eq 'on' ) {
		$ret=vsite_set_ssi( $groupName, 1 );
	} else {
		$ret=vsite_set_ssi( $groupName, 0 );
	}
	if ( $ret!~/^2/ ) {
		 $err = 1; push @info, substr($err, 4);
	}

	# config fpx
	if($fpx eq 'on') {
		$ret=fpx_set_web( $groupName, 1 );
	} else {
		$ret=fpx_set_web( $groupName, 0 );
	}
	if( $ret!~/^2/ ) {
		 $err = 1; push @info, substr($err, 4);
	}

	# For this section we need to know th IPs of any vsites which share our IP/
	my @other_ipaddrs = Cobalt::Meta::query("type" => "vsite",
                                                "keys" => ["name"],
                                                "where" => ["name", "<>", "default",
                                                            "AND", "ipaddr", "=", "$ipAddr"]);

	# config bandwidth limits
	my ($bwchobj,$bwwarn);
	my $oldlimit = bw_getip($ipAddr);
	if ($bwlimit eq "on") {
		foreach (@other_ipaddrs) {
			$bwchobj = Cobalt::Meta->create("type" => "vsite");
			$bwchobj->retrieve($_);
			if ($bwchobj->get("bwlimit") eq "off") {
				$bwchobj->put("bwlimit" => "on");
				$bwchobj->save;
				$bwwarn=1;
			}
		}
		if ($oldlimit ne $bw_setip) {
			$ret = bw_setip("ip" => $ipAddr, "bwlimit" => $bw_setip, "protocol" => "tcp");
			if ($ret!~/^2/) {
				 $err = 1;
				 push @info, substr($err, 4);
			}
			if (scalar(@other_ipaddrs)) {
				$bwwarn=1;
			}
		}
	} else {
		foreach (@other_ipaddrs) {
			$bwchobj = Cobalt::Meta->create("type" => "vsite");
			$bwchobj->retrieve($_);
			if ($bwchobj->get("bwlimit") eq "on") {
				$bwchobj->put("bwlimit" => "off");
				$bwchobj->save;
				$bwwarn=1;
			}
		}
		if ($oldlimit) {
			$ret = bw_delip($ipAddr);
			if ($ret!~/^2/) {
				$info.=substr ($ret, 4).'<BR>';
				$err=1;
			}
		}
	}
	# If there was an error we return 2
	$err ? ($err = 2) : ($err = 1);
	return ($err, @info);
}

1;
