#!/usr/bin/perl

# usage: vacation.pl [message] [from-address]

# Copyright 1999-2000 Cobalt Networks (http://www.cobaltnet.com)

use Cobalt::Product;
use Cobalt::Locale;
use strict;

use DB_File;
use Fcntl qw(O_RDWR O_CREAT);
use Locale::gettext;

my ($message_file,$user_from) = @ARGV;
my ($tmp_message_file) = "$Big_tmp/.vacation_tempmsg.$$";
my ($sendto,$sender,$returnpath,$from,$replyto,$precedence);

# get locale
my $localeStr=`cat /etc/cobalt/locale`;
chomp $localeStr;
if (defined($localeStr)) {
        $ENV{'LANGUAGE'} = $localeStr;
} else {
        $ENV{'LANGUAGE'} = "en_US";
}

# set locale
my $locale = Cobalt::Locale->getByString($localeStr);
if($locale) {$ENV{'LANGUAGE'} = $locale->getLocale();}

my $emailInJIS = $locale->getProperty('emailInJIS');

while (<STDIN>)
{
    if    (/From:\s*(.+)/)        { $from = &extract_address($1);       }
    elsif (/Reply-To:\s*(.+)/)    { $replyto = &extract_address($1);    }
    elsif (/Sender:\s*(.+)/)      { $sender = &extract_address($1);     }
    elsif (/Return-path:\s*(.+)/) { $returnpath = &extract_address($1); }
    elsif (/Precedence:\s*(.+)/)  { $precedence = $1;                   }
}
exit if ($precedence =~ /bulk|junk/oi);

if    ($replyto) { $sendto = $replyto;     }
elsif ($from)    { $sendto = $from;        }
elsif ($sender)  { $sendto = $sender;      }
elsif ($sender)  { $sendto = $returnpath;  }
else             { exit;                   }

my %vacadb;
tie(%vacadb,'DB_File',"$Vaca_dir/$user_from.db",O_RDWR|O_CREAT,0666)
    || die "Cannot open vacation database: $!\n";
if ($vacadb{$sendto} >= ($^T - 604800))
{
    # They've been given a reply recently
    untie %vacadb;
    exit;
}
else
{
    $vacadb{$sendto} = $^T;
    untie %vacadb;
}

my $gcos = (getpwnam($user_from))[6];
$gcos =~ tr/<>//d;
my $as_from = "$gcos <$user_from>";
my $subject = Locale::gettext::dgettext("special-sauce", "Vacation_Subject");
$subject = eval "\"$subject\"";  # need to eval to expand $as_from
my $tmpHeader="$Big_tmp/.header.$$";
open (OUTHEADER, ">$tmpHeader") || die "Cannot open $tmpHeader $!\n";
print OUTHEADER "From: $user_from\n";
print OUTHEADER "To: $sendto\n";
print OUTHEADER "Precedence: junk\n";
print OUTHEADER "Subject: $subject\n";
print OUTHEADER "Date: ", scalar(localtime($^T)), "\n";
if ($emailInJIS) {
    print OUTHEADER "Content-Type: text/plain; charset=\"iso-2022-jp\"\n";
    print OUTHEADER "Content-Transfer-Encoding: 7bit\n";
}
close (OUTHEADER);

if ($emailInJIS) {
    # convert header from Shift-JIS to JIS for sendmail
    my $tmpHeaderJ="$tmpHeader.sjis";
    system("$ShiftJISToJIS_prog $tmpHeader >$tmpHeaderJ");
    system("mv $tmpHeaderJ $tmpHeader");

    # convert message file from SJIS to JIS
    system("$ShiftJISToJIS_prog $message_file >$tmp_message_file");
    $message_file=$tmp_message_file;
}

open (OUT, "|$Sendmail -oi -t") || die "Can't open sendmail $!\n";
open (INHEADER, "<$tmpHeader") || die "Cannot open $tmpHeader to read $!\n";
print OUT <INHEADER>;
print OUT "\n";
if (open (INMESSAGE, "$message_file")) {
    print OUT <INMESSAGE>;
    close INMESSAGE;
} else {
    print OUT Locale::gettext::dgettext("special-sauce", "defaultVacationReply"); 
}
close OUT;
close INHEADER;

# clean up
unlink $tmpHeader;
unlink $tmp_message_file if ($emailInJIS);

# boy, this is ugly
sub extract_address
{
    my ($from) = @_;
    my ($junk,$tmp1,$tmp2);
    if ($from =~ /.*<.*>/)              # From: Foo Bar <foobar@com.edu>
    {
        ($junk,$tmp1) = split('<',$from);
        ($tmp2,$junk) = split('>',$tmp1);
        return $tmp2;
    }
    elsif ($from =~ /.*(.*)/)   # From: foobar@com.edu (Foo Bar)
    {
        ($tmp1,$junk) = split('\(',$from);
        return $tmp1;
    }
    else                                # From: foobar@com.edu [we hope]
    {
        return $from;
    }
}
